/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrWrapLib
// @module  Borrowed Items | This module contains the implementation of the
//          CECOrbBorrowThread class.  This class continually borrows objects
//          from the Factory and prevents them being reaped.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbBorrowThread.h>

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructors main responsiblity is to start the borrowing
//          thread running.
CECOrbBorrowThread::CECOrbBorrowThread(CECOrbServer* server):m_server(server)
{
    try
    {
        // Create new borrowing thread
        m_running = true;
        //      m_borrow = rwMakeThreadFunctionM( CECOrbBorrowThread, *this,
        //                                          void, &CECOrbBorrowThread::Borrower );
        //      m_borrow.start();
    }

    catch( ... )
    {
        throw CyrRepositoryException( "Unable to start borrowing thread",
                                      "CECOrbBorrowThread::CECOrbBorrowThread",
                                      CYR_UNKNOWN_ERROR );
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor must shut down the borrowing thread and wait for it
//          to terminate.
CECOrbBorrowThread::~CECOrbBorrowThread()
{
    // Inform borrow thread that borrower is being destroyed
    // and then wait for thread to terminate.
    m_running = false;
    //  m_borrow.requestCancellation();
    //  m_borrow.join();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method will preserve the borrowed item until it is disposed of
//          by the Dispose method.  The method just adds the entry to the array
//          of entries and will have its lease extended form that moment on.
//          Each entry has a guaranteed unique name and this will be used to
//          identify each entry.
// @parm    The item to preserve.

void CECOrbBorrowThread::Preserve( CECOrbBorrow* item )
{
    // Determine unique name for this item.  If the item does not have
    // a unique name then don't try to preserve it.
    string name = item->GetRegisteredName();
    if ( name.empty() )
        return;

    // If the name is not already registered then add it to list
    if ( m_array.find( name )!=m_array.end())
        m_array.insert( BorrowArray::value_type(name, item ));
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   An item will be maintained until it is removed by calling this
//          method.  The same unique name used during registration will be used
//          to dergister the item.
// @parm    The entry to dispose of.
void CECOrbBorrowThread::Dispose( CECOrbBorrow* item )
{
    // Determine unique name for this item
    string name = item->GetRegisteredName();

    // If the name is in array then remove entry
    if ( m_array.find( name )!=m_array.end())
        m_array.erase( name );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is the actual borrow method.
void CECOrbBorrowThread::Borrower()
{
    /*    // Loop until the thread is being destroyed
        while( m_running )
        {
            try
            {
                // Sleep for alloted time
     
                RWRunnableSelf self = rwRunnable();
     
                int j=0;
                    
                for ( int i = 0; i < TEN_MINUTES; i += TEN_SECONDS ,j++)
                {
                    self.sleep( TEN_SECONDS );
                    self.serviceCancellation();
                    if(j==6)
                    {
                        j=0;
                        long count;
                        m_server->GetProjects(count);
                    }
                }
     
                // Call borrow method on each entry
                BorrowArray::iterator it;
                for ( it = m_array.begin(); it != m_array.end(); it++ )
                    it->second->Borrow( 2 );
            }
            catch( RWCancellation )
            {
                // This exception implies that the daemon is shutting down
                // and we need to drop out of this loop.
            }
        }*/
}
