/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbLink_h
#define _CECOrbLink_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbBorrow.h>

class CECOrbServer;
class CECOrbProject;
class CECOrbFolder;

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbLink | The CECLink class is intended as a base class for all
//          OpenSTARep items that can be linked together.
// @base    public | CECOrbBorrow
class CECOrbLink : public CECOrbBorrow
{
    // @access Public Members
public:
    // @cmember Constructor
    CECOrbLink( CORBAServer* corba, CyrFactory_ptr factory );
    virtual ~CECOrbLink()
    {}

    ;  // @cmember Destructor

    string GetLinkPath();       // @cmember Returns link path of this object
    string GetFirstLinkPath();  // @cmember Returns first linked item path
    string GetNextLinkPath();   // @cmember Returns next linked item path

    CyrStServer_ptr GetHiddenServer();  // @cmember Returns Server reference
    virtual CECOrbServer*  GetParentServer() = 0;
    virtual CECOrbProject* GetParentProject() = 0;
    virtual CECOrbFolder*  GetParentFolder() = 0;

    // @cmember Pure virtual method that returns Link reference
    virtual CyrStLink_ptr GetLinkServant() = 0;

    // @access Protected Members
protected:
    CyrStServer_var m_hidden; // @cmember Reference to Server
}
;

#endif
