/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "definition.h"

#ifndef __EXCEPTION__
#include "exception.hpp"
#endif

#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#ifndef SUN
	#include <logfile.hpp>
#endif

//---------------------------------------------------------------------------
// operator<<
//---------------------------------------------------------------------------
// To ostream'ize an exception
//---------------------------------------------------------------------------
// Arguments :  os          : an ostream reference
//              exception   : an exception
// Return    :  an ostream reference
//---------------------------------------------------------------------------
ostream& operator<< (ostream& os, const CTException& exception)
{
    os << exception.what();
    return os;
}

#ifndef SUN
//---------------------------------------------------------------------------
// CTOLogFileList& operator<< CTException
//---------------------------------------------------------------------------
// 23 09 98 To ostream'ize an exception
//---------------------------------------------------------------------------
// Arguments :  os          : an CTOLogFileList reference
//              exception   : an exception
// Return    :  an ostream reference
//---------------------------------------------------------------------------
CTOLogFileList& operator<< (CTOLogFileList& os, const CTException& exception)
{
    os << exception.what();
    return os;
}
#endif

//---------------------------------------------------------------------------
// CTException::CTException
//---------------------------------------------------------------------------
// Constructor
//---------------------------------------------------------------------------
// Arguments :  msg         : an explanation message
//              filename    : the filename where the exception is throwed
//              sLineno     : the line number where the exception is throwed
// Return    :  None.
//---------------------------------------------------------------------------
CTException::CTException(const string & msg, const string & filename, unsigned int sLineno, long lNumber)
        : exception(),
        ctmsg_(msg), filename_(filename), sLineno_(sLineno), lNumber_(lNumber)
{
    fullMsg_ = ctmsg_;
    fullMsg_ += ",<";
    fullMsg_ += filename_;
    fullMsg_ += "/";
    char acString[32];
    if (lNumber_ != -1)
        sprintf(acString, "%d>,%ld", sLineno_, lNumber_);
    else
        sprintf(acString, "%d>", sLineno_);
    fullMsg_ += acString;
}


//---------------------------------------------------------------------------
// CTException::CTException
//---------------------------------------------------------------------------
// Copy Constructor.
//---------------------------------------------------------------------------
// Arguments :  exception   : an exception
// Return    :  None.
//---------------------------------------------------------------------------

CTException::CTException(const CTException& exception)
#ifndef SUN
        : exception()
#endif
{
    ctmsg_ = exception.getMsg();
    filename_ = exception.getFilename();
    sLineno_ = exception.getLineno();
    lNumber_ = exception.getNumber();
    fullMsg_ = exception.getFullMsg();
}

//---------------------------------------------------------------------------
// CTException::operator=
//---------------------------------------------------------------------------
// Operator Copy.
//---------------------------------------------------------------------------
// Arguments :  exception   : an exception
// Return    :  None.
//---------------------------------------------------------------------------
void CTException::operator= (const CTException& exception)
{
    ctmsg_ = exception.getMsg();
    filename_ = exception.getFilename();
    sLineno_ = exception.getLineno();
    lNumber_ = exception.getNumber();
    fullMsg_ = exception.getFullMsg();
}

//---------------------------------------------------------------------------
// CTException::~CTException
//---------------------------------------------------------------------------
// Destructor.
//---------------------------------------------------------------------------
// Arguments :  None.
// Return    :  None.
//---------------------------------------------------------------------------
CTException::~CTException()
{}

//---------------------------------------------------------------------------
// CTException::what
//---------------------------------------------------------------------------
// Returns information builds from the different fields of the object
//---------------------------------------------------------------------------
// Arguments :  None.
// Return    :  a sentence
//---------------------------------------------------------------------------

const char* CTException::what() const
{
    return fullMsg_.c_str();
}

void CTException::Throw (const CTException &e)
{
    throw e ;
}
