/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
/*===========================================================================*/
/*									     */
/*			     **** C O M M O N ****   			     */
/*									     */
/*===========================================================================*/
/*									     */
/*			Module Name: PSL_PTY_RTNS			     */
/*			============					     */
/*									     */
/* Description:								     */
/* ============								     */
/*									     */
/*      This module contains any routines for dealing with psuedo tty's on   */
/*      the various unix platforms.                                          */
/*          PSL_OPEN_PTYM                                                    */
/*									     */
/*===========================================================================*/
/*
 *	Define useful Include structures
 */
#include <psl_macros.h>			/* Standard PSL macros */
#include <psl_methods.h>                                              
#ifdef UNIX				/* Only build in UNIX */

#include <ptym.h>                       /* PSL_OPEN_PTYM flags */     

/*
 *	Standard ANSI header files.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include 	<errno.h>
#include        <limits.h>
#include	<dirent.h>

/*
 *       System specific header files
 */
#include        <sys/types.h>
#include        <sys/stat.h>
#include	<sys/param.h>
#include        <fcntl.h>
#include        <unistd.h>

/*
 *	Function prototypes
 */
#include psl_common_file(psl_rtndef.h)	/* Standard PSL routine prototypes */

/*
 *       External function prototypes
 */
#ifdef HP
extern char *
ptsname(int mast_fd);		/* No prototype on HP - BUG! */
#elif defined(OSF)
extern char *
ptsname(int mast_fd);		/* No prototype on OSF */
#include <pty.h>		/* Include master device definition */ 
#elif defined(NCR)
#include	<sys/ptms.h>	/* Somewhere peculiar for ptsname */
#endif /* PLATFORM */

/*
 *	Local function prototypes
 */
static int
OPEN_MASTER();

/*
 *  The possible names of master devices done with no default on #ifdefs
 *  so that new platforms will fail to compile.
 */

/* #ifdef SVR4                              */
/* #  define PTY_MASTER "/dev/ptmx"         */
/* #elif defined(HP)                        */
/* #  if  defined(HPV9) || defined(HPV10)   */
/* #    define PTY_MASTER "/dev/ptym/clone" */
/* #    define PTY_MASDIR "/dev/ptym/"      */
/* #  else                                  */
/* #    define PTY_MASTER "/dev/ptmx"       */
/* #  endif                                 */
/* #elif defined(IBM)                       */
/* #  define PTY_MASTER "/dev/ptc"          */
/* #  define PTY_MASDIR "/dev/"             */
/* #elif defined(OSF)                       */
/* #  define PTY_MASTER PTY_MASTER_DEV_SYSV */
/* #else                                    */
/* #  error What is master psuedo clone device for this new platform? */
/* #endif                                   */

/*
 * The above method has been replaced with an array of 2 master device names.  
 * The first is the default master device for the platform.  The second is an
 * alternative master device.
 * The second entry is selected by passing PTYM_ALTERNATE_MASTER in the flags
 * parameter passed to PSL_OPEN_PTYM()
 */ 

#if defined(SVR4)
   char *pty_master_table[2] = 
                                     { "/dev/ptmx", "/dev/ptmx" };
#elif defined(HPV9) || defined(HPV10)
# define PTY_MASDIR "/dev/ptym/"      
   char *pty_master_table[2] = 
                                     { "/dev/ptym/clone", "/dev/ptym/clone" };
#elif defined(HPV1020)
#  define PTY_MASDIR "/dev/ptym/"    
   char *pty_master_table[2] = 
                                     { "/dev/ptmx", "/dev/ptym/clone" };
#elif defined(IBM)
# define PTY_MASDIR "/dev/"             
   char *pty_master_table[2] = 
                                     { "/dev/ptc", "/dev/ptc" };
#elif defined(OSF)
   char *pty_master_table[2] = 
                                     { PTY_MASTER_DEV_SYSV, PTY_MASTER_DEV_SYSV };
#else
# error No master psuedo clone device specified
#endif

/*
 * Static global to hold the flags passed to PSL_OPEN_PTYM.  This is to
 * avoid changing OPEN_MASTER().
 */

static unsigned long open_master_flags;  

/*
 *  PSL_OPEN_PTYM - routine to get master pty with OK slave.
 *  =============
 *  This code gets the master end of a pty, then attempts to open the
 *  the slave end. If the slave end open fails in any way, then we call
 *  ourselves, recursively, keeping the master with failed slave open
 *  until we return from this call. This is so we don't get this one again.
 *  Call:
 *    fd = PSL_OPEN_PTYM(slav_ptr, slav_len);
 *  Where:
 *    fd        (int) File descriptor of open master end of pty,
 *              returns -1 if there is an error. 
 *    slav_ptr  (char *) Pointer to a character buffer to contain the slave
 *              devices name.
 *    slav_len  (size_t) Size of the buffer slv_ptr.
 *    flags     Various flags - see ptym.h
 */

int
PSL_OPEN_PTYM(char *slav_ptr,
	      size_t slav_len,
              unsigned long flags)
{
/*
 *  Local variables
 */
    int mast_fd;		/* File descriptor for open master */
    int next_fd;		/* File descriptor for next open master */
    int slav_fd;		/* File descriptor for slave device */
    char *slavnam_ptr;		/* Pointer to get the slave name */
#if defined(SVR4) || defined(OSF) || defined(HPV1020)                 
    int ists;			/* Routine return status */
#endif /* SVR4 or OSF */

/*
 * Copy the flags to a static global for use by OPEN_MASTER()
 */

    open_master_flags = flags;

/*
 *  Open the Master end of a pty.
 */
    mast_fd = OPEN_MASTER();
    if(mast_fd < 0)		     		/* If error */
    {
	*slav_ptr = '\0';
	return mast_fd;
    }

/*
 *  We now have the master end opened.
 *  We first get the slave device name associated with this.
 */
#ifdef IBM
    slavnam_ptr = ttyname(mast_fd);
#else /* not IBM */
    slavnam_ptr = ptsname(mast_fd);
#endif /* not IBM */
    if(slavnam_ptr == NULL)
    {
	next_fd = PSL_OPEN_PTYM(slav_ptr, slav_len, flags);           
	(void)close(mast_fd);
	return next_fd;
    }

/*
 *  Copy slavnam_ptr, which is a pointer into the space of the function
 *  that returned us this name, into the variable we will return if successful.
 */
    (void)memcpy(slav_ptr,
		 slavnam_ptr,
		 slav_len);
    slav_ptr[slav_len-1] = '\0'; /* Make sure NULL terminated */

#if defined(SVR4) || defined(OSF) || defined(HPV1020)                 
/*
 *  These function given the file descriptor of the master pty changes
 *  the mode and ownership of the slave end, and get prepare it ready for
 *  opening
 */

#   if defined (HPV1020) 
    if( (flags & PTYM_ALTERNATE_MASTER) == 0 ) 
#   endif 
    {
        ists = grantpt(mast_fd);
        if(ists < 0)
        {
            next_fd = PSL_OPEN_PTYM(slav_ptr, slav_len, flags);       
            (void)close(mast_fd);
            return next_fd;
        }
        ists = unlockpt(mast_fd);	/* Unlock slave */
        if(ists < 0)
        {
            next_fd = PSL_OPEN_PTYM(slav_ptr, slav_len, flags);       
            (void)close(mast_fd);
            return next_fd;
        }
    }

#endif /* SVR4 or OSF or HPV1020*/

/*
 *  Attempt to open the slave device to make sure it is not 'wedged'.
 */
    do
    {
	slav_fd = open(slav_ptr, (O_RDWR | O_NOCTTY));
    }
    while((slav_fd < 0) && (errno == EINTR));			     
    if (slav_fd < 0)
    {
	next_fd = PSL_OPEN_PTYM(slav_ptr, slav_len, flags);      
	(void)close(mast_fd);
	return next_fd;
    }

/*
 *  Now close the slave we just opened, we're not really interested in it,
 *  just if it can be opened.
 */
    (void)close(slav_fd);

/*
 *  Exit here successfully with the file descriptor of master.
 */
    return mast_fd;
}

/*
 *  Previous version of PSL_OPEN_PTYM - 42X015 
 *
 *  psl_open_ptym_1 - routine to get master pty with OK slave.
 *  ===============
 *  This code gets the master end of a pty, then attempts to open the
 *  the slave end. If the slave end open fails in any way, then we call
 *  ourselves, recursively, keeping the master with failed slave open
 *  until we return from this call. This is so we don't get this one again.
 *  Call:
 *    fd = psl_open_ptym_1(slav_ptr, slav_len);
 *  Where:
 *    fd        (int) File descriptor of open master end of pty,
 *              returns -1 if there is an error. 
 *    slav_ptr  (char *) Pointer to a character buffer to contain the slave
 *              devices name.
 *    slav_len  (size_t) Size of the buffer slv_ptr.
 */
int
psl_open_ptym_1(char *slav_ptr,
	        size_t slav_len)
{
/*
 *  Local variables
 */
    int mast_fd;		/* File descriptor for open master */
    int next_fd;		/* File descriptor for next open master */
    int slav_fd;		/* File descriptor for slave device */
    char *slavnam_ptr;		/* Pointer to get the salve name */
#if defined(SVR4) || defined(OSF) || defined(HPV1020)                 
    int ists;			/* Routine return status */
#endif /* SVR4 or OSF */

/*
 * Initialise the static global flags for use by OPEN_MASTER() to
 * ensure backwards compatibility.
 */

    open_master_flags = 0x0;

/*
 *  Open the Master end of a pty.
 */
    mast_fd = OPEN_MASTER();
    if(mast_fd < 0)		     		/* If error */
    {
	*slav_ptr = '\0';
	return mast_fd;
    }

/*
 *  We now have the master end opened.
 *  We first get the slave device name associated with this.
 */
#ifdef IBM
    slavnam_ptr = ttyname(mast_fd);
#else /* not IBM */
    slavnam_ptr = ptsname(mast_fd);
#endif /* not IBM */
    if(slavnam_ptr == NULL)
    {
	next_fd = psl_open_ptym_1(slav_ptr, slav_len);
	(void)close(mast_fd);
	return next_fd;
    }

/*
 *  Copy slavnam_ptr, which is a pointer into the space of the function
 *  that returned us this name, into the variable we will return if successful.
 */
    (void)memcpy(slav_ptr,
		 slavnam_ptr,
		 slav_len);
    slav_ptr[slav_len-1] = '\0'; /* Make sure NULL terminated */

#if defined(SVR4) || defined(OSF) || defined(HPV1020)
/*
 *  These function given the file descriptor of the master pty changes
 *  the mode and ownership of the slave end, and get prepare it ready for
 *  opening
 */
    ists = grantpt(mast_fd);
    if(ists < 0)
    {
	next_fd = psl_open_ptym_1(slav_ptr, slav_len);
	(void)close(mast_fd);
	return next_fd;
    }
    ists = unlockpt(mast_fd);	/* Unlock slave */
    if(ists < 0)
    {
	next_fd = psl_open_ptym_1(slav_ptr, slav_len);
	(void)close(mast_fd);
	return next_fd;
    }
#endif /* SVR4 or OSF or HPV1020*/

/*
 *  Attempt to open the slave device to make sure it is not 'wedged'.
 */
    do
    {
	slav_fd = open(slav_ptr, O_RDWR | O_NOCTTY);
    }
    while((slav_fd < 0) && (errno == EINTR));			
    if(slav_fd < 0)
    {
	next_fd = psl_open_ptym_1(slav_ptr, slav_len);
	(void)close(mast_fd);
	return next_fd;
    }

/*
 *  Now close the slave we just opened, we're not really interested in it,
 *  just if it can be opened.
 */
    (void)close(slav_fd);

/*
 *  Exit here successfully with the file descriptor of master.
 */
    return mast_fd;
}

/*
 *  OPEN_MASTER - routine to open the master end of a pty
 *  ===========
 *  This static local routine is used to open a master end of a pty, either
 *  by using a cloning device, if one is present, or searching the available
 *  master to find one which opens.
 *  Call:
 *	fd = OPEN_MASTER();
 *  Where:
 *	fd	is the file desacriptor of a succesfully opened master pty or
 *           	-1 if none are available.
 */
static int
OPEN_MASTER()
{
/*
 *  Local variables
 */
    int mast_fd;		/* For masters file descriptor */
    static char	pt_master[MAXPATHLEN]; /* Buffer for master filename */
#if !defined(SVR4) && !defined(OSF) 
    DIR *dirp;			/* Pointer to directory stream */
    struct dirent *direntp;	/* Pointer to directory entry structure */
    size_t masdir_len;		/* Length of master directory string */    
#endif /* not SVR4 */

/*
 * Define pointer to the PTY master device name
 */

char *pty_master_p;

/*
 * Obtain pointer to the PTY master device
 */

   if( open_master_flags & PTYM_ALTERNATE_MASTER )
   {
      pty_master_p = pty_master_table[1];
   }
   else
   {
      pty_master_p = pty_master_table[0];
   }
 
/*
 *  Try to open the clone device. Looping because this code called when
 *  alarms potentially set up.
 */
    (void)strcpy(pt_master, pty_master_p);
    do
    {
	mast_fd = open(pt_master,
		       (O_RDWR | O_NDELAY | O_NOCTTY));
    }
    while((mast_fd < 0) && (errno == EINTR));

#if defined(HP)
    if((mast_fd < 0) && (errno == EBUSY))
    {
	errno = ENODEV;
    }

#elif defined(OSF)
    if((mast_fd < 0) && (errno == ETOOMANYREFS))
    {
	errno = ENODEV;
    }
#endif /* HP both versions  */

/*
 * If HP-UX 10.20 and not using the alternate cloning device then return
 */
#if defined(HPV1020)
    if( (open_master_flags & PTYM_ALTERNATE_MASTER) == 0 )
    {
       return mast_fd;
    }
#endif

#if !defined(SVR4) && !defined(OSF) 
/*
 *  If fail to open the Master cloning device for a predictable reason,
 *  No access or simply not there.
 */
    if((mast_fd < 0) &&
       ((errno == EACCES) || (errno == ENOENT)))
    {
/*
 *      Here we search for an available master pty should no cloning device
 *      be accessable.
 */
	(void)strcpy(pt_master, PTY_MASDIR);
	masdir_len = strlen(pt_master);
	dirp = opendir(pt_master);		/* Try open the device dir */
	if(dirp == NULL) return -1;		/* If Error */

/*
 *      Loop through all the entries in the directory
 */
	while((direntp = readdir(dirp)) != NULL)
	{
	    if((direntp->d_name[0] == 'p') &&
	       (direntp->d_name[1] == 't') &&
	       (direntp->d_name[2] == 'y'))	/* If filename starts 'pty' */
	    {
		(void)strcpy(&(pt_master[masdir_len]),
			     direntp->d_name);
		do
	        {	
		    mast_fd = open(pt_master,
				   (O_RDWR | O_NDELAY | O_NOCTTY));
		}
		while((mast_fd == -1) && (errno == EINTR));

		if(mast_fd != -1)		/* Open succeeded */
		{
		    goto break_pt;		/* Abandon search sucessfully */
		}
	    }
	}					/* Dir entries loop */
/*
 *	We only get here if we have not found a free master pty and we have
 *	no more entries to try. Therefore we assume that all the pty's have been
 *	exhausted and set errno to ENODEV as the opens on clone device would.
 */
	errno = ENODEV;
/*-40X177*/

/*
 *BREAK_PT:
 *  	This label is used to escape from the while loop on success
 */
break_pt:
        (void)closedir(dirp);	
    }						/* end else no clone */
#endif /* not SVR4 */

/*
 *  Exit here
 */
    return mast_fd;
}

/*
 *  PSL_IS_A_STREAMS_PTY - routine to determine whether the pty is STREAMS
 *  ====================
 *
 *  This code receives the name of a pty and determines whether it is a
 *  STREAMS based pty or not.  Currently this is only useful on 
 *  HP-UX 10.20 where a decision has to be made on the name.  Otherwise
 *  the default type for the current platform is returned.
 *  Call:
 *    result = PSL_IS_A_STREAMS_PTY(pty_name)
 *  Where:
 *    result    (int) Returns 0 if not a STREAMS pty, 1 if a STREAMS pty.
 *              returns -1 if there is an error. 
 *    pty_name  (unsigned char *) Pointer to a character buffer containing 
 *              the full pathname of the pty.
 */


int PSL_IS_A_STREAMS_PTY(char *pty_name)
{
#if defined(USE_PSLDIM) && !defined(TWIN_CTM_SERVERS)
  return 1;
#else
#  if defined(TWIN_CTM_SERVERS)

  if( pty_name == NULL )
    {
      return -1;
    }
  else
    {
      if( strstr(pty_name,"pts") != NULL )  /* Sort out a better way later */
	{
	  return 1;
	}
      else
	{
	  return 0;
	}
    }
#  else
  return 0;
#  endif
#endif

}

#endif /* UNIX */
