/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//TPrmRealRules.h
#ifndef CYR_TPRMREALRULES_H
#define CYR_TPRMREALRULES_H
/**
 * The definitions of the actual parameter rules, all derived
 * from the TParamRule class
 **/

#include <string>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
#else //unknown
#error Unknown Platform
#endif // unknown

// Local includes
#include "TParamRule.h"

/**
 * class definition of parameter rule with no associated value
 **/
class TPrmToknRule : public TParamRule
{
public:
    static const string TYPE;
    // Constructors & Destructor
    TPrmToknRule()
    {}

    TPrmToknRule(const string& token);
    // Accessors
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const;
    virtual string type() const;
};

/**
 * class definition of parameter rule for dates
 **/
class TPrmDateRule : public TParamRule
{
public:
    static const string TYPE;
    // Constructors & Destructor
    TPrmDateRule()
    {}

    TPrmDateRule(const string& token);
    // Accessors
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const;
    virtual string type() const;
private:
    TPrmDateRule(const TPrmDateRule&);
    operator=(const TPrmDateRule&);
};

/**
 * class definition of parameter rule whose value is a short piece of
 * text.  Short text is anything that won't contain line termination
 * chars or possibly start with leading whitespace.
 **/
class TPrmSTxtRule : public TParamRule
{
public:
    static const string TYPE;
    // Constructors & Destructor
    TPrmSTxtRule()
    {}

    TPrmSTxtRule(const string& token);
    // Accessors
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const;
    virtual string type() const;
};

/**
 * class definition of parameter rule whose value is a Long piece
 * of text. Long text is anything that doesn't fall into the above
 * short text catagory.
 **/
class TPrmLTxtRule : public TParamRule
{
public:
    static const string TYPE;
    // Constructors & Destructor
    TPrmLTxtRule()
    {}

    TPrmLTxtRule(const string& token);
    // Accessors
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const;
    virtual string type() const;
};

/**
 * class definition of parameter rule whose value is an integer value.
 **/
class TPrmINumRule : public TParamRule
{
    int _base;
    static const string VALREF;
public:
    static const string TYPE;
    // Constructors & Destructor
    TPrmINumRule()
    {}

    TPrmINumRule(const string& token);
    // Accessors
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const;
    virtual int intVal(const string& str) const;
    virtual string intVal(int val) const;
    virtual bool intVal() const;
    virtual string type() const;
};

#endif // CYR_TPRMREALRULES_H




