/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TRecHolder.h
#ifndef _CYR_TRECHOLDER_H
#define _CYR_TRECHOLDER_H
/**
 * Definititon for class to hold and manipulate Transaction
 * records.
 **/
#include <string>
#include <list>
#include <vector>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
#else //unknown
#error Unknown Platform
#endif // unknown

// local includes
#include "TRecord.h"
#include "mdbmtx.h"

typedef list<TRecord*> treclist_t;

// class definition
class TRecHolder
{
    enum {RES_BLOCK = 32};
    treclist_t _trecs;
    treclist_t::iterator* _trecitArray;
    Cmdbmutex iter_mutex;
    unsigned int _reserved;
    unsigned int _length;
    bool _modified;
    void fix_trecits();
    // Constructors & Destructor
    TRecHolder(treclist_t& trecs);

public:
    TRecHolder();
    virtual ~TRecHolder();

    // Accessors
    virtual unsigned int length()
    {
        fix_trecits();
        return _length;
    };
    virtual TRecord* operator[](unsigned int idx);
    virtual TRecord* rec_at_index(unsigned int idx);
    virtual bool findRecType(const AtvfKey& key, unsigned int& idx);
    virtual bool find(const string& text,
                      int& recidx, int& prmidx,
                      bool matchCase = true,
                      bool forward = true);
    virtual TRecord* copy(unsigned int idx);
    virtual TRecHolder* copy(unsigned int idx1, unsigned int idx2);
    virtual void save(ostream& out) const;
    virtual void intPrmLimits(const string& key, int& min, int& max);

    // Mutators
    virtual void limitIntPrms(const string& key, int min, int max);
    virtual void copyPrms(const string& fromKey, const string& toKey);
    virtual void setPrmVals(const string& key, const string& val);
    virtual bool add(TRecord* recP);
    virtual bool insert(unsigned int idx, TRecord* trecP);
    virtual bool insert(unsigned int idx, TRecHolder* tRHP);
    virtual TRecord* cut(unsigned int idx);
    virtual TRecHolder* cut(unsigned int idx1, unsigned int idx2);
private:
    TRecHolder(const TRecHolder&);
    operator=(const TRecHolder&);
}
; // TRecHolder

ostream& operator<<(ostream& os, TRecHolder& h);

#endif // _CYR_TRECHOLDER_H
