/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//////////////////////////////////////////////////////////////////////
// TExpression.h: interface for the TExpression class.
//////////////////////////////////////////////////////////////////////

#if !defined(CYR_TEXPRESSION)
#define CYR_TEXPRESSION

#if defined(WIN32)
# if _MSC_VER >= 1100   // MSC version 5 and above
#pragma warning(disable:4786)
# endif
#endif

#include <string>
#include <list>
#include <vector>
#include <stack>

#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif

// Forward declarations
class TVarDetails;
class TContext;

class TExpression
{
public:
    TExpression();
    TExpression(const TExpression& trec);
    virtual ~TExpression();

    bool SetExpression(const string& inStr, int pos = 0);
    string Evaluate(TContext& ctxt) const;	// Returns empty string on failure
    const string& Reason() const	// Reason for Prepare/Evaluation failure
    {
        return _reason;
    };

    static const string stFALSE;
    static const string stTRUE;

protected:
    class Operand // Internal class, holds operand as either a double or a string
    {
    public:
        Operand(const string& inStr)
        {
            // Numeric operand ?
            if (!inStr.empty() && (isdigit(inStr[0]) || inStr[0] == '.' || inStr[0] == '-'))
            {
                _strType = false;
                _numVal = atof(inStr.c_str());
            }
            else
            {
                _strType = true;
                if (inStr == "\"\"")
                    _strVal = string();
                else
                    _strVal = inStr;
            }
        }

    Operand(const double inVal) : _strType(false), _numVal(inVal)
    {}
        ;

        bool IsString()
        {
            return _strType;
        }
        double NumVal()
        {
            return _numVal;
        }
        const string& StrVal()
        {
            return _strVal;
        }

    private:
        bool _strType;
        double _numVal;
        string _strVal;
    };

    bool _ProcessChar(string::iterator& it, string::iterator endPos);
    int _Priority(const string& oper) const;
    bool _DefinedForStrings(const string& oper) const;

    list<string>	_postfix;
    vector<TVarDetails*> _varList;

    stack<string>	_opStack;
    string			_operand;

    mutable string	_reason;
};

#endif // !defined(CYR_TEXPRESSION)
