/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TPLOG_H_
#define _TPLOG_H_
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#ifndef CYRANO_TP_STANDALONE
#   include "Logging/CECLog.h"
#   include <list>
#   include <string>
#else
#   include "CECSACORBA.h"
#endif

#ifndef TH_POOL_VER
#   include <mutex.hpp>
#else
#   define _WIN32_WINNT 0x0404
#   define WINVER       0x0400
#   include "windows.h"
#   include "FastLock.hpp"
#endif

#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
extern const char* STARTUP;//Startup trace messages
extern const char* SHUTDOWN;//shutdown trace messages
extern const char* SCRIPTCOMMAND;//script command
extern const char* DBGSTD;//non-specific debug message
extern const char* AUDITFAIL;//message failed to get to Audit type logging
extern const char* NOTE;//Note message
extern const char* USERTRACE;//Trace Message
extern const char* AUDIT;//Copy of Audit message
extern const char* REPOSITORY;//Repository Debug messages
extern const char* THREADING;//Thread messages
extern const int   ERRORLEVEL;//Important Trace error
extern const int   WARNINGLEVEL;
extern const int   DEBUGLEVEL;//Sort of mid range level
extern const int   DETAILLEVEL;//Very specific Trace messages
extern const int   AUDITLEVEL;//Level at which Audit logs are traced
extern const int   TRACELEVEL;//A level to use for trace messages
extern const int   NOTELEVEL;//A level to use for Note messages

#ifndef CYRANO_TP_STANDALONE

class TPLog
{
public:
    TPLog();
    virtual ~TPLog();
    virtual void Setup( CORBAServer* pServer, const char* name, long level = 0 );
    virtual void AddTrace( const char* category, long level, const char* msg );
    virtual void SetLevel( const char* category, long level);
    virtual void Store( CECOrbFolder* pFolder );
    inline CECLog* operator->() const   { return m_Log; }
    inline operator CECLog* () const    { return m_Log; }

private:
    CECLog* m_Log;
    class TraceMsg
    {
    public:
        TraceMsg(const string& cat,long lev,const string& mesg)
            :category(cat),level(lev),msg(mesg)
        {}
        ;
        virtual ~TraceMsg()
        {}
        ;
        TraceMsg(const TraceMsg& t)
            :category(t.category),level(t.level),msg(t.msg)
        {}
        ;
        string category;
        long level;
        string msg;
    };
    typedef std::list<TraceMsg> TraceList_t;
    TraceList_t  m_TraceList;

#   ifndef TH_POOL_VER
    CTMutex LogMtx;
#   else
    CFastLock *g_pTPLogFl;
#   endif
private:
    //// Forbidden:
    TPLog(const TPLog&);
    operator=(const TPLog&);
};

#else   // CYRANO_TP_STANDALONE

#   ifdef __DLLSA__
#       include <map>
#       define _WIN32_WINNT 0x0404
#       define WINVER       0x0400
#       include <windows.h>
#       include <iostream>
        extern HWND gHwnd;
#   endif

class TPLog
{
public:
    TPLog()
    {}

    virtual ~TPLog()
    {}

    virtual void Setup( CORBAServer* pServer, const char* name, long level = 0 )
    {}

    virtual void AddTrace( const char* category, long level, const char* msg)
    {
#ifdef __DLLSA__
        if (level == TRACELEVEL)
        {
            if (gHwnd != NULL)
                PostMessage(gHwnd, WM_APP+1, 0, (LPARAM)(new std::string(msg)));
            else
                std::cout << msg << std::endl ;
        }
        else
            if (level == ERRORLEVEL)
            {
                if (gHwnd != NULL)
                    PostMessage(gHwnd, WM_APP+2, 0, (LPARAM)(new std::string(msg)));
                else
                    std::cout << msg << std::endl ;
            }
            else
                if (level == 10000)
                {
#ifndef TH_POOL_VER
                    if (gHwnd != NULL)
                        PostMessage(gHwnd, WM_APP+3, 0, (LPARAM)(new std::string(msg)));
#endif
                }
#else
if ( (level == TRACELEVEL) || (level == ERRORLEVEL) )
    std::cout << msg << std::endl ;
#endif
    }

    virtual void SetLevel( const char* category, long level)
{}

private:
    bool m_stored;
    long m_level;
};

#endif

extern TPLog*	    logr;//Global Trace Object

#endif
