/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//mdbexcept.h
#ifndef _CYR_MDBEXCEPT_H
#define _CYR_MDBEXCEPT_H
/**
 * This file contains exception definitions for general use.
 * All exceptions should be meaningfully descended from
 * standard C++ exceptions or their descendents.
 **/
#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include <string>
#include <stdexcept>
#include <new>

#ifndef CYR_NO_STD_NAMESPACE
using std::string;
using std::runtime_error;
using std::bad_alloc;
#endif

/**
 * The libg++ exception hierarchy seems broken at 2.7.2 so
 * as a temp fix we create our own runtime_error which
 * effectively overrides everything on the standard one.
 **/
class mdbRuntimeError : public runtime_error
{
    string _error;
public:
    mdbRuntimeError(const string& error)

    //
    // LINUX Port Change
    //
    // Add on LINUX to next statement
#if defined(WIN32) || defined(__VMS) || defined(LINUX) || defined(SUN) || defined(HPUX)
:   runtime_error(""),
#else
:
    runtime_error(),
#endif
    _error(error)
    {}
#ifdef __VMS
    ~mdbRuntimeError() _RWSTD_THROW_SPEC_NULL { ;
                                              }
#endif
    string What() const
    {
        return _error;
    }
};

/**
 * Exception intended to be used with any problem with
 * performing IO.
 **/
class mdbIOException : public mdbRuntimeError
{
    string _ioname;
public:
    mdbIOException(const string& error, const string& ioname)
:   mdbRuntimeError(error),
    _ioname(ioname)
    {}
#ifdef __VMS
    ~mdbIOException() _RWSTD_THROW_SPEC_NULL { ;
                                             }
#endif
    string ioname() const
    {
        return _ioname;
    }
};

/**
 * Exception intended to be used with any problem
 * relating to the parsing of some input
 **/
class mdbParseException : public mdbRuntimeError
{
public:
mdbParseException(const string& error) : mdbRuntimeError(error)
    {}
#ifdef __VMS
    ~mdbParseException() _RWSTD_THROW_SPEC_NULL { ;
#endif
};

// Out of memory condition handler stuff
#if defined(WIN32)
#define SET_NEW_HANDLER     _set_new_handler
// Maps malloc fail to new fail handler
#define SET_NEW_MODE        _set_new_mode(1);
int mdb_new_handler(size_t size);

#else
#define SET_NEW_HANDLER     set_new_handler
// Maps malloc fail to new fail handler, WIN32 only
#define SET_NEW_MODE
void mdb_new_handler(void);

#endif


#endif // _CYR_MDBEXCEPT_H

