/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
//
//
#ifndef CYRANO_TP_STANDALONE
#include "OSTAORB/OSTAORB.h"    
#include <GlobalVariable.h>
#else
#include <CECSACORBA.h>
#endif

#include "TPLog.h"

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#ifdef CYRANO_TEST_MANAGER
#include "TestManager_i.h"
#else
#include "TestExecuter_i.h"
#endif

#include <string>
#include <new.h>
#include "mdbexcept.h"
#include "mdbgetopt.h"
#include "cxxutils.h"
#include <CyrHelper.inl>

#ifdef CYRANO_TEST_MANAGER
#include "TMthreadcontrol.h"
#endif

#include "mdbtypes2.h"

extern "C"
{
#include <psl_macros.h>
#include <psl_rtndef.h>
}
#ifdef WIN32
using std::string;
#endif
#ifdef CYRANO_TEST_MANAGER
#define     VERSION      "3.0.0.0"          // Test Manager version
#define     ODLFILE      "TestManager.odl"  // Test Manager's Odl File
const string    TPTEXT  = "TestManager";    // Used for Tracing filename
#elif defined(CYRANO_TEST_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TestExecuter.odl"
const string    TPTEXT  = "TestExecuter_cmd";
#elif defined(CYRANO_MSSQL_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEMSsql.odl"
const string    TPTEXT  = "TestExecuter_mss";
#elif defined(CYRANO_SYBASE_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TESyb.odl"
const string    TPTEXT  ="TestExecuter_syb";
#elif defined(CYRANO_ORACLE_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEOra.odl"
const string    TPTEXT  = "TestExecuter_ora";
#elif defined(CYRANO_HTTP_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEHttp.odl"
const string    TPTEXT  = "TestExecuter_web";
#else
#error "Undefined TestManager/TestProcess"
#endif
// Environment variable for determining maximum number of threads per Test Executer
#define     TE_MAX_THREADS  "TE_MAX_THREADS"

// Local functions
int Run (CORBAServer* pServer,string exename);
void DeleteStatisticsFiles(const string & prefixStatFileName, int maxthreads = 2000) ;
void sqlr_cleanup (void);

// Global Variables
// Static: for use only here
static CTMutex cleanup_mutex;
static Cmdbgetopt *     cl  = NULL;

//The general idea here is to have all TestManagerImpl Access done in this file
//hopefully removing the need to have it as a fully global variable
//This should encourage minimal access from separate threads
#ifdef CYRANO_TEST_MANAGER
#define TESTPROCESSIMPL     TestManagerImpl
#define TESTPROCESS     TestManager
#define TPROCESS        TManager
#define TESTPROCESS_PTR     TestManager_ptr
#define TPROCESS_PTR        TManager_ptr
#define TESTPROCESS_NIL     TestManager::_nil()
#define THREADCONTROL       TMthreadcontrol
#else
#define TESTPROCESSIMPL     TestExecuterImpl
#define TESTPROCESS     TestExecuter
#define TPROCESS        TExecuter
#define TESTPROCESS_PTR     TestExecuter_ptr
#define TPROCESS_PTR        TExecuter_ptr
#define TESTPROCESS_NIL     TestExecuter::_nil()
#endif
static TESTPROCESSIMPL*     TPROCESS=NULL;
#ifndef CYRANO_TP_STANDALONE
//But just in case we do need to access it elsewhere here is a _ptr ptr
TESTPROCESS_PTR         TPROCESS_PTR=TESTPROCESS_NIL;
#endif

string exedir="";

#ifdef CYRANO_TP_STANDALONE
string toffiledir="";
#endif



int main(int argc, char * argv[])
{
    logr=new TPLog();

    // Set handler for out of memory conditions in new operator.
    SET_NEW_HANDLER(mdb_new_handler);

    // Maps malloc failure to new fail handler on WIN32 only.
    SET_NEW_MODE

    //initialise CORBA for logging purposes
    CORBAServer* pServer;
    try
    {
        pServer=new CORBAServer(argc,argv);
    }
    catch(...)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,"Caught Exception on startup, is Nameservice Running?" );
        sqlr_cleanup();
        return 1;
    }


    string prefixStatFileName = "" ;
    // Catch exceptions for command line errors
    try
    {
        // Create command line arg object, general option syntax error
        // exception (string type) can be thrown here.
        cl = new Cmdbgetopt(argc, argv, "TestManager: ");
        // Define valid command line options and value types
        cl->define_option("-f", CLO_STRINGVAL);
        cl->define_option("-t", CLO_UNSVAL);
        cl->define_option("-u", CLO_UNSVAL);
        cl->define_option("-v", CLO_INTVAL);
        cl->define_option("-l", CLO_NOVAL);
        cl->define_option("-1", CLO_NOVAL);

#ifdef CYRANO_TP_STANDALONE
        cl->define_option("-b", CLO_UNSVAL);
        cl->define_option("-i", CLO_UNSVAL);
#endif
        // New command-line parameter to indicate that the
        // this shall be a Test Manager
        cl->define_option("-r", CLO_STRINGVAL);
        cl->define_option("-d", CLO_NOVAL);
        cl->validate();

        // Get pid for this TestProcess
        MDBPID tppid=getpid();

        try
        {
#ifndef CYRANO_TP_STANDALONE
            CECInetInfo info;
            string HostName=info.GetIPAddressAsString();
            //Initialise Global Logging Object
            int loglevel=cl->get_int_value_for("-v");
            string RegisteredLogName;
            RegisteredLogName=string(TPTEXT)+"_"+HostName+"_"+int_to_string(tppid);
            logr->Setup(pServer, RegisteredLogName.c_str(),loglevel);

            // Statistics prefix file name
            char szBuf[40];
            sprintf(szBuf, "TEW_%s-%d", HostName.c_str(), tppid);
            prefixStatFileName = szBuf;
#endif
        }
        catch(...)
        {
            logr->AddTrace(STARTUP,ERRORLEVEL,"Caught Exception on startup, is Nameservice Running?" );
            sqlr_cleanup();
            return 1;
        }
        logr->AddTrace(STARTUP,ERRORLEVEL,"Start Execution");
        // Now that the logging mechanisms are in place
        // Check if just a request for version info
        if(cl->option_supplied("-1"))
        {
            cout << TPTEXT << " version " << VERSION << " build "
            << __DATE__ << " " << __TIME__ << endl;
            logr->AddTrace(STARTUP,TRACELEVEL,"Version information requested");
            sqlr_cleanup();
            return 0;
        }

    }
    catch(const string& es)
    {
        // Detected error while checking args
        logr->AddTrace(STARTUP,ERRORLEVEL,es.c_str());
        sqlr_cleanup();
        return 1;
    }

    catch(const exception& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what());
        sqlr_cleanup();
        return 1;
    }
    catch( ... )
    {
        // Some other unexpected exception caught
        logr->AddTrace(STARTUP,ERRORLEVEL,"Caught exception while parsing command options" );
        sqlr_cleanup();
        return 1;
    }

    // Read in atvf(tof) file to create the Script Rules, and commence execution
    try
    {
        string exename;
        if(argc>0) // if we have a executable name use in logging else don't bother
            exename=filename_base(string(argv[0]));
        else
            exename="";
        // Get directory of this executable, path used to invoke is argv[0].
        exedir = filename_dir(string(argv[0]));

#if (!defined(CYRANO_TP_STANDALONE) && !defined(CYRANO_TEST_MANAGER) )
        //
        // Delete all statistics files whose name looks like
        // the prefix name
        //
        DeleteStatisticsFiles(prefixStatFileName) ;
#endif
        //
        //    ...and then run
        //

        Run(pServer,exename);
        delete pServer; //End of CORBA
    }

    catch(const string &es)
    {
        // Detected error
        logr->AddTrace(STARTUP,ERRORLEVEL,es.c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(const mdbIOException& e)
    {
        string msg=e.what();
        msg += ": "+e.ioname();
        logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(const mdbParseException& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what());
        sqlr_cleanup();
        return 1;
    }
    catch(const CyrRepositoryException& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what().c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(const exception& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what());
        sqlr_cleanup();
        return 1;
    }
#ifndef CYRANO_TP_STANDALONE
    catch(const CORBA::Exception&e )
    {
        // Some other unexpected exception caught
        string msg="Caught CORBA::"+CORBAExString(e);
        logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str());
        sqlr_cleanup();
        return 1;
    }
#endif
    catch( ... )
    {
        // Some other unexpected exception caught
        logr->AddTrace(STARTUP,ERRORLEVEL,"Caught exception");
        sqlr_cleanup();
        return 1;
    }
    exit(0);

} // end of MAIN


// sqlr_cleanup
// ============
//
void sqlr_cleanup(void)
{
    // no-one else here for a bit
    CTLockGuard<CTMutex> guard(cleanup_mutex); // use guard to deal with exceptions
    if(logr)logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clean up on Exiting ");

    // Catch any exceptions during cleanup. This could happen if
    // a thread was hard killed, leaving some object in an undefined
    // state.
    try
    {
        if (TPROCESS)//Tell monitoring that closedown is imminent and clear up TestProces Object member data
        {
            TPROCESS->SetToCloseDown();
        }
        if (TPROCESS)
        {
#ifndef CYRANO_TP_STANDALONE
            CORBA::release(TPROCESS_PTR);
            TPROCESS->_remove_ref();    
            TPROCESS=NULL;
#endif
        }
        if(cl)
        {
            delete cl;
            cl=NULL;
        }
    }
#ifndef CYRANO_TP_STANDALONE
    catch(const CyrGlobalVariableException& e)
    {
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,(const char*)e.what);
        exit(0);
    }
#endif
    catch(const CyrRepositoryException& e)
    {
        string logstr;
        logstr=e.what()+","+e.where()+" Status: "+int_to_string(e.status());
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,logstr.c_str());
        exit(0);
    }
#ifndef CYRANO_TP_STANDALONE
    catch(const CORBA::Exception& e)
    {
        string msg="Caught CORBA::"+CORBAExString(e)+" in clean-up";
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,msg.c_str());
        exit(0);
    }
#endif
    catch(const exception &e)
    {
        string msg=string("Caught exception in clean-up :")+string(e.what());
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,msg.c_str());
        exit(0);
    }
    catch( ... )
    {
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,"Caught exception in clean-up ");
        exit(0);
    }
    if(logr)
    {
        delete logr;
        logr=NULL;
    }
} //end of sqlr cleanup


//
//    Run
//
//    This is the main processing section of sqlr.
//
int Run (CORBAServer* pServer,string exename)
{

    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Before TScript");
    // Create rule for this apps file type
    string odlfile;
    // create odlfile

    // Build odl file name
    odlfile = exedir + FN_PATHSEP + ODLFILE;
#ifdef CYRANO_TP_STANDALONE
    toffiledir = exedir + FN_PATHSEP;
#endif

    string msg="Using odl file: " +odlfile ;
    logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str());

    TScriptRule* rule ;
    rule = new TScriptRule(odlfile);

    logr->AddTrace(DBGSTD,DEBUGLEVEL,"After TScript");
    //
    //    Initialize the CORBA Initialise - pends Test Manager
    //
    // Create thread control object
    // Get MAXIMUM number of threads required for process if a TestExecuter
#ifndef CYRANO_TEST_MANAGER
    char* c_maxthr=getenv(TE_MAX_THREADS);
    int maxthr;
    if(c_maxthr!=NULL)
    {
        maxthr=atoi(c_maxthr);
    }
    else
    {
        maxthr=0;
    }
#endif
    // Get the test name passed in on the command line - if any at
    // all. This shall determine in External mode whether this object
    // shall act as a Test Manager(TestName) or Test Executer(no TestName)
    string strTestName = cl->get_value_for("-r");
    int ProcessID = getpid();
    // Get the node name
#ifndef CYRANO_TP_STANDALONE
    CECInetInfo info;
    string HostName=info.GetIPAddressAsString();
#else
    string HostName="localhost";
#endif
    // Set the name to be registered, For test managers this is<nodename.project.test.testname.pid>
    char    strProcess[64];
#ifndef CYRANO_TP_STANDALONE
    string  ObjectClass;
    // Test Manager registration
    ObjectClass = TESTPROCESS::CLASS_NAME;
#endif
#ifdef CYRANO_TEST_MANAGER
    sprintf(strProcess, "%s_%s_%8.8lX",  HostName.c_str(), strTestName.c_str(), ProcessID);
    // Create the instance of the test manager CORBA object
    TPROCESS=new TESTPROCESSIMPL(pServer,string(strProcess),HostName,ProcessID,exename,rule);
#else
    sprintf(strProcess, "%s_%8.8lX", HostName.c_str(), ProcessID);  
    // Create the instance of the test manager CORBA object
    TPROCESS=new TESTPROCESSIMPL(pServer,string(strProcess),HostName,ProcessID,exename,rule,maxthr);
#endif
    assert(TPROCESS);
#ifndef CYRANO_TP_STANDALONE
    TPROCESS_PTR=TPROCESS->_this();
    if(cl->option_supplied("-d"))// if run in debug mode, register as such
    {
        string debug_reg="DEBUG_"+HostName+"_"+strip_extension(exename);
        string trcmsg="Debug requested, register as "+debug_reg;
        logr->AddTrace(STARTUP,DEBUGLEVEL,trcmsg.c_str());
        CORBA::String_var dbg_reg=CORBA::string_dup(debug_reg.c_str());
        CORBA::String_var obj_reg=CORBA::string_dup(ObjectClass.c_str());
        pServer->NameObject(TPROCESS->_this(),obj_reg.inout(),dbg_reg.inout()); 
    }
#else
    TPROCESS->Initialise("","","","");
    int no_users=cl->get_int_value_for("-u");
    if(no_users<1)no_users=1;
    int nb_sametime = cl->get_int_value_for("-b");//maximum number of users to kick off at once
    if(nb_sametime<1) nb_sametime=no_users;//default to all users requested
    int interval=cl->get_int_value_for("-i");//interval between batches
    if(interval<1)interval=0;//default to no interval
    interval*=1000;//convert seconds to milliseconds

    string FullTestName = cl->get_value_for("-f");
    if(FullTestName=="")
        throw string("No Tof file given for Execution");
    int nbLoop = no_users / nb_sametime ;//how many batch loops
    int reste = no_users % nb_sametime  ;// and one for the remainder
    int iNbThread = 0 ;
    for (int k=0; k<nbLoop; k++)
    {
        for(int j=0;j<nb_sametime;j++)
        {
            int status = tc->AttachThread(FullTestName.c_str());
            Cmdbthread* tp = tc->get_threadp(iNbThread);
            iNbThread ++ ;
            tp->wake_thread(CW_FILE, FullTestName.c_str());
        }
        rwSleep(interval);
    }
    for(int j=0;j<reste;j++)
    {
        int status = tc->AttachThread(FullTestName.c_str());
        Cmdbthread* tp = tc->get_threadp(iNbThread);
        iNbThread ++ ;
        tp->wake_thread(CW_FILE, FullTestName.c_str());
    }
#endif
    // Need to check for completion of top level thread executing on the top
    // level Test Manager ... when this is complete shutdown the Test manager normally.
#if defined(CYRANO_TEST_MANAGER) || defined(CYRANO_TP_STANDALONE)
    blocking_semaphore.lock();
    //Wait for bloacking_semaphore to be acquired again OR for all threads to finish
    //loop at ~10 second intervals and check both conditions
    //blocking_semaphore acquired means that stop has been called (abnormal termination)
    //all threads finished means test has completed normally
    while(true)
    {
		// While the test is running, record the number of virtual users
		// every 10 seconds for analysis later.
		try
		{
			TestLevel_ptr TL = TPROCESS->GetTestLevel();
			int nVUsers = TL->NoChildThreads(true);
			CECVULogOutput vuLog(Repository->GetTimeOffset(), nVUsers);

			CORBA::Object_var testrunObj = TPROCESS->GetTestRunAsObject();
			CyrStTestRun_ptr TR=CyrStTestRun::_narrow(testrunObj);
			TR->AddTestOutput(CECVULogOutput::OUTPUT_TYPE, vuLog.ToString().c_str());
		}
		catch(...)
		{
		}

        try
        {
            int bret=blocking_semaphore.trylock();
            if(bret==1)break;//semaphore acquired
        }

        catch(...)
        {
            break;//better shut down now without waiting
        }

        bool bret=TPROCESS->WaitForCompletion();
        if(bret==true)break;//if Threads have completed then break (normal shutdown)
    }

#else
// If a Test Executer then wait for this semaphore which should be set
    // on the call of Stop which will be called when a test manager tries to close
    // down all it's child processes
    blocking_semaphore.lock();

    //------------------------------------------
    // To put the stat file back to the server
    //------------------------------------------
	LocalResultFiles_t localResultFiles;
 
    string FullName = CyrGetStatFileName(info.GetIPAddressAsString().c_str(), true);
    string Name = CyrGetStatFileName(info.GetIPAddressAsString().c_str(), false);

    string FullURLName = FullName.substr(0, FullName.find_last_of(".")) + ".urls";
    string URLName = Name.substr(0, Name.find_last_of(".")) + ".urls";
 
    localResultFiles[Name] = FullName;
    localResultFiles[URLName] = FullURLName;
 
    Repository->SetLocalResultsFile(localResultFiles, true);
    //------------------------------------------


    int report_count=0;
    while(true)
    {
        report_count++;
        try
        {
            int bret=blocking_semaphore.trylock();
            if(bret==1)
                break;//semaphore acquired
        }

        catch(...)
        {
            break;//better shut down now without waiting
        }

        if(report_count>12)
        {
            bool i_should_continue=TPROCESS->ReportOnThreads();
            if(!i_should_continue)
            {
                logr->AddTrace(SHUTDOWN,ERRORLEVEL,"Closing down due to too many thread aborted" );
                Repository->LogErrMessage("Closing down due to too many thread aborted",0);
                break;
            }
            report_count=0;
        }
        CyrSleep(5000);//sleep 5 seconds
    }

#endif
    //
    //    All done - perform any cleanup required
    //
    sqlr_cleanup();
    delete rule;
    return 0;
}


void DeleteStatisticsFiles(const string & prefixStatFileName, int maxthreads)
{
    int iThreadId;
    int iMaxThread = maxthreads/10 + maxthreads%10;
    char szBuf[40];

    // Try to remove an eventual concatenated stat file with same Pid
    sprintf(szBuf, "%s.stat", prefixStatFileName.c_str());
    remove(szBuf);

    // Try to remove eventual stat files with same Pid
    for (iThreadId = 0; iThreadId < iMaxThread; iThreadId++)
    {
        sprintf(szBuf, "%s-%d.stat", prefixStatFileName.c_str(), iThreadId);

        remove(szBuf);
    }
}
