/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TMCORBAREPOSITORY_H_
#define _TMCORBAREPOSITORY_H_
///////////////////////////////////////////////////////////////////////////////
// @doc     TMCORBARepository
// @module  TMCORBARepository |
//
///////////////////////////////////////////////////////////////////////////////
#include "CORBARepository.h"
#include "mdbexcept.h"
extern CORBAServer*    pCServer;

class TMCORBARepository:public CORBARepository
{

public:

    TMCORBARepository(CORBAServer* corba,
                      const string& datanode,
                      const string& user,
                      const string& pass,
                      const string& node,
                      int pid,
                      const string& exename)
:CORBARepository(corba,datanode,user,pass,pid,exename),m_node(node)
    {}
    ;

    virtual ~TMCORBARepository  ()
    {
        //try to de-register test run, don't error if problem
        try
        {
            pCServer->UnNameObject(CyrStTestRun::CLASS_NAME,GetTestRunString().c_str());    
        }
        catch(...)
        {}

    };

protected:

    virtual int     MakeActualTestRun(const string& TestRunString,const string& TestName)
    {
        string desc="Test Run by " + m_UserName;
        char *c_pid=new char[64];
        (void)sprintf(c_pid,"%d",m_pid);
        if(TestRunString=="")//No parent test runs
        {
            // From the Test Folder get a handle to a test object
            // Always check that the pointer is valid
            m_TestStore = new CECTestStore(m_TestFolder,TestName.c_str());
            string msg="Create Test Run using: "+desc+","+m_node+","+c_pid;
            logr->AddTrace(REPOSITORY,DEBUGLEVEL,msg.c_str());
            m_TestRun = new CECOrbTestRun(m_TestStore ,desc.c_str(), m_node.c_str(),c_pid);
            logr->AddTrace(REPOSITORY,DEBUGLEVEL,"Test Run Created");
        }
        else
        {
            CECOrbTestRun* pImParentTestRun=GetParentTestRunFromString(TestRunString);
            m_TestRun = new CECOrbTestRun(pImParentTestRun,desc.c_str(),m_node.c_str(),c_pid);
        }

        delete c_pid;
        //Register this new testrun with naming service for pick up by testexecuters
        CyrStTestRun_var l_testrun=m_TestRun->GetServant(); 
        pCServer->NameObject(l_testrun,CyrStTestRun::CLASS_NAME,GetTestRunString().c_str());

        return 0;
    }

    virtual CECOrbTestRun*    CreateParentTestRuns(const vector<string>& TestRunNameList,const string& TestRunString)
    {
        if(TestRunNameList.size()<1)
            throw mdbParseException("Tyring to create Parent TestRuns with no parents");

        CECOrbFolder* Parent=m_TestStore;//first Parent is the TestStore
        CECOrbTestRun* newParent=NULL;

        vector<string>::const_iterator i;
        for(i=TestRunNameList.begin();i!=TestRunNameList.end();i++)
        {

            newParent=new CECOrbTestRun(Parent,i->c_str());//assign new parent
            Parent=newParent;
            m_ParentTestRunList.push_back(newParent);//store for later deletion
        }

        return newParent;
    }

    virtual string AppendOwnTestRunString()
    {
        return (TESTRUNSTRING_SEP + m_TestRun->GetName().c_str() );
    }

    string m_node;
};
#endif
