/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_OBJECTTREE_H__B2FDF651_1611_4EDF_A831_970BD8AFB1E9__INCLUDED_)
#define AFX_OBJECTTREE_H__B2FDF651_1611_4EDF_A831_970BD8AFB1E9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ObjectTree.h : header file
//

#include "DataNameCtrls.h"
#include <afxcview.h>
/////////////////////////////////////////////////////////////////////////////
// CObjectTree window
class CObjectTree : public CTreeCtrl
{
	DECLARE_DYNAMIC(CObjectTree)

// Construction
public:
	CObjectTree();

// Attributes
public:

// Operations
public:
	BOOL RefreshTree();

	HTREEITEM AddNewScript(const CString& strExtension);
	HTREEITEM AddNewTest();
	HTREEITEM AddNewProfile(const CString& strExtension);

	CString GetItemPath(HTREEITEM hItem);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CObjectTree)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual DROPEFFECT OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);
	virtual BOOL OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point);
	protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL
	BOOL DeleteFolder(const CString& strFolder);
	CString GetErrorMessage(DWORD dwError);

// Implementation
public:
	virtual ~CObjectTree();
	
	// Generated message map functions
protected:
	//{{AFX_MSG(CObjectTree)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnBeginlabeledit(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnKeydown(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnRclick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnNewFolder();
	afx_msg void OnDelete();
	afx_msg void OnRename();
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
	
	BOOL GetFileItemImage(const CString& strFile,int& nImage,int& nSelImage);

	BOOL LoadProfilesFromDisk();
	BOOL LoadScriptsFromDisk();
	BOOL LoadTestsFromDisk();
	
	BOOL LoadProfilesInPath(const CString& strPath,HTREEITEM hParent);
	BOOL LoadScriptsInPath(const CString& strPath,HTREEITEM hParent);
	BOOL LoadTestsInPath(const CString& strPath,HTREEITEM hParent);

	BOOL RegisterFileExtension(const CString& strFile);
	void SaveBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState);
	void RestoreBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState);

	CImageList m_TreeImages;
	typedef enum{enEmptyFileOverlay,enTestsFold,enSelTestsFold,enScriptsFold,enSelScriptsFold,
				enProfilesFold,enSelProfilesFold,enFolder,enSelFolder,enRepository,enSelRepository} TreeImages;

	HTREEITEM m_hRepositoryRoot;
	HTREEITEM m_hTestFolder;
	HTREEITEM m_hScriptFolder;
	HTREEITEM m_hProfileFolder;
	CMap <CString,LPCTSTR,int,int> m_mapExtToImage;
	CMap <HTREEITEM,HTREEITEM,CString,CString&> m_mapItemToPath;

	HTREEITEM m_hMenuSel;
	ULARGE_INTEGER m_uliFreeDiskSpace;
	CDataNameEdit m_ctlLabelEdit;

/////////////
	class CDropTarget : public COleDropTarget
	{
	public:
		CDropTarget()
		{
		}
		virtual ~CDropTarget()
		{

		}
	
	protected:
		virtual DROPEFFECT OnDragOver( CWnd* pWnd, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point )
		{
			if(pWnd->IsKindOf(RUNTIME_CLASS(CObjectTree)))
				return ((CObjectTree*)pWnd)->OnDragOver(pDataObject,dwKeyState,point);

			return COleDropTarget::OnDragOver(pWnd,pDataObject,dwKeyState,point);
		}

		virtual BOOL OnDrop( CWnd* pWnd, COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point )
		{
			if(pWnd->IsKindOf(RUNTIME_CLASS(CObjectTree)))
				return ((CObjectTree*)pWnd)->OnDrop(pDataObject,dropEffect,point);
			
			return COleDropTarget::OnDrop(pWnd,pDataObject,dropEffect,point);
		}
	};

	CDropTarget m_DropTarget;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_OBJECTTREE_H__B2FDF651_1611_4EDF_A831_970BD8AFB1E9__INCLUDED_)
