// GraphWnd.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "GraphWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphWnd

IMPLEMENT_DYNCREATE(CGraphWnd, CMDIChildWnd)

CGraphWnd::CGraphWnd()
{
	m_pGraph = NULL;
}

CGraphWnd::~CGraphWnd()
{
	delete m_pGraph;
	m_pGraph = NULL;
}

BEGIN_MESSAGE_MAP(CGraphWnd, CMDIChildWnd)
	//{{AFX_MSG_MAP(CGraphWnd)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_SYSCOMMAND()
	ON_WM_ERASEBKGND()
	ON_WM_MDIACTIVATE()
	ON_WM_NCPAINT()
	ON_WM_NCLBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphWnd message handlers
void CGraphWnd::SetGraph(CWnd *pGraph)
{
	m_pGraph = pGraph;
}

void CGraphWnd::OnSize(UINT nType, int cx, int cy) 
{
	CMDIChildWnd::OnSize(nType, cx, cy);
	if(m_pGraph)
		m_pGraph->MoveWindow(0,0,cx,cy);
}


void CGraphWnd::OnDestroy() 
{
	::SendMessage(m_hWndNotify,WMU_MDISTATECHANGED,(WPARAM)m_hWnd,enDestroyed);
	
	CMDIChildWnd::OnDestroy();
		
	if(m_pGraph)
	{
		m_pGraph->DestroyWindow();
		delete m_pGraph;
		m_pGraph = NULL;
	}
}

void CGraphWnd::OnSysCommand(UINT nID, LPARAM lParam) 
{
	UINT nItemID = (nID & 0xFFF0);

	switch (nItemID)
	{
	case SC_MAXIMIZE:
	{
		if(GetWindowLong(m_hWnd,GWL_STYLE) & WS_MINIMIZE)
		{
			MDIRestore();
		}

		CRect rcClient;
		GetParent()->GetClientRect(&rcClient);
		MoveWindow(rcClient);
	}
	return;
	}
	
	CMDIChildWnd::OnSysCommand(nID, lParam);
}

BOOL CGraphWnd::OnEraseBkgnd(CDC* pDC) 
{
	return true;	
//	return CMDIChildWnd::OnEraseBkgnd(pDC);
}

void CGraphWnd::OnMDIActivate(BOOL bActivate, CWnd* pActivateWnd, CWnd* pDeactivateWnd) 
{
	CMDIChildWnd::OnMDIActivate(bActivate, pActivateWnd, pDeactivateWnd);
	
	::SendMessage(m_hWndNotify,WMU_MDISTATECHANGED,(WPARAM)m_hWnd,(bActivate ? enActive:enInactive));
	if(bActivate)
		SendMessage(WM_NCPAINT);
}

void CGraphWnd::OnNcPaint() 
{
	CMDIChildWnd::OnNcPaint();

	if(GetMDIFrame()->MDIGetActive() == this)
	{
		CRect rcWnd;
		GetWindowRect(&rcWnd);
		rcWnd.OffsetRect(-rcWnd.left,-rcWnd.top);
	
		//Redraw the caption in as active
		
		BOOL bMinimized = (GetWindowLong(m_hWnd,GWL_STYLE) & WS_MINIMIZE);
		
		CRect rcCaption(GetSystemMetrics(SM_CXFRAME),
						GetSystemMetrics(SM_CYFRAME),
						rcWnd.right-(GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSIZE)*3)+(bMinimized ? 3:2),
						GetSystemMetrics(SM_CYFRAME) + GetSystemMetrics(SM_CYCAPTION)-(bMinimized ? 2:1));

		CWindowDC dc(this);
		DrawCaption(m_hWnd,(HDC)dc,&rcCaption,DC_ACTIVE|DC_TEXT);

		//Fill in around the system buttons
		CRect rc(rcCaption.right,
				 rcCaption.top,
				 rcWnd.right-GetSystemMetrics(SM_CXFRAME),
				 rcCaption.top + (bMinimized ? 1:2));
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.bottom-2;
		rc.bottom = rcCaption.bottom;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.top;
		rc.left = rc.right - 2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));
		
		rc.right = rcWnd.right-GetSystemMetrics(SM_CXFRAME)-GetSystemMetrics(SM_CXSIZE);
		rc.left = rc.right-2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		//Fill in around the icon
		rc.SetRect(rcWnd.left+GetSystemMetrics(SM_CXFRAME),
				 rcCaption.top,
				 rcWnd.left+GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSMICON)+1,
				 rcCaption.top + (bMinimized ? 2:3));
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.bottom-4;
		rc.bottom = rcCaption.bottom;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.top;
		rc.right = rc.left + (bMinimized ? 2:3);
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.left = rcWnd.left+GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSMICON)+(bMinimized ? 0:1);
		rc.right = rc.left + 2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));
	}
}

void CGraphWnd::OnNcLButtonDown(UINT nHitTest, CPoint point) 
{
	PostMessage(WM_NCPAINT);
	CMDIChildWnd::OnNcLButtonDown(nHitTest, point);
}
