/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ResultsDlg.h"
#include "TestFile.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"
#include "XAuditViewer.h"
#include "DeleteRunDlg.h"
#include "OstaDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog


CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CResultsDlg::IDD, pParent),
	m_strAuditLogName("Test Audit Log"),
	m_strHistoryLogName("Test History Log"),
	m_strReportLogName("Test Report Log"),
	m_strSummaryName("Test Summary")
{
	//{{AFX_DATA_INIT(CResultsDlg)
	//}}AFX_DATA_INIT
	m_pFrame = NULL;
	m_pTestFile = NULL;
	m_nLastGraphAdded=0;
	m_pCurrentGraph =NULL;
}

CResultsDlg::~CResultsDlg()
{
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDragDropDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	DDX_Control(pDX, IDC_GRAPH_TYPES, m_ctlGraphs);
	DDX_Control(pDX, IDC_RUNS, m_ctlRuns);
	DDX_Control(pDX, IDC_GRAPHS_CAPTION, m_ctlGraphsCaption);
	DDX_Control(pDX, IDC_RUNS_CAPTION, m_ctlRunsCaption);
	DDX_Control(pDX, IDC_DEF_OPERATION, m_ctlDefOperation);
	DDX_Control(pDX, IDC_LIST_FRAME, m_ctlListFrame);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDragDropDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_DEF_OPERATION, OnDefaultOperation)
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_DELETE_RUNS, OnDeleteRuns)
	ON_CBN_SELCHANGE(IDC_GRAPH_TYPES, OnSelchangeGraphTypes)
	ON_CBN_SELCHANGE(IDC_RUNS, OnSelchangeRuns)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_CASCADE, OnCascade)
	ON_COMMAND(ID_TILEHORIZONTALLY, OnTilehorizontally)
	ON_COMMAND(ID_TILEVERTICALLY, OnTilevertically)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(IDC_DEF_OPERATION,OnUpdateDefOperation)
	ON_WM_ERASEBKGND()
	ON_MESSAGE(WMU_MDISTATECHANGED,OnMDIStateChanged)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDragDropDialog::OnInitDialog();

	m_pFrame = new CMDIFrameWnd;
	m_pFrame->LoadFrame(IDR_MAINFRAME,FWS_ADDTOTITLE|WS_CHILD|WS_VISIBLE,this);
	m_pFrame->UpdateWindow();

	m_ctlRuns.Refresh();
		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDragDropDialog::OnSize(nType, cx, cy);
	
	if(!m_ctlRuns.GetSafeHwnd() || !m_ctlGraphs.GetSafeHwnd())
		return;

	int nWidth = cx;
	const int nSeparator = 5;

	//Get size of runs box
	CRect rcRuns;
	m_ctlRuns.GetWindowRect(&rcRuns);
	ScreenToClient(&rcRuns);

	//Fit Frame around 
	m_ctlListFrame.MoveWindow(0,0,cx,rcRuns.bottom+4);

	//Position Def Operation button to right of window
	CRect rcDefOp(0,0,0,0);
	m_ctlDefOperation.GetWindowRect(&rcDefOp);
	ScreenToClient(&rcDefOp);
	m_ctlDefOperation.SetWindowPos(NULL,cx-(rcDefOp.Width()+nSeparator),4*nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);
				
	int nAvailableWidth = cx - (rcDefOp.Width()+2*nSeparator);
		
	//Position captions
	m_ctlRunsCaption.SetWindowPos(NULL,nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);
	m_ctlGraphsCaption.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);

	//Get size of caption
	CRect rcGraphCaption;
	m_ctlGraphsCaption.GetWindowRect(&rcGraphCaption);
	ScreenToClient(&rcGraphCaption);

	//Position List boxes
	m_ctlRuns.SetWindowPos(NULL,nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);
	m_ctlGraphs.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);

	CRect rcFrame;
	m_ctlListFrame.GetWindowRect(&rcFrame);
	ScreenToClient(&rcFrame);

	//Position chart below frame
	if(m_pFrame)
		m_pFrame->SetWindowPos(NULL,0,rcFrame.bottom+(nSeparator/2),cx,cy-rcFrame.bottom-(nSeparator/2),SWP_NOZORDER);
}

void CResultsDlg::OnDefaultOperation() 
{
	OpenGraph OG;
	if(GetCurrentGraph(OG))
	{
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CPluginManager *pPluginManager = pApp->GetPluginManager();
		
		CTestResultsPlugin plug(OG.strPlugin, pPluginManager);
		plug.LoadRun(OG.strRun, m_pTestFile);
		plug.StartDefaultOperation(OG.pGraphWnd->GetGraph());
	}
}

void CResultsDlg::OnSelchangeGraphTypes() 
{
	int nCurSel = m_ctlGraphs.GetCurSel();
	if(nCurSel == LB_ERR)
		return;

	int nIndex = m_ctlGraphs.GetItemData(nCurSel);

	for(int nGraph=0;nGraph<m_arGraphs.GetSize();nGraph++)
	{
		GraphType &GT = m_arGraphs[nGraph];
		if(GT.nGraphIndex != nIndex)
			continue;

		OpenGraph OG;
		OG.strName = GT.strGraphTitle;
		OG.strPlugin = GT.strPlugin;
		OG.bHasDefOperation = false;
		if(!m_ctlRuns.GetCurrentRun(OG.strRun))
			return;
		CString strRunName(OG.strRun);
		strRunName.TrimRight("\\");
		int nSlashPos = strRunName.ReverseFind('\\');
		if(nSlashPos != -1)
			strRunName = strRunName.Mid(nSlashPos+1);

		CRect rcChild;
		OG.pGraphWnd = AddNewChild(GT.strGraphTitle + ":- " + strRunName);
		OG.pGraphWnd->GetClientRect(rcChild);

		if(GT.strPlugin.IsEmpty())
		{
			if(GT.strGraphTitle == m_strAuditLogName ||
			   GT.strGraphTitle == m_strHistoryLogName||
			   GT.strGraphTitle == m_strReportLogName)
			{
				CXAuditViewer *pAudit = new CXAuditViewer;
				pAudit->Create(NULL,WS_CHILD|WS_VISIBLE,rcChild,OG.pGraphWnd,0);
				pAudit->SetRunFolder(OG.strRun);
				
				int nType = 0;
				if(GT.strGraphTitle == m_strHistoryLogName)
					nType = 1;
				else
				if(GT.strGraphTitle == m_strReportLogName)
					nType = 2;
				
				pAudit->SetLogType(nType);
				OG.pGraphWnd->SetGraph(pAudit);
			}
			else
			if(GT.strGraphTitle == m_strSummaryName)
			{
				CFile f;
				f.Open(OG.strRun + strSummaryFile,CFile::modeRead);
				int nLength = f.GetLength();
				char *szBuffer = new char[nLength+1];
				f.Read(szBuffer,nLength);
				szBuffer[nLength] = NULL;
				f.Close();

				CEdit *pEdit = new CEdit;
				pEdit->Create(ES_READONLY|ES_MULTILINE|WS_VSCROLL|WS_HSCROLL|WS_CHILD|WS_VISIBLE|WS_BORDER,rcChild,OG.pGraphWnd,0);
				pEdit->SetWindowText(szBuffer);
				pEdit->SetFont(GetFont());
				delete []szBuffer;
				OG.pGraphWnd->SetGraph(pEdit);
			}
		}
		else
		{
			CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
			CPluginManager *pPluginManager = pApp->GetPluginManager();
			
			CTestResultsPlugin plug(GT.strPlugin, pPluginManager);

			CWnd *pGraph = plug.DisplayGraph(GT.strGraphTitle,OG.pGraphWnd,rcChild);
			if(!pGraph)
			{
				OG.pGraphWnd->MDIDestroy();
				return;
			}

			OG.pGraphWnd->SetGraph(pGraph);
			OG.bHasDefOperation = plug.GetDefaultOperationName(pGraph,OG.strDefOpName);
		}

		m_arOpenGraphs.Add(OG);
		OnMDIStateChanged((WPARAM)OG.pGraphWnd->GetSafeHwnd(),CGraphWnd::enActive);
		OG.pGraphWnd->InitialUpdateFrame(NULL,TRUE);
		OG.pGraphWnd->RedrawWindow(NULL,NULL,RDW_FRAME|RDW_INVALIDATE);
		UpdateDialogControls(this,TRUE);

		if(m_arOpenGraphs.GetSize() >= 9)
		{
			m_ctlGraphs.EnableWindow(FALSE);
			m_ctlRuns.EnableWindow(FALSE);
		}
		
		return;
	}
}

void CResultsDlg::SetTestFile(CTestFile* pFile)
{
	m_pTestFile = pFile;

	CString strTestFile(m_pTestFile->GetTestFilePath());
	int nPos = strTestFile.ReverseFind('\\');
	
	if(nPos!=-1)
	{
		m_strTestFolder = strTestFile.Left(nPos+1);
		m_ctlRuns.SetTestFolder(m_strTestFolder);
	}
}

BOOL CResultsDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_F5:
			m_ctlRuns.Refresh();
		break;
		}
	break;
	}
	
	return CDragDropDialog::PreTranslateMessage(pMsg);
}

void CResultsDlg::OnUpdateDefOperation(CCmdUI *pCmdUI)
{
	OpenGraph OG;
	if(GetCurrentGraph(OG))
		pCmdUI->Enable(OG.bHasDefOperation);
	else
		pCmdUI->Enable(FALSE);
}

void CResultsDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDragDropDialog::OnShowWindow(bShow, nStatus);
	if(bShow)
	{
		m_ctlRuns.Refresh();
		m_ctlRuns.ShowDropDown();
	}
}

void CResultsDlg::OnDeleteRuns() 
{
	CDeleteRunDlg dlg(m_strTestFolder);
	if(dlg.DoModal()==IDOK)
		m_ctlRuns.Refresh();
}

void CResultsDlg::OnSelchangeRuns() 
{
	CString strRun;
	if(!m_ctlRuns.GetCurrentRun(strRun))
		return;
	strRun.TrimRight('\\');

	//Load the selected run
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CPluginManager *pPluginManager = pApp->GetPluginManager();

	m_arGraphs.RemoveAll();
	m_nLastGraphAdded = 0;

	GraphType GT;
	GT.strPlugin.Empty();
	CFileStatus fs;

	if(CFile::GetStatus(strRun + "\\" + strSummaryFile,fs))
	{
		GT.strGraphTitle = m_strSummaryName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}
	
	if(CFile::GetStatus(strRun + "\\TestLog.txt",fs))
	{
		GT.strGraphTitle = m_strAuditLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}

	if(CFile::GetStatus(strRun + "\\TestRep.txt",fs))
	{
		GT.strGraphTitle = m_strReportLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}

	if(CFile::GetStatus(strRun + "\\TestHis.txt",fs))
	{
		GT.strGraphTitle = m_strHistoryLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}
	
	CStringArray arResultsPlugIns;
	pPluginManager->ListPluginsByType(TEST_RESULTS_PLUGIN, arResultsPlugIns);
	
	for (int nPlugIn = 0; nPlugIn<arResultsPlugIns.GetSize();nPlugIn++)
	{
		CTestResultsPlugin plug(arResultsPlugIns[nPlugIn], pPluginManager);
		plug.LoadRun(strRun, m_pTestFile);

		//Get the graphs which are now available
		CStringArray arstrTypes;
		plug.GetAvailableGraphs(arstrTypes);

		GT.strPlugin = arResultsPlugIns[nPlugIn];
		for(int nNdx=0;nNdx<arstrTypes.GetSize();nNdx++)
		{
			GT.strGraphTitle = arstrTypes[nNdx];
			GT.nGraphIndex = m_nLastGraphAdded++;
			m_arGraphs.Add(GT);
		}
	}

	m_ctlGraphs.SetRedraw(FALSE);
	m_ctlGraphs.ResetContent();

	for(int nNdx=0;nNdx<m_arGraphs.GetSize();nNdx++)
	{
		int nItem = m_ctlGraphs.AddString(m_arGraphs[nNdx].strGraphTitle);
		m_ctlGraphs.SetItemData(nItem,m_arGraphs[nNdx].nGraphIndex);
	}
	
	m_ctlGraphs.SetRedraw();
	m_ctlGraphs.ShowDropDown();
}

BOOL CResultsDlg::OnEraseBkgnd(CDC *pDC)
{
	return CDialog::OnEraseBkgnd(pDC);
}

BOOL CResultsDlg::GetCurrentGraph(OpenGraph &OG)
{
	for(int nNdx=0;nNdx<m_arOpenGraphs.GetSize();nNdx++)
	{
		if(m_arOpenGraphs[nNdx].pGraphWnd == m_pCurrentGraph)
		{
			OG = m_arOpenGraphs[nNdx];
			return TRUE;
		}
	}
	return false;
}
	
void CResultsDlg::OnOK() 
{
}

void CResultsDlg::OnCancel() 
{
}

CGraphWnd *CResultsDlg::AddNewChild(const CString& strTitle)
{
	HINSTANCE hInst = AfxGetResourceHandle();
	HMENU hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));
	HACCEL hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));

	CRuntimeClass *pClass = RUNTIME_CLASS(CGraphWnd);
	ASSERT(pClass != NULL);
	CGraphWnd *pFrame = (CGraphWnd*) pClass->CreateObject();
	ASSERT_KINDOF(CGraphWnd, pFrame);

	// load the frame
	CCreateContext context;
	context.m_pCurrentFrame = m_pFrame;

	if(!pFrame->Create(NULL,strTitle,WS_CHILD|WS_VISIBLE|WS_OVERLAPPEDWINDOW,CRect(0,0,0,0),m_pFrame,&context))
	{
		TRACE0("Couldn't load frame window.\n");
		delete (LPVOID)pFrame;
		return NULL;
	}
	pFrame->SetHandles(hMDIMenu, hMDIAccel);
	pFrame->SetNotifyWindow(m_hWnd);

	return pFrame;
}

LRESULT CResultsDlg::OnMDIStateChanged(WPARAM wParam,LPARAM lParam)
{
	HWND hWnd = (HWND)wParam;
	
	for(int nNdx=0;nNdx<m_arOpenGraphs.GetSize();nNdx++)
	{
		if(m_arOpenGraphs[nNdx].pGraphWnd->m_hWnd == hWnd)
		{
			switch(lParam)
			{
			case CGraphWnd::enActive:
				m_pCurrentGraph = m_arOpenGraphs[nNdx].pGraphWnd;
	
				if(!m_arOpenGraphs[nNdx].strDefOpName.IsEmpty())
						m_ctlDefOperation.SetWindowText(m_arOpenGraphs[nNdx].strDefOpName);

				UpdateDialogControls(this,TRUE);
			return 0;
			case CGraphWnd::enInactive:
			return 0;
			case CGraphWnd::enDestroyed:
				CWnd *pWnd = m_arOpenGraphs[nNdx].pGraphWnd;
				m_arOpenGraphs.RemoveAt(nNdx);
				if(m_pCurrentGraph == pWnd)
					m_pCurrentGraph = NULL;
		
				m_ctlGraphs.EnableWindow(TRUE);
				m_ctlRuns.EnableWindow(TRUE);

				UpdateDialogControls(this,TRUE);
			return 0;
			}
		}
	}
	
	return 0;
}

void CResultsDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	//This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_RESULTS_CONTEXT_MENU));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
}

void CResultsDlg::OnCascade() 
{
	m_pFrame->MDICascade();
}

void CResultsDlg::OnTilehorizontally() 
{
	m_pFrame->MDITile(MDITILE_HORIZONTAL);
}

void CResultsDlg::OnTilevertically() 
{
	m_pFrame->MDITile(MDITILE_VERTICAL);
}
