/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ScriptDefData.cpp: implementation of the CScriptDefData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ScriptDefData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptDefData::CScriptDefData()
{
	m_nNumIterations = 1;
	m_nMaxIterDelay = 1;
	m_nMinIterDelay = 0;
	m_nFixedIterDelay = 0;
	m_bConstantDelay = true;
}

CScriptDefData::~CScriptDefData()
{

}

BOOL CScriptDefData::SaveToBuffer(LPBYTE pData,DWORD &dwBufferSize)
{
	const int nVersion=0;
	
	//Calculate required buffer size
	DWORD dwRequiredSize = sizeof(int);	//version
	dwRequiredSize += sizeof(int);		//num iterations
	
	dwRequiredSize += sizeof(BOOL);	//Fixed or const delay

	if(m_bConstantDelay)
		dwRequiredSize += sizeof(int);	//fixed iter delay
	else
	{
		dwRequiredSize += sizeof(int);	//Min iter delay
		dwRequiredSize += sizeof(int);	//max iter delay
	}	
	dwRequiredSize += sizeof(BOOL);		//gen page timers
	
	if(dwBufferSize < dwRequiredSize)
	{
		dwBufferSize = dwRequiredSize;		//buffer is too small
		return false;
	}
	
	LPBYTE pDataPos = pData;
	
	memcpy(pDataPos,&nVersion,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nNumIterations,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_bConstantDelay,sizeof(BOOL));
	pDataPos += sizeof(BOOL);

	if(m_bConstantDelay)
	{
		memcpy(pDataPos,&m_nFixedIterDelay,sizeof(int));
		pDataPos+=sizeof(int);
	}
	else
	{
		memcpy(pDataPos,&m_nMaxIterDelay,sizeof(int));
		pDataPos+=sizeof(int);

		memcpy(pDataPos,&m_nMinIterDelay,sizeof(int));
		pDataPos+=sizeof(int);
	}

	return true;
}

BOOL CScriptDefData::LoadFromBuffer(LPBYTE pData)
{
	if(!pData)
		return false;

	LPBYTE pDataPos = pData;

	int nVersion;
	memcpy(&nVersion,pDataPos,sizeof(int));
	pDataPos+=sizeof(int);

	if(nVersion == 0)
	{
		memcpy(&m_nNumIterations,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);

		memcpy(&m_bConstantDelay,pDataPos,sizeof(BOOL));
		pDataPos += sizeof(BOOL);

		if(m_bConstantDelay)
		{
			memcpy(&m_nFixedIterDelay,pDataPos,sizeof(int));
			pDataPos+=sizeof(int);
		}
		else
		{
			memcpy(&m_nMaxIterDelay,pDataPos,sizeof(int));
			pDataPos+=sizeof(int);

			memcpy(&m_nMinIterDelay,pDataPos,sizeof(int));
			pDataPos+=sizeof(int);
		}
	}
	return true;
}