/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ScriptDefDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ScriptDefDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScriptDefDlg dialog


CScriptDefDlg::CScriptDefDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CScriptDefDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CScriptDefDlg)
	m_nNumIterations = 1;
	m_nMaxIterDelay = 1;
	m_nMinIterDelay = 0;
	m_nFixedIterDelay = 0;
	//}}AFX_DATA_INIT
	m_bConstantDelay = true;
}


void CScriptDefDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScriptDefDlg)
	DDX_Control(pDX, IDC_FIXED_ITER_DELAY_SPIN, m_ctlFixedIterDelaySpin);
	DDX_Control(pDX, IDC_MIN_DELAY_SPIN, m_ctlMinDelaySpin);
	DDX_Control(pDX, IDC_MAX_DELAY_SPIN, m_ctlMaxDelaySpin);
	DDX_Control(pDX, IDC_ITERATIONS_SPIN, m_ctlIterationSpin);
	DDX_Text(pDX, IDC_ITERATIONS, m_nNumIterations);
	DDX_Text(pDX, IDC_MAX_ITER_DELAY, m_nMaxIterDelay);
	DDX_Text(pDX, IDC_MIN_ITER_DELAY, m_nMinIterDelay);
	DDX_Text(pDX, IDC_FIXED_ITER_DELAY, m_nFixedIterDelay);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScriptDefDlg, CDialog)
	//{{AFX_MSG_MAP(CScriptDefDlg)
	ON_BN_CLICKED(IDC_RANDOM_DELAY, OnRandomDelay)
	ON_BN_CLICKED(IDC_CONSTANT_DELAY, OnConstantDelay)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptDefDlg message handlers

BOOL CScriptDefDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

    const int nNumAccel=4;
    UDACCEL Accel[nNumAccel];
    Accel[0].nSec=1;
    Accel[0].nInc=1;
    Accel[1].nSec=2;
    Accel[1].nInc=10;
    Accel[2].nSec=3;
    Accel[2].nInc=50;
    Accel[3].nSec=4;
    Accel[3].nInc=200;

	m_ctlFixedIterDelaySpin.SetRange(0,UD_MAXVAL);
    m_ctlFixedIterDelaySpin.SetAccel(nNumAccel,Accel);
    m_ctlFixedIterDelaySpin.SetBuddy(GetDlgItem(IDC_FIXED_ITER_DELAY));

	m_ctlMinDelaySpin.SetRange(0,UD_MAXVAL);
    m_ctlMinDelaySpin.SetAccel(nNumAccel,Accel);
	m_ctlMinDelaySpin.SetBuddy(GetDlgItem(IDC_MIN_ITER_DELAY));

	m_ctlMaxDelaySpin.SetRange(1,UD_MAXVAL);
    m_ctlMaxDelaySpin.SetAccel(nNumAccel,Accel);
    m_ctlMaxDelaySpin.SetBuddy(GetDlgItem(IDC_MAX_ITER_DELAY));

	m_ctlIterationSpin.SetRange(1,UD_MAXVAL);
    m_ctlIterationSpin.SetAccel(nNumAccel,Accel);
    m_ctlIterationSpin.SetBuddy(GetDlgItem(IDC_ITERATIONS));

	if(m_bConstantDelay)
	{
		((CButton*)GetDlgItem(IDC_CONSTANT_DELAY))->SetCheck(1);
		OnConstantDelay();
	}
	else
	{
		((CButton*)GetDlgItem(IDC_RANDOM_DELAY))->SetCheck(1);
		OnRandomDelay();
	}

	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CScriptDefDlg::OnOK() 
{
	UpdateData(TRUE);

	m_ScriptData.SetNumIterations(m_nNumIterations);
	m_ScriptData.SetMaxIterDelay(m_nMaxIterDelay);
	m_ScriptData.SetMinIterDelay(m_nMinIterDelay);
	m_ScriptData.SetFixedIterDelay(m_nFixedIterDelay);
	m_ScriptData.SetUseFixedIterDelay(m_bConstantDelay);

	//TODO: validation 
	CDialog::OnOK();
}

void CScriptDefDlg::LoadFromBuffer(LPBYTE pData)
{
	m_ScriptData.LoadFromBuffer(pData);
	
	m_nNumIterations = m_ScriptData.GetNumIterations();
	m_nMaxIterDelay = m_ScriptData.GetMaxIterDelay();
	m_nMinIterDelay = m_ScriptData.GetMinIterDelay();
	m_nFixedIterDelay = m_ScriptData.GetFixedIterDelay();
	m_bConstantDelay = m_ScriptData.GetUseFixedIterDelay();
}

BOOL CScriptDefDlg::SaveToBuffer(LPBYTE pData,DWORD& dwBufferSize)
{
	return m_ScriptData.SaveToBuffer(pData,dwBufferSize);
}

void CScriptDefDlg::OnRandomDelay() 
{
	GetDlgItem(IDC_MAX_ITER_DELAY)->EnableWindow(TRUE);
	GetDlgItem(IDC_MIN_ITER_DELAY)->EnableWindow(TRUE);
	GetDlgItem(IDC_FIXED_ITER_DELAY)->EnableWindow(FALSE);
	m_bConstantDelay = false;
}

void CScriptDefDlg::OnConstantDelay() 
{
	GetDlgItem(IDC_MAX_ITER_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_MIN_ITER_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_FIXED_ITER_DELAY)->EnableWindow(TRUE);
	m_bConstantDelay = true;
}
