/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "TestCtrl.h"
#include "OstaDefs.h"
#include "TaskGroupDefDlg.h"
#include "TaskGroupDefData.h"
#include "ScriptDefDlg.h"
#include "HostDefDlg.h"
#include "TestPluginDoc.h"
#include "RegFunctions.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_REFRESH_LIST 1
/////////////////////////////////////////////////////////////////////////////
// CTestCtrl

CTestCtrl::CTestCtrl():
	m_nMaxTasks(17),
	m_nMaxGroups(23),
	m_pTestFile(NULL),
	m_colMissing(RGB(255,0,0)),
	m_colNormal(RGB(255,255,255)),
	m_pDoc(NULL),
	m_ptContextClick(0,0)
{
	m_ClipFormat = RegisterClipboardFormat(strClipboardFormat);
}

CTestCtrl::~CTestCtrl()
{
}


BEGIN_MESSAGE_MAP(CTestCtrl, CECGridCtrl)
	//{{AFX_MSG_MAP(CTestCtrl)
	ON_WM_LBUTTONDOWN()
	ON_WM_TIMER()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_DUPLICATETASKGROUP, OnDuplicatetaskgroup)
	//}}AFX_MSG_MAP
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclkTestList)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestCtrl message handlers
BOOL CTestCtrl::HitTest(CPoint pt,int& nRow,int& nCol)
{
	CCellID cell = GetCellFromPt(pt);
	if(cell.IsValid())
	{
		nRow = cell.row;
		nCol = cell.col;
		return true;
	}
	return false;
}

void CTestCtrl::PreSubclassWindow() 
{
	CECGridCtrl::PreSubclassWindow();

	m_DropTarget.Revoke();	//don't let grid handle drag and drop
	EnableDragAndDrop(FALSE);

	m_ListImages.Create(IDB_CONFIG_IMAGES,16,1,RGB(192,192,192));
	SetImageList(&m_ListImages);

	Refresh();

	SetTimer(IDT_REFRESH_LIST,2000,NULL);
}

void CTestCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int nRow,nCol;
	if(HitTest(point,nRow,nCol))
	{
		if(nRow && (nCol == enNameCol) && (m_pTestFile->GetNumTaskGroups()>nRow-1))
		{
			DWORD dwDataSize=0;
			LPBYTE pData = m_pTestFile->WriteTaskGroupToBuffer(nRow-1,dwDataSize);

			int nDataSize = ClipboardData::GetSize() + dwDataSize;
		
			HGLOBAL hData = GlobalAlloc(GMEM_FIXED|GMEM_ZEROINIT,nDataSize);
			ClipboardData *pClipData = (ClipboardData*)GlobalLock(hData);
			pClipData->nDataType = ClipboardData::enTaskGroup;
			memcpy(pClipData->GetData(),pData,dwDataSize);
			GlobalUnlock(hData);

			delete []pData;

			COleDataSource DataSource;
			DataSource.CacheGlobalData(m_ClipFormat,hData);
			DROPEFFECT DE = DataSource.DoDragDrop();
			if(DE == DROPEFFECT_NONE)
			{
				CECGridCtrl::OnLButtonDown(nFlags, point);
				CECGridCtrl::OnLButtonUp(nFlags, point);
			}
			return;
		}
	}
	
	CECGridCtrl::OnLButtonDown(nFlags, point);
}

void CTestCtrl::SetupList()
{
    //Setup up the columns in the grid
	SetColumnCount(m_nMaxTasks+enFirstTaskCol);
	SetFixedRowCount();
	SetItemText(0,enNameCol,"Task Group");
	AutoSizeColumn(enNameCol);
	SetItemText(0,enHostCol, "Host");
	AutoSizeColumn(enHostCol);
	SetItemText(0,enVUCol, "VUs");
	AutoSizeColumn(enVUCol);

	CString strHeader;
	for(int nCol=0;nCol<m_nMaxTasks;nCol++)
	{
		strHeader.Format("Task %d",nCol+1);
		SetItemText(0,nCol+enFirstTaskCol, strHeader);
		AutoSizeColumn(nCol+enFirstTaskCol);
	}
	SetRowCount(m_nMaxGroups+1);

	for(nCol=0;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
	{
		for(int nRow=1;nRow<=m_nMaxGroups;nRow++)
		{
			DWORD dwState = GetItemState(nRow,nCol);
			SetItemState(nRow,nCol,dwState | GVIS_READONLY);
		}
	}
}

void CTestCtrl::Refresh()
{
	SetRedraw(FALSE);
	DeleteAllItems();
	SetupList();
	LoadTestFile();
	SetRedraw(TRUE);
}

void CTestCtrl::LoadTestFile()
{
	if(!m_pTestFile)
		return;

	CString strTestPath(m_pTestFile->GetTestFilePath());
	int nPos = strTestPath.ReverseFind('\\');
	if(nPos!=-1)
	{
		m_strTestFolder = strTestPath.Left(nPos+1);
		m_strTestName = strTestPath.Mid(nPos+1);
	}
	else
		m_strTestName = strTestPath;

	nPos = m_strTestName.ReverseFind('.');
	if(nPos != -1)
		m_strTestName = m_strTestName.Left(nPos);

	CFileStatus fs;
	int nNumGroups = m_pTestFile->GetNumTaskGroups();
	for(int nGroup=0;nGroup<nNumGroups;nGroup++)
	{
		CString strName = m_pTestFile->GetTaskGroupName(nGroup);
		SetItemText(nGroup+1,enNameCol,strName);
		SetItemImage(nGroup+1,enNameCol,enNameImage);
		AutoSizeColumn(enNameCol);
		
		CString strHost = m_pTestFile->GetTaskGroupHost(nGroup);
		SetItemText(nGroup+1,enHostCol, strHost );
		SetItemImage(nGroup+1,enHostCol,enHostImage);
		AutoSizeColumn(enHostCol);

		int nNumTasks = m_pTestFile->GetNumTasks(nGroup);

		if(nNumTasks)
		{
			switch(m_pTestFile->GetTaskType(nGroup,0))
			{
			case CTestFile::enScript:
			{
				SetItemImage(nGroup+1,enVUCol,enVUImage);

				LPBYTE pData;
				DWORD dwBufSize;
				m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
				CTaskGroupDefData TGData;
				TGData.LoadFromBuffer(pData);
				
				CString strNumVUs;
				strNumVUs.Format("%d",TGData.GetNumVUs());
				SetItemText(nGroup+1,enVUCol,strNumVUs);

				AutoSizeColumn(enVUCol);
			}
			break;
			case CTestFile::enProfile:
			{
				SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
				for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
				{
					SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
				}
			}
			break;
			}
		}

		for(int nTask=0;nTask<nNumTasks;nTask++)
		{
			CString strName = m_pTestFile->GetTaskName(nGroup,nTask);
			SetItemText(nGroup+1,nTask+enFirstTaskCol, strName);
			
			CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
			int nImage = RegisterFileExtension(strPath);
			SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);

			if(!CFile::GetStatus(strPath,fs))
				SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colMissing);
			else
				SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colNormal);

			AutoSizeColumn(nTask+enFirstTaskCol);
		}
	}
}

void CTestCtrl::OnTimer(UINT nIDEvent) 
{
	switch(nIDEvent)
	{
	case IDT_REFRESH_LIST:
	{
		CFileStatus fs;
		int nNumGroups = m_pTestFile->GetNumTaskGroups();
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			int nNumTasks = m_pTestFile->GetNumTasks(nGroup);
			for(int nTask=0;nTask<nNumTasks;nTask++)
			{
				CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
				int nImage = RegisterFileExtension(strPath);
				SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
				
				COLORREF col = GetItemBkColour(nGroup+1,nTask+enFirstTaskCol);
				if(!CFile::GetStatus(strPath,fs))
				{
					if(col != m_colMissing)
					{
						SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colMissing);
						RedrawCell(nGroup+1,nTask+enFirstTaskCol);
					}
				}
				else
				{
					if(col != m_colNormal)
					{
						SetItemBkColour(nGroup+1,nTask+enFirstTaskCol,m_colNormal);
						RedrawCell(nGroup+1,nTask+enFirstTaskCol);
					}
				}
			}
		}
		return;
	}
	break;
	}
	
	CECGridCtrl::OnTimer(nIDEvent);
}

CString CTestCtrl::GetScriptOutputPath()
{
	return m_strTestFolder;
}

void CTestCtrl::GenerateScripts()
{
	CString strOutputPath = GetScriptOutputPath();

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + m_strTestName + ".TSS",CFile::modeCreate|CFile::modeWrite);

		str = GetScriptHeader(m_strTestName + ".TSS");

		//ENVIRONMENT
		str += "Environment\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";

		//DEFINITIONS
		str += "Definitions\n";

		int nNumGroups = m_pTestFile->GetNumTaskGroups();
		CString strCode;
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			CString strGroupName( m_pTestFile->GetTaskGroupName(nGroup));
			switch(m_pTestFile->GetTaskType(nGroup,0))
			{
			case CTestFile::enScript:
				str += "\tTest " + strGroupName + "\n";
				strCode += "\tExecute Test " + strGroupName + "\n";
				
				GenerateTaskGroupScripts(nGroup);
			break;
			case CTestFile::enProfile:
				strTemp.Format("\tThread %s for target %s\n",strGroupName,m_pTestFile->GetTaskTargetType(nGroup,0));
				str += strTemp;

				strTemp.Format("\tExecute Thread %s using script \"%s\", on node \"%s\"\n",strGroupName,m_pTestFile->GetTaskName(nGroup,0),m_pTestFile->GetTaskGroupHost(nGroup));
				strCode += strTemp;
			break;
			}

		}
		str += "\n";
		str += "Code\n";

		str += strCode;

		str += "\n";
		str += "Exit";

		f.WriteString(str);
	}
	catch(...)
	{

	}
}

void CTestCtrl::GenerateTaskGroupScripts(int nGroup)
{
	GenerateTaskGroupTSS(nGroup);
	GenerateTaskGroupSCL(nGroup);
}

void CTestCtrl::GenerateTaskGroupTSS(int nGroup)
{
	CString strGroupName;
	strGroupName.Format("%s_%d",m_strTestName,nGroup+1);

	CString strOutputPath = GetScriptOutputPath();

	LPBYTE pData;
	DWORD dwDataSize;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwDataSize);
	CTaskGroupDefData TGData;
	TGData.LoadFromBuffer(pData);

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + strGroupName + ".TSS",CFile::modeCreate|CFile::modeWrite);
	
		str = GetScriptHeader(strGroupName + ".TSS");

		str += "Environment\n";
		str += "\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";
		str += "Definitions\n";

		str += "\tThread HTTP_INJECTOR For Target " + m_pTestFile->GetTaskTargetType(nGroup,0) + "\n";
		str += "\n";
		
		str += "\tConstant HOST_NAME = \"" + m_pTestFile->GetTaskGroupHost(nGroup) + "\"\n";
		str += "\tConstant SCRIPT_NAME = \"" + strGroupName + "\"\n";

		strTemp.Format("\tConstant VU_RAMP = %d\n",TGData.GetNumVUPerInterval());
		str += strTemp;

		strTemp.Format("\tConstant VU_TODO = %d\n",TGData.GetNumVUs());
		str += strTemp;
		
		strTemp.Format("\tConstant VU_WAIT = %d\n",TGData.GetStartInterval());
		str += strTemp;

		strTemp.Format("\tConstant INITIAL_WAIT = %d\n",TGData.GetStartDelay());
		str += strTemp;
		str += "\n";

		str += "\tInteger VU_LEFT\n";
		str += "\n";
		str += "Code\n";
		str += "\tWait INITIAL_WAIT\n";
		str += "\tSet VU_LEFT = VU_TODO\n";
		str += "\n";
		str += "WHILE-LOOP:\n";
		str += "\tIf (VU_LEFT > VU_RAMP) Then\n";
		str += "\t\tExecute Thread HTTP_INJECTOR Using Script SCRIPT_NAME, on node HOST_NAME, number VU_RAMP\n";
		str += "\t\tSet VU_LEFT = VU_LEFT - VU_RAMP\n";
		str += "\t\tWait VU_WAIT\n";
		str += "\t\tGoto WHILE-LOOP\n";
		str += "\tEndIf\n";
		str += "\n";
		str += "\tIf (VU_LEFT > 0) Then\n";
		str += "\t\tExecute Thread HTTP_INJECTOR Using Script SCRIPT_NAME, on node HOST_NAME, number VU_LEFT\n";
		str += "\tEndIf\n";
		str += "\n";
		str += "Exit";
	
		f.WriteString(str);

		f.Close();
	}
	catch(...)
	{
	}
}

void CTestCtrl::GenerateTaskGroupSCL(int nGroup)
{
	CString strGroupName;
	strGroupName.Format("%s_%d",m_strTestName,nGroup+1);

	CString strOutputPath = GetScriptOutputPath();

	LPBYTE pData;
	DWORD dwDataSize;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwDataSize);
	CTaskGroupDefData TGData;
	TGData.LoadFromBuffer(pData);

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + strGroupName + ".SCL",CFile::modeCreate|CFile::modeWrite);
	
		str = GetScriptHeader(strGroupName + ".SCL");

		str += "Environment\n";
		str += "\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";
		str += "Definitions\n";

		str += "\tInclude	\"GLOBAL_VARIABLES.INC\"\n";
		str += "\n";
		str += "\tConstant	USE_PAGE_TIMERS = " + (TGData.GetUsePageTimers() ? CString("1\n") :CString("0\n")); 
		str += "\tConstant  USER_AGENT = \"DEFAULT\"\n";

		strTemp.Format("\tConstant	HTTP-STAT-LV = \"%c\"\n",TGData.GetStatLevel());
		str += strTemp;
		str += "\n";

		int nMinDelay;
		int nMaxDelay;
		LPBYTE pData;
		DWORD dwDataSize;
		CScriptDefData m_scriptData;
		
		int nNumTasks = m_pTestFile->GetNumTasks(nGroup);
		for(int nTask=0;nTask <nNumTasks;nTask++)
		{
			m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwDataSize);
			m_scriptData.LoadFromBuffer(pData);

			if(m_scriptData.GetUseFixedIterDelay())
				nMaxDelay = nMinDelay = m_scriptData.GetFixedIterDelay();
			else
			{
				nMinDelay = m_scriptData.GetMinIterDelay();
				nMaxDelay = m_scriptData.GetMaxIterDelay();
			}
			
			strTemp.Format("\tInteger	DELAY_%d (%d - %d)\n",nTask+1,nMinDelay,nMaxDelay);
			str += strTemp;
		}
		str += "\n";
		str += "\tInteger 	IC\n";
		str += "\tCharacter	CYR_HTTP-STAT-LV, Thread\n";

		str += "\n";
		str += "Code\n";
	
		str += "\tSet CYR_HTTP-STAT-LV = HTTP-STAT-LV\n";
		str += "\n";

		for(nTask=0;nTask <nNumTasks;nTask++)
		{
			m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwDataSize);
			m_scriptData.LoadFromBuffer(pData);

			strTemp.Format("! Task #%d\n",nTask+1);
			str += strTemp;

			strTemp.Format("\tDo IC = 1, %d\n",m_scriptData.GetNumIterations());
			str += strTemp;

			strTemp.Format("\t\tCall Script \"%s\" [USER_AGENT,USE_PAGE_TIMERS]\n",m_pTestFile->GetTaskName(nGroup,nTask));
			str += strTemp;
			
			strTemp.Format("\t\tGenerate DELAY_%d\n",nTask+1);
			str += strTemp;
			
			strTemp.Format("\t\tWait DELAY_%d\n",nTask+1);
			str += strTemp;

			str += "\tEndDo\n";
			str += "\n";
		}

		str += "Exit";
	
		f.WriteString(str);

		f.Close();
	}
	catch(...)
	{
	}
}

CString CTestCtrl::GetScriptHeader(const CString& strName)
{
	CString str;
	str += "!/////////////////////////////////////////////////////////////////////////\n";
	str += "!//                                                                     //\n";
	str += "!// OpenSTA http://www.opensta.org/                                     //\n";
	str += "!//                                                                     //\n";
	str += "!//  This code has been generated for you as a starting template.       //\n";
	str += "!//  Please refer to the SCL Documentation if you wish to modify it     //\n";
	str += "!//  to provide further functionality to your script.                   //\n";
	str += "!//                                                                     //\n";
	str += "!//  File Name:  "+strName + "\n";
	str += "!//                                                                     //\n";
	str += "!/////////////////////////////////////////////////////////////////////////\n";
	str += "\n";
	return str;
}

void CTestCtrl::DeleteSelection()
{
	int nNumRowsDeleted=0;
	int nNumColsDeleted=0;
	CCellRange cellRange = GetSelectedCellRange();
	
	for(int nRow=cellRange.GetMinRow();nRow<=cellRange.GetMaxRow();nRow++)
	{
		int nGroup = nRow-1-nNumRowsDeleted;
		if(nGroup>=m_pTestFile->GetNumTaskGroups())
			break;

		for(int nCol=cellRange.GetMinCol();nCol<=cellRange.GetMaxCol();nCol++)
		{
			int nTask = nCol-enFirstTaskCol-nNumColsDeleted;
			if(nCol == enHostCol || nCol == enVUCol || (nTask>=m_pTestFile->GetNumTasks(nGroup)))
				continue;

			if(GetCell(nRow,nCol)->IsSelected())
			{
				if(nCol==enNameCol)
				{
					m_pTestFile->DeleteTaskGroup(nGroup);
					nNumRowsDeleted++;
					break;
				}
				else
				{
					m_pTestFile->DeleteItem(nGroup,nTask);
					nNumColsDeleted++;
				}
			}
		}
		nNumColsDeleted=0;
	}

	Refresh();
	m_pDoc->SetModifiedFlag();
}

BOOL CTestCtrl::EditVUs(int nGroup)
{
	DWORD dwBufSize=0;
	LPBYTE pData = NULL;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);

	CTaskGroupDefDlg dlg;
	dlg.LoadFromBuffer(pData);
	if(dlg.DoModal() == IDOK)
	{
		delete []pData;

		dwBufSize = 0;
		dlg.SaveToBuffer(NULL,dwBufSize);
		
		pData = new BYTE[dwBufSize];
		dlg.SaveToBuffer(pData,dwBufSize);
		
		m_pTestFile->SetTaskGroupData(nGroup,pData,dwBufSize);

		CString strNumVUs;
		strNumVUs.Format("%d",dlg.m_nNumVUs);
		SetItemText(nGroup+1,enVUCol,strNumVUs);
		RedrawCell(nGroup+1,enVUCol);

		return true;
	}

	return false;
}

void CTestCtrl::DeleteItem(int nGroup,int nTask)
{
	if(m_pTestFile->GetNumTaskGroups() >= nGroup ||
		m_pTestFile->GetNumTasks(nGroup) >= nTask)
		return;

	m_pTestFile->DeleteItem(nGroup,nTask);

	Refresh();
	m_pDoc->SetModifiedFlag();
}

DROPEFFECT CTestCtrl::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	if(!pDataObject->IsDataAvailable(m_ClipFormat))
		return DROPEFFECT_NONE;

	int nRow,nCol;
	if(!HitTest(point,nRow,nCol))
		return DROPEFFECT_NONE;

	//Can't drop on header row
	if(!nRow)
		return DROPEFFECT_NONE;

	int nGroup = nRow-1;
	int nTask = nCol-enFirstTaskCol;

	HGLOBAL hClipData = pDataObject->GetGlobalData(m_ClipFormat);
	ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);

	int nType = pClipData->nDataType;
	switch(nType)
	{
	case ClipboardData::enTaskGroup:
	{
		GlobalUnlock(hClipData);

		//Can drop on a new line only
		if(m_pTestFile->GetNumTaskGroups()<=nGroup)
			return DROPEFFECT_COPY;
		return DROPEFFECT_NONE;
	}
	case ClipboardData::enScript:
	case ClipboardData::enProfile:
	{
		CString strItem = (char*)pClipData->GetData();
		GlobalUnlock(hClipData);

		//Dropping on the hosts column or VUs column or name column
		if((nCol==enHostCol) || (nCol==enVUCol) || (nCol == enNameCol))
			return DROPEFFECT_NONE;

		//Disallow if the file does not exist or is a directory
		CFileStatus fs;
		if(!CFile::GetStatus(strItem,fs) || (fs.m_attribute & CFile::Attribute::directory))
			return DROPEFFECT_NONE;

		//Dropping on a new line
		if(m_pTestFile->GetNumTaskGroups()<=nGroup)	
			return DROPEFFECT_COPY;
		//Only one item in a group with a profile
		if(nTask>0 && m_pTestFile->GetNumTasks(nGroup) &&
			(m_pTestFile->GetTaskType(nGroup,0)==CTestFile::enProfile))
			return DROPEFFECT_NONE;

		//Disallow drop if dragging profile anywhere but column 1
		if((nType==ClipboardData::enProfile)&&nTask)
			return DROPEFFECT_NONE;

		//Can only have scripts of the same type in a group
		CString strType = m_pTestFile->GetTaskTargetType(nGroup,0);

		int nDotPos = strItem.ReverseFind('.');
		if(strItem.Mid(nDotPos+1).CompareNoCase(strType)!=0)
			return DROPEFFECT_NONE;
	
		return DROPEFFECT_COPY;
	}
	default:
		GlobalUnlock(hClipData);
		return DROPEFFECT_NONE;
	}
}


BOOL CTestCtrl::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	if(pDataObject->IsDataAvailable(m_ClipFormat))
	{
		int nRow,nCol;
		if(!HitTest(point,nRow,nCol))
			return false;

		HGLOBAL hClipData = pDataObject->GetGlobalData(m_ClipFormat);
		ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);

		switch(pClipData->nDataType)
		{
		case ClipboardData::enTaskGroup:
		{
			LPBYTE pTaskGroup = pClipData->GetData();
			m_pTestFile->AddTaskGroupFromBuffer(pTaskGroup);

			Refresh();
			m_pDoc->SetModifiedFlag();
		}
		break;
		case ClipboardData::enScript:
		{
			CString strItem = (char*)pClipData->GetData();
			InsertScript(nRow,nCol,strItem);
			m_pDoc->SetModifiedFlag();
		}
		break;
		case ClipboardData::enProfile:
		{
			CString strItem = (char*)pClipData->GetData();
			InsertProfile(nRow,nCol,strItem);
			m_pDoc->SetModifiedFlag();
		}
		break;
		}
					
		GlobalUnlock(hClipData);
		return true;
	}
	return false;
}

void CTestCtrl::InsertScript(int nItem,int nSubItem,const CString& strItemPath)
{
	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		SetItemText(nGroup+1,enNameCol,m_pTestFile->GetTaskGroupName(nGroup));
		SetItemImage(nGroup+1,enNameCol,enNameImage);
		AutoSizeColumn(enNameCol);

		SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		SetItemImage(nGroup+1,enHostCol,enHostImage);
		AutoSizeColumn(enHostCol);

		//Insert VUs
		SetItemImage(nGroup+1,enVUCol,enVUImage);
		SetItemText(nGroup+1,enVUCol,"1");
		AutoSizeColumn(enVUCol);
	}

	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	if(m_pTestFile->GetNumTasks(nGroup)<=nTask)
		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	}

	SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	AutoSizeColumn(nTask+enFirstTaskCol);
	Invalidate();

	if(!EditScriptTask(nGroup,nTask))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}

void CTestCtrl::InsertProfile(int nItem,int nSubItem,const CString& strItemPath)
{
	//Only one task allowed in a profiles group
	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		SetItemText(nGroup+1,enNameCol,m_pTestFile->GetTaskGroupName(nGroup));
		SetItemImage(nGroup+1,enNameCol,enNameImage);
		AutoSizeColumn(enNameCol);

		SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		SetItemImage(nGroup+1,enHostCol,enHostImage);
		AutoSizeColumn(enHostCol);

		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enProfile,strName,strItemPath,GetTargetType(strItemPath));

		//Disable other columns
		SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
		for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
		{
			SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
		}
	}
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	}

	SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	AutoSizeColumn(nTask+enFirstTaskCol);
	Invalidate();

	if(!EditProfileTask(nGroup))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}

void CTestCtrl::OnDblclkTestList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_GRIDVIEW *pNMGV = (NM_GRIDVIEW*)pNMHDR;

	*pResult = 0;

	if(!pNMGV->iRow || (pNMGV->iRow-1 >= m_pTestFile->GetNumTaskGroups()))
		return;
	
	if(pNMGV->iColumn == enHostCol)
	{
		if(EditHost(pNMGV->iRow-1))
			m_pDoc->SetModifiedFlag();
		return;
	}
	else
	if(pNMGV->iColumn == enVUCol)
	{
		CTestFile::TaskType nType = m_pTestFile->GetTaskType(pNMGV->iRow-1,0);
		if((nType == CTestFile::enScript) && EditVUs(pNMGV->iRow-1))
			m_pDoc->SetModifiedFlag();
		return;
	}
	
	if(pNMGV->iColumn < enFirstTaskCol)
		return;
	
	if(m_pTestFile->GetNumTasks(pNMGV->iRow-1)<=pNMGV->iColumn-enFirstTaskCol)
		return;

	CTestFile::TaskType nType = m_pTestFile->GetTaskType(pNMGV->iRow-1,pNMGV->iColumn-enFirstTaskCol);
	switch(nType)
	{
	case CTestFile::enScript:
		if(EditScriptTask(pNMGV->iRow-1,pNMGV->iColumn-enFirstTaskCol))
			m_pDoc->SetModifiedFlag();
	break;
	case CTestFile::enProfile:
		if(EditProfileTask(pNMGV->iRow-enFirstTaskCol))
			m_pDoc->SetModifiedFlag();
	break;
	}
}

BOOL CTestCtrl::EditHost(int nGroup)
{
	CHostDefDlg dlg;
	dlg.m_strHostName = m_pTestFile->GetTaskGroupHost(nGroup);
	dlg.m_pTestFile = m_pTestFile;
	if(dlg.DoModal() == IDOK)
	{
		m_pTestFile->SetTaskGroupHost(nGroup,dlg.m_strHostName);
		SetItemText(nGroup+1,enHostCol,dlg.m_strHostName);
		AutoSizeColumn(enHostCol);
		Invalidate();

		return true;
	}
	return false;
}

BOOL CTestCtrl::EditProfileTask(int nGroup)
{
	return true;
}

BOOL CTestCtrl::EditScriptTask(int nGroup,int nTask)
{
	DWORD dwBufSize=0;
	LPBYTE pData = NULL;
	m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwBufSize);
	
	CScriptDefDlg dlg;
	dlg.LoadFromBuffer(pData);
	if(dlg.DoModal() == IDOK)
	{
		delete []pData;

		dwBufSize = 0;
		dlg.SaveToBuffer(NULL,dwBufSize);
		pData = new BYTE[dwBufSize];
		dlg.SaveToBuffer(pData,dwBufSize);
		m_pTestFile->SetTaskData(nGroup,nTask,pData,dwBufSize);
		return true;
	}
	return false;
}

int CTestCtrl::RegisterFileExtension(const CString& strFile)
{
	int nPos = strFile.ReverseFind('.');
	if(nPos == -1)
		return -1;

	CString strExtension(strFile.Mid(nPos));

	int nIndex=0;
	if(m_mapExtToImage.Lookup(strExtension,nIndex))
		return nIndex;

	SHFILEINFO shfi;
	SHGetFileInfo( strFile, 0, &shfi, sizeof( SHFILEINFO ), 
               SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);

	if(!shfi.hIcon || !(nIndex = (WORD)m_ListImages.Add(shfi.hIcon)))
		return -1;

	DestroyIcon(shfi.hIcon);

	m_mapExtToImage.SetAt(strExtension,nIndex);	
	return nIndex;
}

CString CTestCtrl::GetTargetType(const CString& strPath)
{
	int nPos = strPath.ReverseFind('.');
	return strPath.Mid(nPos+1);
}

void CTestCtrl::SetDocument(CTestPluginDoc* pDoc)
{
	m_pDoc = pDoc;
	m_pTestFile = m_pDoc->GetTestFile();
}

void CTestCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	m_ptContextClick=point;

	CPoint pt(point);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol) || !nRow)
		return;

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

	//This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_CONFIG_CONTEXT_MENU));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

}

void CTestCtrl::OnDuplicatetaskgroup() 
{
	CPoint pt(m_ptContextClick);
	ScreenToClient(&pt);
	int nRow,nCol;
	
	if(!HitTest(pt,nRow,nCol) || !nRow)
		return;

	if(m_pTestFile->GetNumTaskGroups()<=nRow-1)
		return;

	DWORD dwDataSize=0;
	LPBYTE pTG = m_pTestFile->WriteTaskGroupToBuffer(nRow-1,dwDataSize);
	m_pTestFile->AddTaskGroupFromBuffer(pTG);
	delete []pTG;
	Refresh();
}
