/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "ChangeDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const char* DEFAULT_TEXT = _T( "TraceLevel" );

/////////////////////////////////////////////////////////////////////////////
// CChangeDialog dialog

CChangeDialog::CChangeDialog(CWnd* pParent /*=NULL*/)
        : CDialog(CChangeDialog::IDD, pParent)
{
    //{{AFX_DATA_INIT(CChangeDialog)
    //}}AFX_DATA_INIT
    m_szLevel.Empty();
    m_strCategory = DEFAULT_TEXT;
    m_bAllLevels = FALSE;
}


void CChangeDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CChangeDialog)
    DDX_Control(pDX, IDC_LEVELS, m_cbLevels);
    DDX_Control(pDX, IDC_CATEGORY, m_ebCategory);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CChangeDialog, CDialog)
//{{AFX_MSG_MAP(CChangeDialog)
ON_BN_CLICKED(IDC_ALL_RADIO, OnAllRadio)
ON_BN_CLICKED(IDC_CATEGORY_RADIO, OnCategoryRadio)
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDC_CATEGORY, OnUpdateCategory)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChangeDialog message handlers

////////////////////////////////////////////////////////////////////////////////
// @mfunc   When the OK button is pressed this method simply ensures the member
//          variables are up to date and then closes the dialog.
void CChangeDialog::OnOK()
{
    // Determine the scope of the request
    CButton* pAll = (CButton*) GetDlgItem(IDC_ALL_RADIO);
    ASSERT (pAll != NULL);

    m_bAllLevels = pAll->GetCheck();

    // And the level to set it to
    m_cbLevels.GetWindowText(m_szLevel);

    if (m_bAllLevels)
    {
        m_strCategory.Empty();
    }
    else
    {
        // Grab the category
        m_ebCategory.GetWindowText(m_strCategory);
    }


    CDialog::OnOK();
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is called when the dialog is first shown.  Its main
//          responsibility is to initialize the starting values and to anchor
//          the edit box to the spin control.
BOOL CChangeDialog::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Set initial values for controls
    m_ebCategory.SetWindowText(m_strCategory);
    m_cbLevels.SelectString(-1, m_szLevel);

    // Initialize Category Radios
    CButton* pAll = (CButton*) GetDlgItem(IDC_ALL_RADIO);
    ASSERT (pAll != NULL);

    CButton* pCategory = (CButton*) GetDlgItem(IDC_CATEGORY_RADIO);
    ASSERT (pCategory != NULL);

    // And initialize allow modification of specific categories
    pAll->SetCheck(FALSE);
    pCategory->SetCheck(TRUE);

    UpdateDialogControls(GetParent(), FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// OnUpdateCategory
// ================
//
// Enable the Category Field

void CChangeDialog::OnUpdateCategory(CCmdUI *pCmdUI)
{
    CButton* pCategory = (CButton*) GetDlgItem(IDC_CATEGORY_RADIO);
    ASSERT (pCategory != NULL);

    BOOL bState = pCategory->GetCheck();

    pCmdUI->Enable(bState);
}

//
// OnAllRadio
// ==========
//
// Enable the Install Option
void CChangeDialog::OnAllRadio()
{
    // Disable the Category Check Box
    UpdateDialogControls(GetParent(), FALSE);
}

//
// OnCategoryRadio
// ===============
//
// Enable the Install Option
void CChangeDialog::OnCategoryRadio()
{
    // Enable the Category Check Box
    UpdateDialogControls(GetParent(), FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CAddDialog dialog

CAddDialog::CAddDialog(CWnd* pParent /*=NULL*/)
        : CDialog(CAddDialog::IDD, pParent)
{
    //{{AFX_DATA_INIT(CAddDialog)
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_szLevel.Empty();
    m_strCategory = DEFAULT_TEXT;
}


void CAddDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CAddDialog)
    DDX_Control(pDX, IDC_LEVELS, m_cbLevels);
    DDX_Control(pDX, IDC_CATEGORY, m_ebCategory);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CAddDialog, CDialog)
//{{AFX_MSG_MAP(CChangeDialog)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddDialog message handlers

////////////////////////////////////////////////////////////////////////////////
// @mfunc   When the OK button is pressed this method simply ensures the member
//          variables are up to date and then closes the dialog.
void CAddDialog::OnOK()
{
    // Grab the category
    m_ebCategory.GetWindowText(m_strCategory);

    // And the level to set the category to
    m_cbLevels.GetWindowText(m_szLevel);

    CDialog::OnOK();
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is called when the dialog is first shown.  Its main
//          responsibility is to initialize the starting values and to anchor
//          the edit box to the spin control.
BOOL CAddDialog::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Set initial values for controls
    m_ebCategory.SetWindowText(m_strCategory);
    m_cbLevels.SetCurSel(0);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}
