/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _ASYNCVIRTUALUSERCONTROLLER_H_
#define _ASYNCVIRTUALUSERCONTROLLER_H_

#include "AsyncVirtualUser.h"
#include "ThreadPoolController.hpp"
#include "TimerQueue.hpp"

#ifndef CYRANO_TP_STANDALONE
#   include "TestExecuter.h"
#   include "TestLevel.h"
#endif


// /////////////////////////////////////////////////////////////////////////////
//
// Class used to maintain an array of CAsyncVirtualUsers.
// Access to instances of that class should be synchronised by callers.
//
// ////////////////////////////////////////////////////////////////////////////

class CVirtualUsersGroup
{
public:
    CVirtualUsersGroup(CTrace* pTrace = NULL);
    ~CVirtualUsersGroup()   { DropAllUsers(); }

    BOOL CreateUsers(DWORD nbUsers);
    BOOL InitUsers(CIOCP* pIocp, CAsyncVirtualUserController& controller, CTimerQueue* pTimerQueue, DWORD dwParallelism);

#ifndef CYRANO_TP_STANDALONE
    DWORD   StartMoreUsers(DWORD dwNbUsers, const std::string &szTofFile, TestLevel_ptr pTL, const ThreadDef &thrdef);
#else
    DWORD   StartMoreUsers(DWORD dwNbUsers, const std::string &szTofFile);
#endif

    void    CancelAllUsers();
    void    DropAllUsers();

    void    AppendThreadIdents      (std::string &str)                      const;
    DWORD   GetMatchingThreadsCount (const std::string &sname)              const;
    BOOL    GetVuScriptName         (const char *ident, std::string &name)  const;
    BOOL    GetVuStatus             (const char *ident, EVuExecPhase &phase)const;
    BOOL    GetVuStartTime          (const char *ident, ULONG &ulTime)      const;
#ifndef CYRANO_TP_STANDALONE
    DWORD   AppendThreadInformation (ThreadInfoList &TIList, DWORD dwStartPos) const;
    DWORD   GetTestCaseResults      (TestCaseStatus Status) const;
#endif

private:
    // Helpers.

    // Forbidden operators.
    CVirtualUsersGroup(const CVirtualUsersGroup &VUG);
    CVirtualUsersGroup &operator = (const CVirtualUsersGroup &VUG);

    // Data structures.
    CIOCP* m_pIocp;

    CAsyncVirtualUser *m_VirtualUsersArray;
    DWORD m_dwVirtualUsersCount;
    DWORD m_dwStartedUsersCount;

    CTrace *m_pTrace;
};  // class CVirtualUsersGroup

class CVirtualUsersList
{
public:
    CVirtualUsersList();
    ~CVirtualUsersList();

    DWORD Init(CIOCP* pIocp, CTimerQueue* pTimerQueue, CAsyncVirtualUserController *pVUController, DWORD dwInitialVUCount, DWORD dwVUCountGrowBy, DWORD dwParallelism, CTrace *pTrace = NULL);

    DWORD StartUsers(DWORD dwNbUsers, const std::string &szTofFile
#ifndef CYRANO_TP_STANDALONE
                    , TestLevel_ptr pTL, const ThreadDef &thrdef
#endif
        );
    void                CancelAllUsers();
    DWORD               DropVirtualUser(DWORD dwVuId);
    void                DeleteGroups();

    void    GetThreadIdents(std::string &str)                   const;
    DWORD   GetMatchingThreadsCount(const std::string &sname)   const;
    DWORD   GetUsersCount()                                     const   { return m_dwVirtualUsersCount; }

    EVuExecPhase    GetVuStatus(const char *ident)      const;
    std::string     GetVuScriptName(const char *ident)  const;
    ULONG           GetVuStartTime(const char *ident)   const;

#ifndef CYRANO_TP_STANDALONE
    DWORD           GetThreadInformation(ThreadInfoList &TIList)    const;
    DWORD           GetTestCaseResults(TestCaseStatus Status)       const;
#endif

protected:
    // Helpers.
    BOOL CreateGroup(DWORD dwNbUsers);

    // Data structures.
    CIOCP *         m_pIocp;
    CTimerQueue *   m_pTimerQueue;
    CTrace *        m_pTrace;
    CAsyncVirtualUserController *   m_pVUController;
    DWORD   m_dwParallelism;

    std::vector<CVirtualUsersGroup*>    m_GroupsList;
    mutable CFastLock                   m_userGroupListFl;
    DWORD                               m_dwVirtualUsersCount;

    vector<CVirtualUsersGroup*>::size_type   m_currGroup;

    DWORD   m_dwVUCountGrowBy;

private:
    //// Forbidden operators.
    CVirtualUsersList(const CVirtualUsersList&);
    CVirtualUsersList& operator = (const CVirtualUsersList&);
};  // class CVirtualUsersList


// -------------------------------------------------------------------------------

class CAsyncVirtualUserController : public CThreadPoolController
{
public:
    CAsyncVirtualUserController(TScriptRule* rule, CTrace *pTrace = NULL);
    ~CAsyncVirtualUserController();

    virtual DWORD Init(DWORD dwThreadPoolSize, DWORD dwNumberOfConcurrentThreadsInIocp, DWORD dwInitialVUCount, DWORD dwVUCountGrowBy, DWORD dwBrowserParallelism);

    virtual void OnAllRequestsFinished(DWORD dwVuId);   // Called by each user when it has finished processing its TOF list.
    virtual void OnAllRequestsCancelled(DWORD dwVuId);  // Called by each user when it has cancelled processing its TOF list.

    //// IocpEvent methods.
    virtual void OnEvent      (DWORD /*dwNumberOfBytesTransferred*/, BOOL /*bIsIo*/, DWORD /*dwEventType*/);
    virtual void OnEventFailed(DWORD /*dwNumberOfBytesTransferred*/, BOOL /*bIsIo*/, DWORD /*dwEventType*/, DWORD /*dwError*/);

    ScriptCache& get_cache()        { return m_scriptCache; }

    // Starts nbUsers users and let them use TofFile.
    // Return the number of users that got used actually (not very usefull).
    // Nb : stands for AttachThread.
    int     StartNewUsersGroup(DWORD nbUsers, const std::string &szTofFile
#ifndef CYRANO_TP_STANDALONE
        , TestLevel_ptr pTL, const ThreadDef &thrdef
#endif
        );
    void    CancelAllUsers();
    void    DropUsers() { m_VirtualUsersList.DeleteGroups(); }
    // TODO: check if should serialize the calls to the users list to avoid maintain integrity. However, seems to be synchronised since CORBA calls are.

    void    GetThreadIdents(std::string &str)                   const   { m_VirtualUsersList.GetThreadIdents(str); }
    DWORD   GetMatchingThreadsCount(const std::string &sname)   const   { return m_VirtualUsersList.GetMatchingThreadsCount(sname); }  // Return the number of threads matching sname.
    DWORD   GetUsersCount()                                     const   { return m_VirtualUsersList.GetUsersCount(); } // Return the whole users count.

    std::string     GetVuScriptName(const char *ident)  const   { return m_VirtualUsersList.GetVuScriptName(ident); }
    EVuExecPhase    GetVuStatus(const char *ident)      const   { return m_VirtualUsersList.GetVuStatus(ident);     }
    DWORD           GetVuStartTime(const char *ident)   const   { return m_VirtualUsersList.GetVuStartTime(ident);  }

#ifndef CYRANO_TP_STANDALONE
    DWORD           GetThreadInformation(ThreadInfoList &TIList)    const   { return m_VirtualUsersList.GetThreadInformation(TIList); }
    DWORD           GetTestCaseResults(TestCaseStatus Status)       const   { return m_VirtualUsersList.GetTestCaseResults(Status); }
#endif

    void    IncFailedUsers()        { InterlockedIncrement((LONG*)&m_dwFailedUsersCount);     }
    void    IncSuccessfullUsers()   { InterlockedIncrement((LONG*)&m_dwSuccessfullUsersCount);}

    DWORD   GetFailedUsersCount()       const   { return m_dwFailedUsersCount; }
    DWORD   GetSuccessfullUsersCount()  const   { return m_dwSuccessfullUsersCount; }

protected:
    CVirtualUsersList   m_VirtualUsersList;
    CTimerQueue*        m_pTimerQueue;

    ScriptCache         m_scriptCache;

#ifdef USE_PERF_DATA
    static LONG*        m_spActiveUsers;
#endif

    DWORD   m_dwFailedUsersCount;
    DWORD   m_dwSuccessfullUsersCount;

private:
    //// Forbidden operators.
    CAsyncVirtualUserController& operator = (const CAsyncVirtualUserController&);
};

#endif
