/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* #########################################################################

    CDataBuffListBaseAccessor

   ######################################################################### */

/*  // Do not define the template constructor to avoid instanciation with something else
    than CDataBuffListChunkAccessor or CDataBuffListAccessor.
template <class T>
WILL_INLINE CDataBuffListBaseAccessor<T>::CDataBuffListBaseAccessor(CDataBuffList *pDataBuffList)
    : m_pDataBuffList(pDataBuffList), m_dwPosInCurrDataBuff(0)
{
    if(pDataBuffList != NULL)
    {
        m_pCurrDataBuff = m_pDataBuffList->GetBuffList();
    }
}
*/


// ----------------------------------------------------------------------------
//
// Won't use the chunk position info since that value depends directly on the 
// position (if m_pCurrDataBuff and m_dwPosInCurrDataBuff match, the chunk 
// position will match as well).
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator==(const CDataBuffListBaseAccessor<T> &item) const
{
    return m_pCurrDataBuff == item.m_pCurrDataBuff && m_dwPosInCurrDataBuff == item.m_dwPosInCurrDataBuff;
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator!=(const CDataBuffListBaseAccessor<T> &item) const
{
    return !operator==(item);
}

// ----------------------------------------------------------------------------
//
// Return TRUE if can find dBLA in the list following this.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator< (const CDataBuffListBaseAccessor<T> &dBLA) const
{
    return IsBefore(dBLA);
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator<=(const CDataBuffListBaseAccessor<T> &dBLA) const
{
    return !operator>(dBLA);
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator> (const CDataBuffListBaseAccessor<T> &dBLA) const
{
    return dBLA.IsBefore(*this);
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator>=(const CDataBuffListBaseAccessor<T> &dBLA) const
{
    return !operator<(dBLA);
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator!=(void *p) const
{
    assert(p == NULL);
    return m_pDataBuffList != NULL;
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::operator==(void *p) const
{
    assert(p == NULL);
    return m_pDataBuffList == NULL;
}

// Don't care for chunk position. Will be calculated again if initialising.
template <class T>
WILL_INLINE void *CDataBuffListBaseAccessor<T>::operator= (void *p)
{
    assert(p == NULL);
    m_pDataBuffList = NULL;
    m_pCurrDataBuff = NULL;
    m_dwPosInCurrDataBuff = 0;

    return NULL;
}


template <class T>
WILL_INLINE const CDataBuffListBaseAccessor<T> &CDataBuffListBaseAccessor<T>::operator= (const CDataBuffListBaseAccessor<T> &dBLA)
{
    m_pDataBuffList             = dBLA.m_pDataBuffList;
    m_pCurrDataBuff             = dBLA.m_pCurrDataBuff;
    m_dwPosInCurrDataBuff       = dBLA.m_dwPosInCurrDataBuff;
    m_dwCurrentChunkSizeLeft    = dBLA.m_dwCurrentChunkSizeLeft;    // Performed even when not working with chunks in case
                                                                    // a chunk iterator would be set to this one's position later on.

    return *this;
}

template <class T>
WILL_INLINE const CDataBuffListBaseAccessor<T> &CDataBuffListBaseAccessor<T>::operator+=(DWORD dwInc)
{
    Skip(dwInc);
    return *this;
}

template <class T>
WILL_INLINE const CDataBuffListBaseAccessor<T> CDataBuffListBaseAccessor<T>::operator+ (DWORD dwInc) const
{
    CDataBuffListBaseAccessor<T> tmpIter = *this;
    tmpIter.Skip(dwInc);
    return tmpIter;
}

// ----------------------------------------------------------------------------
//
// Skips the white spaces in a sequence of buffers.
// Return TRUE if managed to move to something else than a white space.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipWS()
{
    CHAR curChar;

    while(TRUE)
    {
        CurrentChar(curChar);

        if(IsWS(curChar) == FALSE)
            return TRUE;

        if(MoveNextChar() == FALSE)
            return FALSE;
    }
}

// ----------------------------------------------------------------------------
//
// Return TRUE if found a WS after a word.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipTillWS()
{
    CHAR curChar;

    while(TRUE)
    {
        CurrentChar(curChar);
        
        if(IsWS(curChar) == TRUE)
            return TRUE;

        if(MoveNextChar() == FALSE)
            return FALSE;
    }
}

// ----------------------------------------------------------------------------
//
// Return TRUE if a CRLF was found after the current iterator position, and 
// if can move one char further.
// NOTE that if the CRLF sequence is found but no char follows it, that method
// leaves the iterator in an UNSTABLE state. This requires that we call 
// AdjustIterator().
// Note that because we are a tolerant application, we consider 
// as CRLF any sequence containing any \r and a \n.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipCRLF()
{
    CHAR curChar;

    while(TRUE)
    {
        CurrentChar(curChar);

        if(MoveNextChar() == FALSE)
            return FALSE;

        if(curChar == '\n')
            return TRUE;
    }
}

// ----------------------------------------------------------------------------
//
// Return TRUE if char c was found, and move one char further.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipChar(CHAR c)
{
    CHAR curChar;

    while(TRUE)
    {
        CurrentChar(curChar);

        if(MoveNextChar() == FALSE)
            return FALSE;

        if(curChar == c)
            return TRUE;
    }
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipTillAny(CHAR* str, DWORD dwStrLen)
{
    CHAR curChar;
    DWORD dwCurs;

    while(TRUE)
    {
        CurrentChar(curChar);

        for(dwCurs=0 ; dwCurs<dwStrLen ; dwCurs++)
        {
            if(curChar == str[dwCurs])
            {
                return TRUE;
            }
        }

        if(MoveNextChar() == FALSE)
            return FALSE;
    }
}

// ----------------------------------------------------------------------------
//
// Move to the next data buffer.
// WARNING: no guarantie that will point to a valid data.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE CDataBuffListBaseAccessor<T> &CDataBuffListBaseAccessor<T>::operator++()
{
    MoveNextChar();
    return *this;
}

// ----------------------------------------------------------------------------
//
// Returns the current char pointed to by that iterator, or 0 (the value)
// if reached the end of buffer.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE CHAR CDataBuffListBaseAccessor<T>::operator*()
{
    CHAR curChar;

    if(IsEndOfBuffs())
        return '\0';

    CurrentChar(curChar);
    return curChar;
}

// ----------------------------------------------------------------------------
//
// Read the char currently pointed at.
// Return FALSE if no value could be read.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE void CDataBuffListBaseAccessor<T>::CurrentChar(CHAR &curChar)  const
{
    assert(m_pCurrDataBuff);
    assert(m_pCurrDataBuff->GetDataPtr());

    curChar = m_pCurrDataBuff->GetDataPtr()[m_dwPosInCurrDataBuff];
}

// ----------------------------------------------------------------------------
//
// Computes the number of items separating the two iterators.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE DWORD CDataBuffListBaseAccessor<T>::operator- (const CDataBuffListBaseAccessor<T> &nextIter) const
{
    DWORD dw = ComputeDistance(nextIter, *this);
    assert(dw != ULONG_MAX);

    return dw;
}


// ----------------------------------------------------------------------------
//
// Return the number of chars separating current pos from "c".
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::FindChar(CHAR c, DWORD &dwPos) const
{
    CDataBuffListBaseAccessor<T> tmpIter = *this;
    CHAR curChar;

    dwPos = 0;

    while(TRUE)
    {
        tmpIter.CurrentChar(curChar);

        if(curChar == c)
        {
            return TRUE;
        }

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
        dwPos++;
    }
}

// ----------------------------------------------------------------------------
//
// Return the number of chars separating current pos from the first char in "str".
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::FindAny(LPCSTR str, DWORD dwStrLen, DWORD &dwPos)   const
{
    CDataBuffListBaseAccessor<T> tmpIter = *this;
    CHAR curChar;
    DWORD dwCurs;
    dwPos = 0;

    while(TRUE)
    {
        tmpIter.CurrentChar(curChar);

        for(dwCurs=0 ; dwCurs<dwStrLen ; dwCurs++)
        {
            if(curChar == str[dwCurs])
            {
                return TRUE;
            }
        }

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
        dwPos++;
    }
}


// ----------------------------------------------------------------------------
//
// Check if the next word in the buffer matches str.
// Return FALSE if strings are different OR buffer is not complete.
// Does not move the cursor through the buffers.
// Comparison is case sensitive.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::MatchFirstWord(LPCSTR str, DWORD strSize) const
{
    CHAR curChar;
    CDataBuffListBaseAccessor<T> tmpIter = *this;
    DWORD dw = 0;

    while(dw < strSize)
    {
        tmpIter.CurrentChar(curChar);

        if(str[dw] != curChar)
        {
            return FALSE;
        }

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
        dw++;
    }

    return TRUE;
}

// ----------------------------------------------------------------------------
//
// Check if the next word in the buffer matches str.
// Return FALSE if strings are different OR buffer is not complete.
// Does not move the cursor through the buffers.
// Comparison is case sensitive, and str is lower case.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::MatchFirstWordToLower(LPCSTR str, DWORD strSize) const
{
    CHAR curChar;
    CDataBuffListBaseAccessor<T> tmpIter = *this;
    DWORD dw = 0;

    while(dw < strSize)
    {
        tmpIter.CurrentChar(curChar);

        if(str[dw] != curChar && str[dw] != _tolower(curChar))
        {
            return FALSE;
        }

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
        dw++;
    }

    return TRUE;
}

// ----------------------------------------------------------------------------
//
// Return TRUE if pointing at a CRLF or FALSE if can't read current char or not
// a CRLF.
// Note that because we are a tolerant application, we consider 
// as CRLF any of the following sequences: \r\n or \n.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::IsCRLF()
{
    CHAR curChar;

    CurrentChar(curChar);

    if(curChar == '\n')
        return TRUE;
    else
    {
        if(curChar == '\r')
        {
            CHAR nextChar;
            if(NextChar(nextChar) == TRUE)
                return nextChar == '\n';
            else
                return FALSE;   // Don't know.
        }
        else
            return FALSE;
    }
}

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::IsEndOfBuffs()
{
    assert(m_pDataBuffList != NULL);
    assert(m_pCurrDataBuff != NULL);

    return (m_pCurrDataBuff->GetNextBuff() == NULL 
                    && m_dwPosInCurrDataBuff == m_pCurrDataBuff->GetUsedDataSize());
}


// ----------------------------------------------------------------------------
//
// Parse the next word found into a DWORD.
// Does not move the cursor through the buffers.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::ReadDWORD(DWORD &dwRes) const
{
    CHAR curChar;
    CDataBuffListBaseAccessor<T> tmpIter = *this;

    if(tmpIter.SkipWS() == FALSE)
        return FALSE;

    dwRes = 0;

    while(TRUE)
    {
        tmpIter.CurrentChar(curChar);

        if(IsDigit(curChar) == FALSE)
        {
            // Reached the end of the value string. The value was successfully parsed.
            return TRUE;
        }

        dwRes *= 10L;
        dwRes += curChar - '0';

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
    }

    // Reached the end of the buffers, but not the end of the value string.
    return FALSE;
}

// ----------------------------------------------------------------------------
//
// Parse the next hexadecimal word found into a DWORD.
// Does not move the cursor through the buffers.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::ReadHexDWORD(DWORD &dwRes) const
{
    CHAR curChar;
    CDataBuffListBaseAccessor<T> tmpIter = *this;

    if(tmpIter.SkipWS() == FALSE)
        return FALSE;

    dwRes = 0;

    while(TRUE)
    {
        tmpIter.CurrentChar(curChar);

        if(IsAlpha(curChar) == FALSE && IsDigit(curChar) == FALSE)
        {
            // Reached the end of the value string. The value was successfully parsed.
            return TRUE;
        }

        dwRes *= 16L;
        if(curChar >= 'a')
            dwRes += curChar - 'a' + 10;
        else
            if(curChar >= 'A')
                dwRes += curChar - 'A' + 10;
            else
                dwRes += curChar - '0';

        if(tmpIter.MoveNextChar() == FALSE)
            return FALSE;
    }

    // Reached the end of the buffers, but not the end of the value string.
    return FALSE;
}


// ----------------------------------------------------------------------------
//
// Applies callBackFct to all items while callBackFct returns TRUE.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::ForEach(BOOL (__cdecl *callBackFct)(CHAR c, void *param), void *callBackFctParam)
{
    CHAR curChar;

    do
    {
        CurrentChar(curChar);

        if(MoveNextChar() == FALSE)
            return FALSE;
    }
    while(callBackFct(curChar, callBackFctParam) == TRUE);

    return TRUE;
}


// Return TRUE if the current object represents a position "before" the other object.
// Note the code is the same for the raw and chunk iterators.

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::IsBefore(const CDataBuffListBaseAccessor<T> &other) const
{
    assert(m_pCurrDataBuff != NULL && other.m_pCurrDataBuff != NULL);

    // Walk forward from the current buffer to the other buffer or NULL.
    for(const CDataBuff *pOtherDataBuff = m_pCurrDataBuff;
        pOtherDataBuff != NULL && pOtherDataBuff != other.m_pCurrDataBuff;
        pOtherDataBuff = pOtherDataBuff->GetNextBuff())
            ;

    return
        (pOtherDataBuff == m_pCurrDataBuff ?
            (m_dwPosInCurrDataBuff < other.m_dwPosInCurrDataBuff)   // Same buffer: compare positions.
        :
            (pOtherDataBuff == other.m_pCurrDataBuff)               // The other buffer is further in the sequence.
        );
}

// ----------------------------------------------------------------------------
//
// Skips to the end of the str string.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::SkipStr(LPCSTR str, DWORD dwStrLen, BOOL bCaseSensitive)
{
    return FindSkipStr(str, dwStrLen, SKIP, bCaseSensitive);
}


// ----------------------------------------------------------------------------
//
// Skip all chars till the str string.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE BOOL CDataBuffListBaseAccessor<T>::MoveToStr(LPCSTR str, DWORD dwStrLen, BOOL bCaseSensitive)
{
    return FindSkipStr(str, dwStrLen, FIND, bCaseSensitive);
}


// ----------------------------------------------------------------------------
//
// Compute the amount of data left to iterate through.
//
// ----------------------------------------------------------------------------

template <class T>
WILL_INLINE DWORD CDataBuffListBaseAccessor<T>::ComputeDataLeft() const
{
    DWORD dwRes = 0;
    CDataBuff *pCurrBuff = m_pCurrDataBuff;

    assert(pCurrBuff != NULL);

    dwRes = pCurrBuff->GetUsedDataSize() - m_dwPosInCurrDataBuff;
    pCurrBuff = pCurrBuff->GetNextBuff();

    while(pCurrBuff != NULL)
    {
        dwRes += pCurrBuff->GetUsedDataSize();
        pCurrBuff = pCurrBuff->GetNextBuff();
    }

    return dwRes;
}
