/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////
// Tag parsing helpers.
///////////////////////

#include "stdafx.h"
#include "TagParsing.hpp"
#include<iostream>
using namespace std;

NameToTag nameToTag;
TagToBehavior tagToBehavior;
TagToDefaultBehavior tagToDefaultBehavior;
TagToName tagToName;

extern const long tagBehaviors[] =
{
    TAG_NONE,
        (long)"**UNKNOWN**", TAG_CREATES_CHILD,
        -1,

    TAG_A,
        (long)"A", TAG_CLOSES,
        // inline - A
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG, TAG_DFN,
        TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR, TAG_ACRONYM,
        TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q, TAG_SUB,
        TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT, TAG_TEXTAREA,
        TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_ABBR,
        (long)"ABBR", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_ACRONYM,
        (long)"ACRONYM", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_ADDRESS,
        (long)"ADDRESS", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_APPLET,
        (long)"APPLET", TAG_CLOSES,
        TAG_CENTER, TAG_PARAM,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_AREA,
        (long)"AREA", TAG_CLOSES,
        -1,

    TAG_B,
        (long)"B", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_BASE,
        (long)"BASE", TAG_CLOSES,
        -1,

    TAG_BASEFONT,
        (long)"BASEFONT", TAG_CLOSES,
        -1,

    TAG_BDO,
        (long)"BDO", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_BGSOUND,
        (long)"BGSOUND", TAG_CLOSES,
        -1,

    TAG_BIG,
        (long)"BIG", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_BLINK,
        (long)"BLINK", TAG_CLOSES,
        -1,

    TAG_BLOCKQUOTE,
        (long)"BLOCKQUOTE", TAG_CLOSES,
        TAG_CENTER, TAG_SCRIPT,
        // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
        -1,

    TAG_BODY,
        (long)"BODY", TAG_CLOSES,
        TAG_TD, TAG_TR, TAG_CENTER, TAG_SCRIPT, TAG_INS, TAG_DEL,
        // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
        // inline - SCRIPT(transitional DTD).
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_BR,
        (long)"BR", TAG_CLOSES,
        -1,

    TAG_BUTTON,
        (long)"BUTTON", TAG_CLOSES,
        TAG_CENTER,
        // flow - A, formctrl, form, fieldset
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE,
        TAG_HR, TAG_TABLE, TAG_ADDRESS,
        TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO,
        -1,

    TAG_CAPTION,
        (long)"CAPTION", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_CENTER,
        (long)"CENTER", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_CITE,
        (long)"CITE", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_CODE,
        (long)"CODE", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_COL,
        (long)"COL", TAG_CLOSES,
        -1,

    TAG_COLGROUP,
        (long)"COLGROUP", TAG_CLOSES,
        TAG_COL,
        -1,

    TAG_COMMENT,
        (long)"COMMENT", TAG_CLOSES,
        -1,

    TAG_DD,
        (long)"DD", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_DEL,
        (long)"DEL", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_DFN,
        (long)"DFN", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_DIR,
        (long)"DIR", TAG_CLOSES,
        TAG_LI,
        -1,

    TAG_DIV,
        (long)"DIV", TAG_CLOSES,
        TAG_TD, TAG_TR, TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_DL,
        (long)"DL", TAG_CLOSES,
        TAG_DD, TAG_DT,
        -1,

    TAG_DT,
        (long)"DT", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_EM,
        (long)"EM", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_EMBED,
        (long)"EMBED", TAG_CLOSES,
        -1,

    TAG_FIELDSET,
        (long)"FIELDSET", TAG_CLOSES,
        TAG_CENTER, TAG_LEGEND,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_FONT,
        (long)"FONT", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_FORM,
        (long)"FORM", TAG_CLOSES,
        TAG_CENTER,
        // block - form
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
        // inline (in violation to the DTD)
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_FRAME,
        (long)"FRAME", TAG_CLOSES,
        -1,

    TAG_FRAMESET,
        (long)"FRAMESET", TAG_CLOSES,
        TAG_FRAMESET, TAG_FRAME, TAG_NOFRAMES,
        -1,

    TAG_H1,
        (long)"H1", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_H2,
        (long)"H2", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_H3,
        (long)"H3", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_H4,
        (long)"H4", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_H5,
        (long)"H5", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_H6,
        (long)"H6", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_HEAD,
        (long)"HEAD", TAG_CLOSES,
        TAG_TITLE, TAG_BASE, TAG_SCRIPT, TAG_STYLE, TAG_META, TAG_LINK,
        TAG_OBJECT,
        -1,

    TAG_HR,
        (long)"HR", TAG_CLOSES,
        -1,

    TAG_HTML,
        (long)"HTML", TAG_CLOSES,
        TAG_CENTER, TAG_HEAD, TAG_BODY, TAG_FRAMESET, TAG_NOFRAMES,
        TAG_SCRIPT, TAG_INS, TAG_DEL,
        TAG_TITLE, TAG_BASE, TAG_SCRIPT, TAG_STYLE, TAG_META, TAG_LINK,
        TAG_OBJECT,
        // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
        -1,

    TAG_I,
        (long)"I", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_IFRAME,
        (long)"IFRAME", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_ILAYER,
        (long)"ILAYER", TAG_CLOSES,
        -1,

    TAG_IMG,
        (long)"IMG", TAG_CLOSES,
        -1,

    TAG_INPUT,
        (long)"INPUT", TAG_CLOSES,
        -1,

    TAG_INS,
        (long)"INS", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_ISINDEX,
        (long)"ISINDEX", TAG_CLOSES,
        -1,

    TAG_KBD,
        (long)"KBD", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_KEYGEN,
        (long)"KEYGEN", TAG_CLOSES,
        -1,

    TAG_LABEL,
        (long)"LABEL", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_LAYER,
        (long)"LAYER", TAG_CLOSES,
        -1,

    TAG_LEGEND,
        (long)"LEGEND", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_LI,
        (long)"LI", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_LINK,
        (long)"LINK", TAG_CLOSES,
        -1,

    TAG_LISTING,
        (long)"LISTING", TAG_CLOSES,
        -1,

    TAG_MAP,
        (long)"MAP", TAG_CLOSES,
        TAG_CENTER, TAG_AREA,
        // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
        -1,

    TAG_MARQUEE,
        (long)"MARQUEE", TAG_CLOSES,
        -1,

    TAG_MENU,
        (long)"MENU", TAG_CLOSES,
        TAG_LI,
        -1,

    TAG_META,
        (long)"META", TAG_CLOSES,
        -1,

    TAG_MULTICOL,
        (long)"MULTICOL", TAG_CLOSES,
        -1,

    TAG_NOBR,
        (long)"NOBR", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_NOEMBED,
        (long)"NOEMBED", TAG_CLOSES,
        -1,

    TAG_NOFRAMES,
        (long)"NOFRAMES", TAG_CLOSES,
        TAG_BODY, TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_NOLAYER,
        (long)"NOLAYER", TAG_CLOSES,
        -1,

    TAG_NOSCRIPT,
        (long)"NOSCRIPT", TAG_CLOSES,
        TAG_CENTER,
        // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline (in violation to the DTD)
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_OBJECT,
        (long)"OBJECT", TAG_CLOSES,
        TAG_CENTER, TAG_PARAM,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_OL,
        (long)"OL", TAG_CLOSES,
        TAG_LI,
        -1,

    TAG_OPTGROUP,
        (long)"OPTGROUP", TAG_CLOSES,
        TAG_OPTION,
        -1,

    TAG_OPTION,
        (long)"OPTION", TAG_CLOSES,
        -1,

    TAG_P,
        (long)"P", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_PARAM,
        (long)"PARAM", TAG_CLOSES,
        -1,

    TAG_PLAINTEXT,
        (long)"PLAINTEXT", TAG_CLOSES,
        -1,

    TAG_PRE,
        (long)"PRE", TAG_CLOSES,
        // inline - img, object, big, small, sub, sup
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_Q,
        (long)"Q", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_RT,
        (long)"RT", TAG_CLOSES,
        -1,

    TAG_RUBY,
        (long)"RUBY", TAG_CLOSES,
        TAG_RT,
        -1,

    TAG_S,
        (long)"S", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_SAMP,
        (long)"SAMP", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_SCRIPT,
        (long)"SCRIPT", TAG_CLOSES,
        -1,

    TAG_SELECT,
        (long)"SELECT", TAG_CLOSES,
        TAG_OPTGROUP, TAG_OPTION,
        -1,

    TAG_SERVER,
        (long)"SERVER", TAG_CLOSES,
        -1,

    TAG_SMALL,
        (long)"SMALL", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_SPACER,
        (long)"SPACER", TAG_CLOSES,
        -1,

    TAG_SPAN,
        (long)"SPAN", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_STRIKE,
        (long)"STRIKE", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_STRONG,
        (long)"STRONG", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_STYLE,
        (long)"STYLE", TAG_CLOSES,
        -1,

    TAG_SUB,
        (long)"SUB", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_SUP,
        (long)"SUP", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_TABLE,
        (long)"TABLE", TAG_CLOSES,
        TAG_CAPTION, TAG_COL, TAG_COLGROUP, TAG_THEAD, TAG_TFOOT, TAG_TBODY,
        TAG_TR,
        -1,

    TAG_TBODY,
        (long)"TBODY", TAG_CLOSES,
        TAG_TR,
        -1,

    TAG_TD,
        (long)"TD", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_TEXTAREA,
        (long)"TEXTAREA", TAG_CLOSES,
        -1,

    TAG_TFOOT,
        (long)"TFOOT", TAG_CLOSES,
        TAG_TR,
        -1,

    TAG_TH,
        (long)"TH", TAG_CLOSES,
        TAG_CENTER,
        // flow
            // block
        TAG_P, TAG_H1, TAG_H2, TAG_H3, TAG_H4, TAG_H5, TAG_H6, TAG_UL, TAG_OL,
        TAG_PRE, TAG_DL, TAG_DIV, TAG_NOSCRIPT, TAG_BLOCKQUOTE, TAG_FORM,
        TAG_HR, TAG_TABLE, TAG_FIELDSET, TAG_ADDRESS, TAG_MARQUEE,
            // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_THEAD,
        (long)"THEAD", TAG_CLOSES,
        TAG_TR,
        -1,

    TAG_TITLE,
        (long)"TITLE", TAG_CLOSES,
        -1,

    TAG_TR,
        (long)"TR", TAG_CLOSES,
        TAG_TH, TAG_TD,
        -1,

    TAG_TT,
        (long)"TT", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_U,
        (long)"U", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_UL,
        (long)"UL", TAG_CLOSES,
        TAG_LI,
        -1,

    TAG_VAR,
        (long)"VAR", TAG_CLOSES,
        // inline
        TAG_BLINK,
        TAG_APPLET, TAG_IFRAME,
        TAG_A, TAG_TT, TAG_I, TAG_B, TAG_BIG, TAG_SMALL, TAG_EM, TAG_STRONG,
        TAG_DFN, TAG_CODE, TAG_SAMP, TAG_KBD, TAG_VAR, TAG_CITE, TAG_ABBR,
        TAG_ACRONYM, TAG_IMG, TAG_OBJECT, TAG_BR, TAG_SCRIPT, TAG_MAP, TAG_Q,
        TAG_SUB, TAG_SUP, TAG_SPAN, TAG_BDO, TAG_INPUT, TAG_SELECT,
        TAG_TEXTAREA, TAG_LABEL, TAG_BUTTON, TAG_FONT, TAG_BASEFONT,
        -1,

    TAG_WBR,
        (long)"WBR", TAG_CLOSES,
        -1,

    TAG_XML,
        (long)"XML", TAG_CLOSES,
        -1,

    TAG_XMP,
        (long)"XMP", TAG_CLOSES,
        -1,

    -1  // END OF TAGS.
};

void InitTagParsing()
{
    const long *pTb = tagBehaviors;
    Tags eTag;
    while((eTag = (Tags)*pTb) != -1)
    {
        const char *pTagName = (const char*)pTb[1];
        nameToTag[pTagName] = eTag;
        tagToName[eTag] = pTagName;
#ifdef PHTRACE
        cout << "nameToTag[" << (const char*)pTb[1] << "]=" << eTag << endl;
#endif
        TagBehavior beh = (TagBehavior)pTb[2];
        tagToDefaultBehavior[eTag] = beh;

        // Process the specific behaviors.
        pTb += 3;
        while(*pTb != -1)
        {
            TagCouple tc = {eTag, (Tags)*pTb};
            tagToBehavior[tc] = (beh == TAG_CLOSES ? TAG_CREATES_CHILD : TAG_CLOSES);
            ++pTb;
        }
        ++pTb;
    }
}
