/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLWriter.cpp: implementation of the SCLWriter class.
//
//////////////////////////////////////////////////////////////////////
#include "SCLWriter.h"
#include "TofPrmDef.h"


extern char szSCLFilename[MAX_PATH];
extern char szHeadersIniPath[MAX_PATH];
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
SCLWriter::SCLWriter()
{
    m_nBufferLen=0;
    m_nBufferPos=0;
    m_pszBuffer = NULL;

    m_bHeadersInitialised = false;
    m_bOnPrimaryGet = true;
    m_strDefIndent = _T("\t");
    m_bLastWriteWasConnect = true;
	m_nPageTimer=1;
	m_bMatchedTimers = true;

	//Generate the Page timer name format
    char *pszTofName = strrchr(szSCLFilename,'\\');
    if(pszTofName)
        pszTofName++;

    m_strPageTimerName = pszTofName;
	m_strPageTimerName = m_strPageTimerName.SpanExcluding(".");
	m_strPageTimerName.MakeUpper();
	m_strPageTimerName = "T_" + m_strPageTimerName.Left(57);
	m_strPageTimerName.TrimRight("_-");
	m_strPageTimerName += "_%d";
}

SCLWriter::~SCLWriter()
{
    if(m_pszBuffer == NULL)
        return;

    WriteExitString();

    //Flush to buffer
    WriteString("");

    //Insert header at start of file
    CString strBuffer,strDefs,strDefHead;

    GetEnvironmentString(strBuffer);
    GetDefinitionsString(strDefs);
    GetDefaultHeadersString(strDefHead);

    strBuffer += strDefs + strDefHead + m_pszBuffer;

    free(m_pszBuffer);

    try
    {
        CStdioFile SclFile;
        SclFile.Open(szSCLFilename,CFile::modeCreate|CFile::modeWrite);
        SclFile.WriteString(strBuffer);
        SclFile.Close();
    }
    catch(CFileException *e)
    {
        e->Delete();
    }

}

//Get environment section of SCL File
void SCLWriter::GetEnvironmentString(CString& strEnvironment)
{
    char *pszTofName = strrchr(szSCLFilename,'\\');
    if(pszTofName)
        pszTofName++;

    CString strTofName(pszTofName);

    strEnvironment = "Environment\n";
    strEnvironment += "\tDescription \"\"\n";// + strTofName + "\"\n";
    strEnvironment += "\tMode\t\tHTTP\n";
    strEnvironment += "\tWait\t\tUNIT MILLISECONDS\n\n";
}

//Get definitions section of SCL File
void SCLWriter::GetDefinitionsString(CString& strDefinitions)
{
    strDefinitions = "Definitions\n";
    strDefinitions += "\t! Standard Defines\n";
    strDefinitions += "\tInclude\t\t\t\"RESPONSE_CODES.INC\"\n";
    strDefinitions += "\tInclude\t\t\t\"GLOBAL_VARIABLES.INC\"\n\n";
    strDefinitions += "\tCHARACTER*512\tDEFAULT_HEADERS\n";
    strDefinitions += "\tCHARACTER*512\tUSER_AGENT\n";
    strDefinitions += "\tCHARACTER*65535\tVAR_TMP\n";
    //
    // And add in definitions for the request timeout
    strDefinitions += "\tCHARACTER*256\tMESSAGE\n\n";
    strDefinitions += "\tInteger\t\tREQUEST_TIMEOUT\n\n";
	strDefinitions += "\tInteger\t\tUSE_PAGE_TIMERS\n";
    strDefinitions += "\tInteger\t\tVAR_LOOP\n";

    for(int nVariable=0;nVariable<m_arVariables.GetSize();nVariable++)
    {
        strDefinitions += m_arVariables[nVariable] + "\n";
    }

    strDefinitions += "\n";
}

//Get Any scl which should go at the end of the script
void SCLWriter::WriteExitString()
{
	if(!m_bMatchedTimers)
		WritePageTimer();
	
    CString strExit;
    strExit += "Exit\n\n";
    //
    // Add in error label for the HTTP Request Timeout to jump to - which
    // appears after the main exit statement so that only http timeouts go through
    // the code.
    strExit += "ERR_LABEL:\n\n";
    strExit += "\tIf (MESSAGE <> \"\") Then\n";
    strExit += "\t\tReport MESSAGE\n";
    strExit += "\tEndif\n\n";
    strExit += "Exit\n";

    WriteString(strExit);
}

//Get default headers
void SCLWriter::GetDefaultHeadersString(CString& strDefaultHeaders)
{
    strDefaultHeaders = "Code\n";

    strDefaultHeaders += "\t!Read in the default browser user agent field\n";
    strDefaultHeaders += "\tEntry[USER_AGENT,USE_PAGE_TIMERS]\n\n";
    strDefaultHeaders += "\tIf (USER_AGENT = \"DEFAULT\") then\n";
    strDefaultHeaders += "\t\tSet USER_AGENT = \"User-Agent: " + m_strUserAgent + "\"\n";
    strDefaultHeaders += "\tEndif\n";

    // Set the default request timeout value
    strDefaultHeaders += "\n\t! Setup default request timeout\n";
    strDefaultHeaders += "\tSET  REQUEST_TIMEOUT = 20000\n\n";

    strDefaultHeaders += "\t! Setup default headers\n";
    strDefaultHeaders += "\tSET  DEFAULT_HEADERS = ";

    CString strHeader,strValue;
    CString strTemp;
    POSITION pos = m_mapDefaultHeaders.GetStartPosition();
    while(pos!=NULL)
    {
        m_mapDefaultHeaders.GetNextAssoc(pos,strHeader,strValue);

        strTemp = "\"" + strHeader + ": " + strValue + "^J\"";
        ValidateString(strTemp,2);

        strDefaultHeaders += strTemp;

        if(pos == NULL)
            strDefaultHeaders += " +\t&\n\t\t";
        else
            strDefaultHeaders += "\t&\n\t\t";
    }

    strDefaultHeaders += "USER_AGENT\n";

    strDefaultHeaders += "\n";
}

void SCLWriter::WriteConnectString(const char* szHostName,int nConId)
{
    m_bLastWriteWasConnect = true;

    CString strConnect;

    CString strHost(szHostName);
    ValidateString(strHost,1);

    strConnect.Format("%sCONNECT TO \"%s\" ON %d\n\n",m_strDefIndent,strHost,nConId);

    WriteString(strConnect);
}

void SCLWriter::WriteWaitString(int nDelay)
{
    m_bLastWriteWasConnect = false;

    CString strWait;
    strWait.Format("%sWAIT %d\n\n",m_strDefIndent,nDelay);

    WriteString(strWait);
}

void SCLWriter::WriteDisconnectString(int nConId,int nOptions)
{
    m_bLastWriteWasConnect = false;

    CString strDisconnect;
    strDisconnect.Format("%sDISCONNECT FROM %d",m_strDefIndent,nConId);

    if(nOptions == 1)
        strDisconnect += ",WITH CANCEL";

    strDisconnect += "\n\n";

    WriteString(strDisconnect);
}

void SCLWriter::WriteRequestString(int nConId,
                                   RequestTypes RequestType,
                                   const char* szHttpVersion,
                                   const char* szHeaders,
                                   const char* szBody,
								   const char* szCertificate,
								   int nOption)
{
    CString strType;
	
	if(nOption & TOF_REQOPT_PRIMARY)
	{
//		m_bOnPrimaryGet = true;
		strType += "PRIMARY ";
	}

    switch(RequestType)
    {
    case enGet:		strType += "GET URI";	break;
    case enHead:	strType += "HEAD URI";	break;
    case enPost:	strType += "POST URI";	break;
    default:
        ASSERT(FALSE);
        break;
    }

    BOOL bLastGetWasSecondary = !m_bOnPrimaryGet;

    m_bOnPrimaryGet = false;

    CString strHttpVersion("\"");
	strHttpVersion += szHttpVersion;
    strHttpVersion.TrimRight();
    int nPos = strHttpVersion.ReverseFind(' '); //This will find the gap between the URL and HTTP/X.X
    if(nPos!=-1)
    {
        CString strURL = strHttpVersion.Left(nPos);
        strURL.TrimRight();
        m_bOnPrimaryGet = (strURL.Right(1) == "/");
    }

    ValidateString(strHttpVersion,1);

    CString strHeader;
    ProcessHeaders(szHeaders,strHeader);

	if(m_bOnPrimaryGet && !m_bMatchedTimers)
	{
		WriteSynchronize();
		WritePageTimer();
	}

	if(m_bOnPrimaryGet)
		WritePageTimer();

    if(m_bLastWriteWasConnect && bLastGetWasSecondary && m_bOnPrimaryGet)
    {
        //Remove indent from connect on previous line
        m_strPrevLine.Replace("\t\t","\t");
    }

    m_strDefIndent = (m_bOnPrimaryGet ? "\t" : "\t\t");

    CString strRequest;
    strRequest.Format("%s%s %s\" ON %d\t\t\t&\n",m_strDefIndent,strType,strHttpVersion,nConId);

    strRequest += m_strDefIndent + "\tHEADER DEFAULT_HEADERS";

    strRequest += strHeader;

    if(szBody && RequestType == enPost)
    {
        strRequest += m_strDefIndent + "\t\t&\n\t\t,";

        if(strncmp(szBody,"0x",2)==0)
            strRequest += "BINARY ";

        CString strBody("\"");
		strBody += szBody;
		strBody += "\"";
        ValidateString(strBody,2);
        strRequest += "BODY " + strBody;
    }

	if(szCertificate)
	{
        strRequest += m_strDefIndent + "\t\t&\n\t\t,CERTIFICATE ";
		
		if(nOption & TOF_REQOPT_CERTFIL)
			strRequest += "FILE ";
		else
			strRequest += "NAME ";

		CString strCertificate("\"");
		strCertificate += szCertificate;
		strCertificate += "\"";
        ValidateString(strCertificate,2);
		strRequest += strCertificate;
	}

    strRequest += "\n\n";

    WriteString(strRequest);
}

void SCLWriter::WriteLoadResponseInfoString(int nConId,int nData,const char* szIdentifier,const char *szName)
{
    m_bLastWriteWasConnect = false;

    CString strLoadRespInfo(m_strDefIndent + "Load Response_Info ");

    CString strType;
    switch(nData)
    {
    case 0:
        strLoadRespInfo += "Header ";
        break;
    case 1:
        strLoadRespInfo += "Body ";
        break;
    default:
        ASSERT(FALSE);
        break;
    }

    CString strConId;
    strConId.Format("on %d\t\t&\n",nConId);
    strLoadRespInfo += strConId;

    strLoadRespInfo += m_strDefIndent + "\tInto ";

    CString strTemp(szName);
    ValidateString(strTemp,2);

    strLoadRespInfo += strTemp;

    if(szIdentifier)
    {
        strLoadRespInfo += "\t&\n";
        strLoadRespInfo += m_strDefIndent + "\t,WITH \"";
        strLoadRespInfo += szIdentifier;
        strLoadRespInfo += "\"";
    }

    strLoadRespInfo += "\n\n";

    WriteString(strLoadRespInfo);
}

void SCLWriter::WriteString(const CString& str)
{
    if(!m_strPrevLine.IsEmpty())
    {
        int nStrLen = m_strPrevLine.GetLength();
        if(nStrLen+1 > (m_nBufferLen-m_nBufferPos))
        {
            m_nBufferLen += nStrLen + 10000;
            char *pszNewBuffer = (char*)malloc(m_nBufferLen);

            memcpy(pszNewBuffer,m_pszBuffer,m_nBufferPos);

			free(m_pszBuffer);
            m_pszBuffer = pszNewBuffer;
        }

        strcpy(&m_pszBuffer[m_nBufferPos],m_strPrevLine);
        m_nBufferPos+=nStrLen;
    }
    m_strPrevLine = str;
}

void SCLWriter::ProcessHeaders(const char* szHeaders,CString& strHeader)
{
    CString strCopy(szHeaders);
    if(strCopy.IsEmpty())
    {
        m_bHeadersInitialised = true;
        return;
    }

	strCopy.TrimLeft("\"");
	strCopy.TrimRight("\"");

    CString strKey,strValue,strDefValue;
    CMapStringToString mapNewHeaders;

    BOOL bFirstWith=true;
    //Go through each header in the list (separated by newline)

    BOOL bFoundAcceptAll=m_bOnPrimaryGet;
    char *pszToken = strCopy.GetBuffer(0);
    char *pszTemp = pszToken;
    while(pszTemp!=NULL)
    {
        pszTemp = strstr(pszToken,"^J");

        if(pszTemp)
            pszTemp[0] = NULL;
#ifdef _DEBUG_ON_
        CString str(pszToken);
        OutputDebugString(str + "\n");
#endif

        //Separate each header into a key and a value
        //The key always has a : following it
        char *pValue = strchr(pszToken,':');

        if(pValue++)
        {
            int nColonPos = pValue - pszToken-1;

            //Obtain the key and value values
            strKey = CString(pszToken).Left(nColonPos);
            if(pValue[0] == '\0')
                strValue.Empty();
            else
                strValue = pValue+1;
            strValue.TrimRight();

            if(!bFoundAcceptAll && strKey.CompareNoCase("Accept")==0)
            {
                if(strValue == "*/*")
                    bFoundAcceptAll = true;
            }

            if(strKey.CompareNoCase("user-agent")!=0)
            {
                //The headers for the first request are assumed to be the default for
                //the remainder, so we need a copy of these to compare.
                if(!m_bHeadersInitialised)
                    m_mapDefaultHeaders.SetAt(strKey,strValue);
                else
                {
                    //Decide whether this key is the same as the default
                    //If not it must be added in the WITH
                    mapNewHeaders.SetAt(strKey,strValue);

                    BOOL bFound = m_mapDefaultHeaders.Lookup(strKey,strDefValue);
                    if(!bFound || (strDefValue != strValue))
                    {
                        if(bFirstWith)
                        {
                            bFirstWith = false;
                            strHeader += "\t\t\t&\n\t\t,WITH {";
                        }
                        else
                            strHeader += ",\t\t\t&\n\t\t\t";

                        CString strTemp("\"" + strKey + ": " + strValue );
						
						if(strKey.CompareNoCase("cookie")!=0 || (strValue.ReverseFind('\"')==-1))
							strTemp += "\"";
						
                        ValidateString(strTemp,3);
                        strHeader += strTemp;
                    }
                }
            }
            else
                if(!m_bHeadersInitialised)
                    m_strUserAgent = strValue;
        }
        pszToken = pszTemp+2;
    }

    if(!bFirstWith)
        strHeader += "}";

    if(m_bHeadersInitialised)
    {
        BOOL bFirstWithout=true;

        //Go through all the default headers ...
        POSITION pos = m_mapDefaultHeaders.GetStartPosition();
        while(pos != NULL)
        {
            m_mapDefaultHeaders.GetNextAssoc(pos,strKey,strDefValue);

            //..if this request doesn't contain the default then
            //we will need a WITHOUT
            if(!mapNewHeaders.Lookup(strKey,strValue))
            {
                if(bFirstWithout)
                {
                    bFirstWithout = false;
                    strHeader += "\t\t\t&\n\t\t,WITHOUT {";
                }
                else
                    strHeader += ",\t\t\t&\n\t\t\t";

                ValidateString(strKey,3);

                strHeader += "\"" + strKey + "\"";
            }
        }
        mapNewHeaders.RemoveAll();

        if(!bFirstWithout)
            strHeader += "}";
    }
    else
    {
        CString strIniFile;
        if(szHeadersIniPath)
        {
            strIniFile = szHeadersIniPath;
            strIniFile.TrimRight();

            if(!strIniFile.IsEmpty() && strIniFile.Right(1) != "\\")
                strIniFile += "\\";
        }
        strIniFile += "headers.ini";

        ProcessIniFile(strIniFile);
        m_bHeadersInitialised = true;
        ProcessHeaders(szHeaders,strHeader);
    }

    m_bOnPrimaryGet |= !bFoundAcceptAll;

    if(!m_bOnPrimaryGet)
    {
        strHeader = "\t" + strHeader;
        strHeader.Replace("\n\t","\n\t\t");
    }
}

void SCLWriter::WriteCommentString(const char* szComment)
{
    CString strComment;

    CString strCopy(szComment);

    CString strTemp;
    char *pszToken = strtok(strCopy.GetBuffer(0),"\n");
    while(pszToken!=NULL)
    {
        strComment += m_strDefIndent + "!";

        strTemp = pszToken;
        int nLength = strTemp.GetLength();
        int nIndex=80;
        while(nLength>nIndex)
        {
            strTemp.Insert(nIndex,"\n"+m_strDefIndent + "!");
            nIndex+=83;
            nLength++;
        }
        //  ValidateString(strTemp,1);

        strComment += strTemp;
        strComment += "\n";

        pszToken = strtok(NULL,"\n");
    }

    WriteString(strComment);
}

void SCLWriter::ProcessIniFile(const CString& strIniFile)
{
    try
    {
        CStdioFile f(strIniFile,CFile::modeRead);

        CString strKey,strValue,strHeader;
        CMapStringToString mapIniHeaders;
        while(f.ReadString(strHeader))
        {
            int nEqualsPos = strHeader.Find('=');
            if(nEqualsPos != -1)
            {
                strKey = strHeader.Left(nEqualsPos);
                strValue = strHeader.Right(strHeader.GetLength()-nEqualsPos-1);
                if(strValue == "1")
                {
                    strKey.MakeUpper();
                    mapIniHeaders.SetAt(strKey,strValue);
                }
            }
        }

        CString strKeyCopy;
        POSITION pos = m_mapDefaultHeaders.GetStartPosition();
        while(pos != NULL)
        {
            m_mapDefaultHeaders.GetNextAssoc(pos,strKey,strValue);
            strKeyCopy = strKey;
            strKey.MakeUpper();
            if(!mapIniHeaders.Lookup(strKey,strValue))
            {
                m_mapDefaultHeaders.RemoveKey(strKeyCopy);
            }
        }

        f.Close();

    }
    catch(CFileException* e)
    {
        m_mapDefaultHeaders.RemoveAll();
        e->Delete();
    }
}

//Check that a quoted line in SCL is not too long
//if it is, split it into 2 or more lines
void SCLWriter::ValidateString(CString& strLine,int nNumTabs)
{
    //The actual max line length is 132 chars, so if
    //we pick a smaller number, there is no need to be too careful
    const int nMaxLineLength=80;

    //If this line is shorter than the max length
    //there is no point checking it
    int nLineLength = strLine.GetLength();
    if(nLineLength <= nMaxLineLength)
        return;

    //...found a line that is too long.

    //Get the Indentation for this line
    CString strIndent;
    for(int nTab=0;nTab<nNumTabs;nTab++)
        strIndent += "\t";

    //...split it into 2 or more lines
    CString strTemp;
    while(nLineLength > 0)
    {
        //If the line length is longer than the max (give or
        //take a few, so we don't write one char on its own on the next line)
        if(nLineLength>(nMaxLineLength + 10))
        {
			//need to check if we're splitting a variable name
            CString strNewLine = strLine.Left(nMaxLineLength);
			
			strNewLine.TrimRight("~");	//can't split a line at a ~

			int nPos = strNewLine.ReverseFind('\"');
			if(nPos > 0)
			{
				CString str = strNewLine.Left(nPos+1);
			
				//If there is an odd number of quotes we have found the
				//beginning of a string and need do no more. Otherwise 
				//split the string at the last quote
				int nCount=0;
				for(nPos=0;nPos<str.GetLength();nPos++)
					if(str[nPos] == '\"') nCount++;

				if(nCount%2)
				{
					int nPos = strLine.Find("\"",nMaxLineLength);
					if(nPos < nMaxLineLength+10)
					{
						strNewLine = strLine.Left(nPos+1);
					}
				}
				else
					strNewLine = str;
			}
			
			int nNewLength = strNewLine.GetLength();
			strTemp += strNewLine;
			
			//Get number of quotes in the line so far...
			int nCount=0;
			for(nPos=0;nPos<strTemp.GetLength();nPos++)
				if(strTemp[nPos] == '\"') nCount++;

			//...if there is an odd number we need to add a quote to the end 
			//of the line
			if(nCount%2)
				strTemp += "\" &\n" + strIndent + "\t\"";
			else
				strTemp += " &\n" + strIndent + "\t";

            nLineLength = strLine.Delete(0,nNewLength)-nNewLength;
        }
        else
        {
            strTemp += strLine;
            nLineLength = 0;
        }
    }

    strLine = strTemp;
}

void SCLWriter::WriteVariableDef(const char *szName,int nOption,int nSize)
{
    CString strVarDef("\t");

    //Get the variable type
    if(nOption & VTCHAR)
        strVarDef += "CHARACTER";
    else
        if(nOption & VTINT)
            strVarDef += "Integer";
        else
        {
            ASSERT(FALSE);
        }

    if(nSize != 1)
    {
        CString strSize;
        strSize.Format("*%d",nSize);
        strVarDef += strSize;
    }

    strVarDef += "\t";
    strVarDef += szName;

    //Get scope
    if(nOption & VCLOCAL);
    else if(nOption & VCSCRIPT)
        strVarDef += ",SCRIPT";
    else if(nOption & VCGLOBAL)
        strVarDef += ",GLOBAL";
    else if(nOption & VCTHREAD)
        strVarDef += ",THREAD";
    else if(nOption & VCTESTWIDE)
        strVarDef += ",TEST-WIDE";
    else
    {
        ASSERT(FALSE);
    }

    m_arVariables.Add(strVarDef);
}

void SCLWriter::WriteTimer(const char *szName,bool bStart)
{
    CString strTimer;
    CString szTimerName(szName);

	int nLength = min(szTimerName.GetLength(),64);
    if(bStart)
    {
        strTimer.Format("\tStart Timer %s\n",szTimerName.Left(nLength));
    }
    else
    {
        strTimer.Format("\tEnd Timer %s\n",szTimerName.Left(nLength));
    }

    WriteString(strTimer);
}

void SCLWriter::WritePageTimer()
{
	CString strName;
	strName.Format(m_strPageTimerName,m_nPageTimer);

	CString str;
	str = "\n\tIf (USE_PAGE_TIMERS=1) then\n\t";
	
	//If previous line was a comment insert the timer before it
	CString strPrevLine;
	if(m_strPrevLine.Find("\t!")!=-1)
	{
		strPrevLine = m_strPrevLine;
		m_strPrevLine.Empty();
	}
	
	WriteString(str);
	WriteTimer(strName,m_bMatchedTimers);
	str = "\tEndIf\n\n";
	WriteString(str);
	
	if(m_bMatchedTimers)
	{
		WriteTimerDef(strName);
		m_bMatchedTimers = false;
	}
	else
	{
		m_nPageTimer++;
		m_bMatchedTimers = true;
	}
	if(!strPrevLine.IsEmpty())
		WriteString(strPrevLine);
}

void SCLWriter::WriteTimerDef(const char *szName)
{
    CString strTimDef;

    // Trim the timer variable if over 64 characters
    CString szTimerName = szName;

    strTimDef.Format("\tTimer\t\t\t%s",szTimerName.Left(64));
    m_arVariables.Add(strTimDef);
}

void SCLWriter::WriteSynchronize()
{
    CString strSynchronize;
    strSynchronize = m_strDefIndent + "SYNCHRONIZE REQUESTS\n\n";

	//If previous line was a comment insert the timer before it
	CString strPrevLine;
	if(m_strPrevLine.Find("\t!")!=-1)
	{
		strPrevLine = m_strPrevLine;
		m_strPrevLine.Empty();
	}

    WriteString(strSynchronize);

	if(!strPrevLine.IsEmpty())
		WriteString(strPrevLine);
}

void SCLWriter::WriteSetVar(const char *szName,const char *szValue)
{
	CString strSetVar;
	strSetVar.Format("%sSet %s = \"%s\"\n",m_strDefIndent,szName,szValue);
	ValidateString(strSetVar,m_strDefIndent.GetLength());
	WriteString(strSetVar);
}

