/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// BitmapCombobox.cpp : implementation file
//

#include "stdafx.h"
#include "BitmapCombobox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CBitmapCombobox, CComboBox)
//{{AFX_MSG_MAP(CBitmapCombobox)
// NOTE - the ClassWizard will add and remove mapping macros here.
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


//------------------------------------------------------------------------
//constructor of the combobox : parameter = the width and the height of the bmp
CBitmapCombobox::CBitmapCombobox(UINT nSizex /*=16*/ , UINT nSizey /*= 16*/ )
// 16 = standard bitmap size
{
    m_sizeBitmap.cx = nSizex;
    m_sizeBitmap.cy = nSizey;
}

//------------------------------------------------------------------------

CBitmapCombobox::~CBitmapCombobox()
{}

//------------------------------------------------------------------------

//Add an item in the combo box
//parameters : Resource ID + string associated

int CBitmapCombobox::AddBitmap(UINT _nIDResource, LPCTSTR lpszString)
{
    int nIndex = CComboBox::AddString(lpszString);

    if (nIndex != CB_ERR && nIndex != CB_ERRSPACE)
        SetItemData(nIndex, (DWORD)_nIDResource);

    return nIndex;
}

//------------------------------------------------------------------------

//Insert an item in the combo box at the location nIndex
//Parameters : Location (nIndex = zero based index) +  Resource ID + string associated
int CBitmapCombobox::InsertBitmap(int nIndex, UINT _nIDResource, LPCTSTR lpszString)
{

    nIndex = CComboBox::InsertString(nIndex, lpszString);

    if (nIndex != CB_ERR && nIndex != CB_ERRSPACE)
        SetItemData(nIndex, (DWORD)_nIDResource);

    return nIndex;
}

//------------------------------------------------------------------------

//Select the item located at nIndex (zero-based index) in the list of the combo
int CBitmapCombobox::SelectBitmap(int nIndex)
{
    return CComboBox::SetCurSel(nIndex);
}

//------------------------------------------------------------------------

//Delete the item located at nIndex (zero-based index) in the list of the combo
int CBitmapCombobox::DeleteBitmap(int nIndex)
{
    return CComboBox::DeleteString(nIndex);
}

//------------------------------------------------------------------------

//Can't do this
int CBitmapCombobox::AddString(LPCTSTR lpszString)
{
    ASSERT(FALSE); // cannot add strings
    return CB_ERR;
}

//------------------------------------------------------------------------

//Can't do this
int CBitmapCombobox::InsertString(int nIndex, LPCTSTR lpszString)
{
    ASSERT(FALSE); // cannot insert strings
    return CB_ERR;
}

//------------------------------------------------------------------------

//Can't do this
int CBitmapCombobox::DeleteString(int nIndex)
{
    ASSERT(FALSE); // cannot delete strings
    return CB_ERR;
}

//------------------------------------------------------------------------

//The combobox is OwnerDrawVariable so we have to set its height
void CBitmapCombobox::MeasureItem(LPMEASUREITEMSTRUCT lpMIS)
{
    lpMIS->itemHeight = (m_sizeBitmap.cy + 2);
}

//------------------------------------------------------------------------

//Draw the combo box
void CBitmapCombobox::DrawItem(LPDRAWITEMSTRUCT lpDIS)
{
    CDC* pDC = CDC::FromHandle(lpDIS->hDC);

    if (!IsWindowEnabled())
    {
        CBrush brDisabled(RGB(192,192,192)); // light gray
        CBrush* pOldBrush = pDC->SelectObject(&brDisabled);
        CPen penDisabled(PS_SOLID, 1, RGB(192,192,192));
        CPen* pOldPen = pDC->SelectObject(&penDisabled);
        OnOutputBitmap(lpDIS, FALSE);
        pDC->SelectObject(pOldBrush);
        pDC->SelectObject(pOldPen);
        return;
    }

    // Selected
    if ((lpDIS->itemState & ODS_SELECTED)
            && (lpDIS->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)))
    {
        CBrush brHighlight(::GetSysColor(COLOR_HIGHLIGHT));
        CBrush* pOldBrush = pDC->SelectObject(&brHighlight);
        CPen penHighlight(PS_SOLID, 1, ::GetSysColor(COLOR_HIGHLIGHT));
        CPen* pOldPen = pDC->SelectObject(&penHighlight);
        pDC->Rectangle(&lpDIS->rcItem);
        pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
        pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
        OnOutputBitmap(lpDIS, TRUE);
        pDC->SelectObject(pOldBrush);
        pDC->SelectObject(pOldPen);
    }

    // De-Selected
    if (!(lpDIS->itemState & ODS_SELECTED)
            && (lpDIS->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)))
    {
        CBrush brWindow(::GetSysColor(COLOR_WINDOW));
        CBrush* pOldBrush = pDC->SelectObject(&brWindow);
        CPen penHighlight(PS_SOLID, 1, ::GetSysColor(COLOR_WINDOW));
        CPen* pOldPen = pDC->SelectObject(&penHighlight);
        pDC->Rectangle(&lpDIS->rcItem);
        pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));
        pDC->SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
        OnOutputBitmap(lpDIS, FALSE);
        pDC->SelectObject(pOldBrush);
        pDC->SelectObject(pOldPen);
    }

    // Focus
    if (lpDIS->itemAction & ODA_FOCUS)
    {
        pDC->DrawFocusRect(&lpDIS->rcItem);
    }
}

//------------------------------------------------------------------------

//Draw Bmp in the combo and set the text
void CBitmapCombobox::OnOutputBitmap(LPDRAWITEMSTRUCT lpDIS, BOOL bSelected)
{
    if (GetCurSel() == CB_ERR || GetCount() == 0)
        return; // no item selected

    CString strText;
    CBitmap * pbmOld = NULL ;
    CDC       dcMem ;
    CBitmap   bmp;

    CDC* pDC = CDC::FromHandle(lpDIS->hDC);

    bmp.LoadBitmap((UINT)lpDIS->itemData);

    dcMem.CreateCompatibleDC(pDC);
    pbmOld = dcMem.SelectObject(&bmp);

    pDC->BitBlt(lpDIS->rcItem.left ,
                lpDIS->rcItem.top ,
                m_sizeBitmap.cx ,
                m_sizeBitmap.cy ,
                &dcMem, 0, 0, SRCCOPY);

    dcMem.SelectObject(pbmOld);
    dcMem.DeleteDC();

    GetLBText( lpDIS->itemID, strText );
    pDC->TextOut( lpDIS->rcItem.left + m_sizeBitmap.cx + 2, lpDIS->rcItem.top + m_sizeBitmap.cy/4 - 2,
                  strText, strText.GetLength() );

}


