// HTTPGraphs.cpp: implementation of the CHTTPGraphs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPGraphs.h"
#include "HTTPStatsOcx.h"
#include "TimerList.h"
#include "GraphTypes.h"
#include "GraphWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHTTPGraphs::CHTTPGraphs():
	m_mapURLs(500),

	m_strDataList("HTTP Data List"),
	m_strBytesPerSecVTime("HTTP Monitored Bytes / Sec v Elapsed Time"),
	m_strRespTimeVNumResps("HTTP Response Time v Number Of Responses"),
	m_strErrorsVUsers("HTTP Errors v Active Users"),
	m_strErrorsVTime("HTTP Errors v Elapsed Time"),
	m_strRespsVTime("HTTP Responses v Elapsed Time"),
	m_strRespTimeVTime("HTTP Response Time v Elapsed Time"),
	m_strUsersVTime("HTTP Active Users v Elapsed Time"),

	m_strTimerList("Timer List"),
	m_strTimersVUsers("Timer Values v Active Users"),
	m_strTimersVTime("Timer Values v Elapsed Time")
{
}

CHTTPGraphs::~CHTTPGraphs()
{
	for(int nNdx=0;nNdx<m_arHttp.GetSize();nNdx++)
		delete m_arHttp[nNdx];
	m_arHttp.RemoveAll();

	POSITION pos = m_listTimers.GetHeadPosition();
	while(pos)
		delete m_listTimers.GetNext(pos);
	m_listTimers.RemoveAll();

	pos = m_listVUs.GetHeadPosition();
	while(pos)
		delete m_listVUs.GetNext(pos);
	m_listVUs.RemoveAll();
}

void CHTTPGraphs::LoadRun(const CString& strRunFolder)
{
	m_strLoadedRun = strRunFolder;
	m_strLoadedRun.TrimRight("\\");
	LoadHttp();
	LoadTimers();
	LoadVUData();

	int nSlashPos = m_strLoadedRun.ReverseFind('\\');
	CString strRunName(m_strLoadedRun.Mid(nSlashPos+1));
	
	int nDay,nMonth,nYear,nHour,nMinute,nSecond;
	char *pszToken = strtok(strRunName.GetBuffer(0),"-");
	if(pszToken)
		nDay = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMonth = atoi(pszToken);

	if(pszToken = strtok(NULL," "))
		nYear = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nHour = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMinute = atoi(pszToken);

	if(pszToken = strtok(NULL,"."))
		nSecond = atoi(pszToken);

	CTime T(nYear,nMonth,nDay,nHour,nMinute,nSecond);
	m_dwTestStartTime = T.GetTime();
}

void CHTTPGraphs::GetAvailableGraphs(CStringArray& arstrGraphs)
{
	arstrGraphs.RemoveAll();

	if(m_arHttp.GetSize())
	{
		arstrGraphs.Add(m_strDataList);
		arstrGraphs.Add(m_strBytesPerSecVTime);
		arstrGraphs.Add(m_strRespTimeVNumResps);
		arstrGraphs.Add(m_strErrorsVUsers);
		arstrGraphs.Add(m_strErrorsVTime);
		arstrGraphs.Add(m_strRespsVTime);
		arstrGraphs.Add(m_strRespTimeVTime);
	}

	if(m_listVUs.GetCount())
	{
		arstrGraphs.Add(m_strUsersVTime);
	}

	if(m_listTimers.GetCount())
	{
		arstrGraphs.Add(m_strTimerList);
		arstrGraphs.Add(m_strTimersVUsers);
		arstrGraphs.Add(m_strTimersVTime);
	}
}

CWnd* CHTTPGraphs::DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	if(strName == m_strDataList)
	{
		return LoadHTTPDataList(pParent,rcBounds);
	}
	else
	if(strName == m_strTimerList)
	{
		return LoadTimerList(pParent,rcBounds);
	}
	else
	if(strName == m_strBytesPerSecVTime ||	//All http only graphs
		strName == m_strRespsVTime ||
		strName == m_strErrorsVUsers ||
		strName == m_strErrorsVTime ||
		strName == m_strRespTimeVTime ||
		strName == m_strRespTimeVNumResps)
	{
		return LoadHttpGraph(strName,rcBounds,pParent);
	}
	else				
	if(strName == m_strTimersVUsers ||		//all timer only graphs
		strName == m_strTimersVTime)
	{
		return LoadTimerGraph(strName,rcBounds,pParent);
	}
	else
	if(strName == m_strUsersVTime)
	{
		return LoadVUGraph(strName,rcBounds,pParent);
	}
	
	return NULL;
}

CGraphWnd* CHTTPGraphs::LoadHttpGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CHttpGraph *pGraph = NULL;
	if(strName == m_strBytesPerSecVTime)
		pGraph = new CThroughputGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespsVTime)
		pGraph = new CHitsPerSecondGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strErrorsVUsers)
		pGraph = new CErrorsPerUserGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strErrorsVTime)
		pGraph = new CErrorsTimeGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespTimeVTime)
		pGraph = new CPagesPerSecondGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespTimeVNumResps)
		pGraph = new CAvPageRTPerUserGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_arHttp,&m_mapURLs,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();

	return pGraphWnd;
}

CGraphWnd* CHTTPGraphs::LoadTimerGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CTimerGraph *pGraph = NULL;
	if(strName == m_strTimersVUsers)
		pGraph = new CRTATGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strTimersVTime)
		pGraph = new CRTTimeGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_listTimers,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();

	return pGraphWnd;
}

CGraphWnd* CHTTPGraphs::LoadVUGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CVUsersTimeGraph *pGraph = NULL;
	if(strName == m_strUsersVTime)
		pGraph = new CVUsersTimeGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_listVUs,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();

	return pGraphWnd;
}

CTimerList* CHTTPGraphs::LoadTimerList(CWnd* pParent,const CRect& rcBounds)
{
	LoadTimers();

	CTimerList *pTimerList = new CTimerList(m_listTimers);
	pTimerList->Create(WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	return pTimerList;
}

CHttpStatsOcx* CHTTPGraphs::LoadHTTPDataList(CWnd *pParent,const CRect& rcBounds)
{
	CHttpStatsOcx *pHttpStatsOcx = new CHttpStatsOcx;
	pHttpStatsOcx->Create("",WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);
	
	CString strStatTarget = m_strLoadedRun + "\\HTTP_ALL.stat";
	CString strURLTarget = m_strLoadedRun + "\\HTTP_ALL.urls";
	CFile fStatTarget(strStatTarget,CFile::modeCreate|CFile::modeWrite);
	CFile fUrlTarget(strURLTarget,CFile::modeCreate|CFile::modeWrite);

	//Concatenate all the http files in this test run directory
	DWORD dwRead=0;
	const DWORD dwBuffSize=4096;
	BYTE *pBuffer = new BYTE[dwBuffSize];

	CFileFind FF;
	if(!FF.FindFile(m_strLoadedRun + "\\TEW_*.stat"))
		return NULL;
	
	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		CString strStatFile = FF.GetFilePath();

		int nDotPos = strStatFile.ReverseFind('.');
		CString strUrlFile = strStatFile.Left(nDotPos) + ".urls";

		CFile fStats,fUrls;
			
		// Get the path for the url file and open it
		if(fUrls.Open(strUrlFile,CFile::modeRead|CFile::typeBinary) &&
			fStats.Open(strStatFile,CFile::modeRead|CFile::typeBinary))
		{
			DWORD dwLength = fUrls.GetLength();

			//copy urls file to concat file
			while(dwLength>0)
			{
				if(dwLength > dwBuffSize)
					dwRead = fUrls.Read(pBuffer,dwBuffSize);
				else
					dwRead = fUrls.Read(pBuffer,dwLength);

				fUrlTarget.Write(pBuffer,dwRead);
				dwLength -= dwRead;
			}
			fUrls.Close();

			//Copy stats to concat file
			dwLength = fStats.GetLength();

			while(dwLength>0)
			{
				if(dwLength > dwBuffSize)
					dwRead = fStats.Read(pBuffer,dwBuffSize);
				else
					dwRead = fStats.Read(pBuffer,dwLength);

				fStatTarget.Write(pBuffer,dwRead);
				dwLength -= dwRead;
			}
			fStats.Close();
		}
	}

	fStatTarget.Close();
	fUrlTarget.Close();
	delete []pBuffer;

	pHttpStatsOcx->LoadStatsFile(strStatTarget);

	DeleteFile(strStatTarget);
	DeleteFile(strURLTarget);

	return pHttpStatsOcx;
}

void CHTTPGraphs::LoadTimers()
{
	POSITION pos = m_listTimers.GetHeadPosition();
	while(pos)
		delete m_listTimers.GetNext(pos);
	m_listTimers.RemoveAll();

	CString strTimerFile(m_strLoadedRun + "\\Timer.txt");
    try
    {
        CStdioFile TimerFile(strTimerFile,CFile::modeRead);

		CArray <Timer *,Timer*> arTimers;
        CString strDescription;
        CString strTimer;
        while(TimerFile.ReadString(strTimer))
        {
			Timer *pTimer = new Timer;

			char *pszToken = strtok(strTimer.GetBuffer(0),",");
			if(pszToken)	//Timestamp
				pTimer->dwTimeStamp = atoi(pszToken);

			pszToken = strtok(NULL,",");
			if(pszToken)	//Value
				pTimer->dValue = ((double)atoi(pszToken))/100.0;	//Convert to Seconds

			pszToken = strtok(NULL,",");
			if(pszToken)	//Num Active Users
				pTimer->dwActiveUsers =	atoi(pszToken);

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Test Name
				pTimer->strTestName = pszToken;

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Thread Name
				pTimer->strThreadName = pszToken;

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Script Name
				pTimer->strScriptName = pszToken;

			pszToken = strtok(NULL,",");
			if(pszToken)	//Timer name
				pTimer->strTimerName = pszToken;
            
			arTimers.Add(pTimer);
        }

		int nNumTimers = arTimers.GetSize();
		qsort(arTimers.GetData(),nNumTimers,sizeof(Timer*),&CompareTimerStart);

		for(int nNdx=0;nNdx<nNumTimers;nNdx++)
			m_listTimers.AddTail(arTimers[nNdx]);
    }
	catch(CFileException *e)
	{
		e->Delete();
	}
    catch (...)
    {
	}
}

int CHTTPGraphs::CompareTimerStart( const void *arg1, const void *arg2 )
{
	Timer *a = *(Timer **)arg1;
	Timer *b = *(Timer **)arg2;

	if(a->dwTimeStamp > b->dwTimeStamp)
		return 1;
	else
	if(a->dwTimeStamp < b->dwTimeStamp)
		return -1;
	
	return 0;
}

void CHTTPGraphs::LoadHttp()
{
	for(int nNdx=0;nNdx<m_arHttp.GetSize();nNdx++)
		delete m_arHttp[nNdx];
	m_arHttp.RemoveAll();

	CFileFind FF;
	if(!FF.FindFile(m_strLoadedRun + "\\TEW_*.stat"))
		return;

	int nCount=0;
 	CFileStatus fsUrls,fsStat;
	BOOL bContinue = TRUE;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		CString strStatFile( FF.GetFilePath() );
		CString strUrlFile ( m_strLoadedRun + "\\" + FF.GetFileTitle() + ".urls");
		
		//Check url file exists
		if(!CFile::GetStatus(strUrlFile,fsUrls))
			continue;

		//Check stat file exists and get its size
		if(!CFile::GetStatus(strStatFile,fsStat) || !fsStat.m_size)
			continue;

		//Calculate number of stats in file
	    int nStatSize = sizeof(HttpData)-sizeof(int);
		int nNumStats = fsStat.m_size/nStatSize;

		//Resize data array to fit new stats
		m_arHttp.SetSize( m_arHttp.GetSize() + nNumStats);

		//Open the stats file
        HANDLE hStatFile = CreateFile(strStatFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hStatFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hStatMap = CreateFileMapping(hStatFile,NULL,PAGE_READONLY,0,0,"StatsData");
        if(hStatMap == NULL)
            return;

        LPBYTE pStatBuffer = (LPBYTE)MapViewOfFile(hStatMap,FILE_MAP_READ,0,0,0);
        LPBYTE pStatBufPos = pStatBuffer;
		
		//Open the urls file
        HANDLE hUrlFile = CreateFile(strUrlFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hUrlFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hUrlMap = CreateFileMapping(hUrlFile,NULL,PAGE_READONLY,0,0,"UrlData");
        if(hUrlMap == NULL)
            return;

        LPBYTE pUrlBuffer = (LPBYTE)MapViewOfFile(hUrlMap,FILE_MAP_READ,0,0,0);
        char *pUrlBufPos = (char*)pUrlBuffer;

		for(int nStat=0;nStat<nNumStats;nStat++)
        {
            int nIndex=-1;
            if(!m_mapURLs.Lookup(pUrlBufPos,nIndex))
            {
                nIndex = m_mapURLs.GetCount();
                m_mapURLs.SetAt(pUrlBufPos,nIndex);
            }
            pUrlBufPos += strlen(pUrlBufPos)+1;

            //Do the load
            HttpData *pData = m_arHttp[nStat+nCount] = new HttpData;
            memcpy(pData,&pStatBuffer[nStatSize*nStat],nStatSize);

            pData->nURLNdx = nIndex;
        }
        nCount += nNumStats;

        //Clean up
        UnmapViewOfFile(pStatBuffer);
        CloseHandle(hStatMap);
        CloseHandle(hStatFile);
        UnmapViewOfFile(pUrlBuffer);
        CloseHandle(hUrlMap);
        CloseHandle(hUrlFile);
    }
    
    //Sort data into time order
    qsort(m_arHttp.GetData(),(size_t)m_arHttp.GetSize(),sizeof(HttpData**),SortHttpData);
}

int CHTTPGraphs::SortHttpData( const void *arg1, const void *arg2 )
{
	HttpData *pStat1 = *(HttpData**)arg1;
	HttpData *pStat2 = *(HttpData**)arg2;
    DWORD dwTime1 = pStat1->dwTimeStamp;
    DWORD dwTime2 = pStat2->dwTimeStamp;

    if(dwTime1>dwTime2)
        return 1;

    return ((dwTime2>dwTime1) ? -1 : 0);
}

void CHTTPGraphs::LoadVUData()
{
	POSITION pos = m_listVUs.GetHeadPosition();
	while(pos)
		delete m_listVUs.GetNext(pos);
	m_listVUs.RemoveAll();

	CString strPath(m_strLoadedRun + "\\VUsersLog.txt");
    
    try
    {
        CStdioFile VUFile(strPath,CFile::modeRead);

		CString strTimeStamp;
		CString strCount;
        CString strVUData;
		char *pszToken = NULL;
        while(VUFile.ReadString(strVUData))
        {
			VUData *pVD = new VUData;
			try
			{
				if(!(pszToken = strtok(strVUData.GetBuffer(0),","))) throw -1;
				
				pVD->dwTimeStamp = atoi(pszToken);
				
				if(!(pszToken = strtok(NULL,","))) throw -1;

				pVD->dwCount = atoi(pszToken);
			}
			catch(int)
			{
				delete pVD;
				continue;
			}

			m_listVUs.AddTail(pVD);
		}

		VUFile.Close();
	}
	catch(CFileException *e)
	{
		e->Delete();
	}
	catch(...)
	{
	}
}

