/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Modeller.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cmaxafx.h>
#include <ConstantesLib.h>
#include <RecoverManager.h>
#include <wininet.h>

#include "Constantes.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "ModellerDoc.h"
#include "SubroutineDocument.h"
#include "SubroutineChildFrame.h"
#include "SubroutineView.h"
#include "BasedDocument.h"
#include "EnumModellerData.h"
#include "ModellerView.h"
#include "Modeller.h"
#include "AboutDlg.h"
#include "CaptureOpenDlg.h"
#include "SelectBrowserDlg.h"
#include "OpenDocumentDlg.h"
#include "GatewayOptionsDlg.h"
#include "VariableOptionDlg.h"
#include <IE4Information.h>
#include <IE5Information.h>
#include <NS47Information.h>
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//--------------------------------------------------------------------------

CModellerApp    theApp;

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CModellerApp, CWinApp)
//{{AFX_MSG_MAP(CModellerApp)
ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
ON_COMMAND(ID_OPTIONS_BROWSER, OnOptionsBrowser)
ON_COMMAND(ID_OPTIONS_GATEWAY, OnOptionsGateway)
ON_COMMAND(ID_OPTIONS_VARIABLES, OnOptionsVariables)
ON_COMMAND(ID_TOOLS_EDIT, OnToolsEdit)
//}}AFX_MSG_MAP
// Standard file based document commands
ON_COMMAND(ID_FILE_NEW, OnFileNew)
ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
// Standard print setup command
ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

CModellerApp::CModellerApp() :
        m_bRecording( FALSE ),
        m_bReplay( FALSE ),
        m_bRegisterClass( FALSE) ,
        m_bOffLine( FALSE ),
        m_bTrace(FALSE),
        m_bConsole(FALSE),
		m_bAutoCookieGeneration(TRUE)
{
    memset(&m_ProcessInformation, 0, sizeof(m_ProcessInformation));
}

CModellerApp::~CModellerApp()
{
    m_CaptureGateway.stopGateway() ;
    TerminateProcess(m_ProcessInformation.hProcess, 0xFFFF);
}

//--------------------------------------------------------------------------

CWnd *CModellerApp::FirstInstance()
{
    CWnd *pWndPrev, *pWndChild;
    //----

    // Determine if another window with our class name exists...
    if (pWndPrev = CWnd::FindWindow( m_strClassName, NULL ))
    {
        // if so, does it have any popups?
        pWndChild = pWndPrev->GetLastActivePopup();

        // If iconic, restore the main window
        if (pWndPrev->IsIconic())
            pWndPrev->ShowWindow(SW_RESTORE);

        // Bring the main window or its popup to
        // the foreground
        pWndChild->SetForegroundWindow();

        // and we are done activating the previous one.
        return pWndChild;
    }
    // First instance. Proceed as normal.
    else
        return NULL;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::InitInstance()
{
    // 1- Set App Class Name
    m_strClassName.LoadString( IDS_APPCLASS_NAME ) ;

    // 2 - Check if the App already exists
    CWnd *pWndOtherInstance = NULL ;
    // not exists : create App
    if((pWndOtherInstance = FirstInstance()) == NULL)
    {
		//Check for Unrestored Browser parameters file
		CRecoverManager RecoverManager ;
		RecoverManager.Show() ;

        // 3- Init the OffLine state of the browser
        m_bOffLine = IsGlobalOffLine() ;

        // 4- Init App
        if( !InitializeApp() )
            return FALSE ;
        
		//5 - Apply App project Info
        return ApplyProjectInfo( m_lpCmdLine ) ;
    }
    //exists : send message to the mainframe of the existing app
    else
    {
        COPYDATASTRUCT DataStruct ;
        //----
        DataStruct.dwData = 0 ;
        DataStruct.cbData = sizeof( TCHAR) * (strlen( m_lpCmdLine ) + 1 ) ;
        DataStruct.lpData = m_lpCmdLine ;

        pWndOtherInstance->SendMessage( WM_COPYDATA , 0, (LPARAM) &DataStruct );
        return FALSE ;
    }
}

//--------------------------------------------------------------------------

BOOL CModellerApp::ApplyProjectInfo( const CString& strArgList )
{
	//Only file name should be being passed to modeller
	CString strArgs(strArgList);
	strArgs.Remove('\"');
	if(strArgList.IsEmpty() )
        OnFileNew();
	else
		OpenDocumentFromArgs(strArgs) ;

    return TRUE ;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::InitializeApp()
{
    // 1 - Register our unique class name that we wish to use : it is associated with the CMainFrame
    // See BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
    WNDCLASS wndcls;
    //----
    memset(&wndcls, 0, sizeof(WNDCLASS));   // start with NULL defaults

    wndcls.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
    wndcls.lpfnWndProc = ::DefWindowProc;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.hIcon = LoadIcon(IDR_MAINFRAME); // or load a different icon
    wndcls.hCursor = LoadCursor( IDC_ARROW );
    wndcls.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wndcls.lpszMenuName = NULL;

    // Specify our own class name for using FindWindow later

    wndcls.lpszClassName = m_strClassName ;


    // Register new class and exit if it fails
    if(!AfxRegisterClass(&wndcls))
    {
        TRACE("Class Registration Failed\n");
        return FALSE;
    }

    m_bRegisterClass = TRUE ;

    //2 - Init App
    CWinApp::InitInstance() ;
    AfxEnableControlContainer();

    // Standard initialization
    // If you are not using these features and wish to reduce the size
    //  of your final executable, you should remove from the following
    //  the specific initialization routines you do not need.

#ifdef _AFXDLL
    Enable3dControls();         // Call this when using MFC in a shared DLL
#else
    Enable3dControlsStatic();   // Call this when linking to MFC statically
#endif

    // Change the registry key under which our settings are stored.
    // TODO: You should modify this string to be something appropriate
    // such as the name of your company or organization.
    SetRegistryKey(_T("OpenSTA"));

    ReadPreferences();
    //Init Serialization object
    m_Serialization.SetLocation(    CString("Software\\OpenSTA\\")  +
                                    CString(m_pszAppName)                    +
                                    CString("\\Serialization")
                               );
    m_Serialization.Read() ;

    LoadStdProfileSettings();  // Load standard INI file options (including MRU)

    //3 - Add New Document model
    CMultiDocTemplate* pDocTemplate = new CMultiDocTemplate(
                                          IDR_MODELLTYPE,
                                          RUNTIME_CLASS(CModellerDoc),
                                          RUNTIME_CLASS(CChildFrame), // custom MDI child frame
                                          RUNTIME_CLASS(CModellerView));
    AddDocTemplate(pDocTemplate);
    m_pCaptureTemplate = pDocTemplate;

    pDocTemplate = new CMultiDocTemplate(
                       IDR_SUBROUTYPE,
                       RUNTIME_CLASS(CSubroutineDocument),
                       RUNTIME_CLASS(CSubroutineChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CSubroutineView));
    AddDocTemplate(pDocTemplate);
    m_pSubroutineTemplate = pDocTemplate;

    //4 - create main MDI Frame window
    CMainFrame* pMainFrame = new CMainFrame;
    if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
        return FALSE;
    m_pMainWnd = pMainFrame;

    // register CodeMax
    CMRegisterControl();

    m_RepositoryAPI.SetWindow(AfxGetMainWnd());

    // The main window has been initialized, so show and update it.
    pMainFrame->ShowWindow(m_nCmdShow);
    pMainFrame->UpdateWindow();

    return TRUE ;
}

//--------------------------------------------------------------------------

void CModellerApp::OpenDocumentFromArgs(const CString& strFileName)
{
	//Look at extension, if it is .HTP look for a capture file
	//if it exist open as a capture otherwise as an edit
	int nSlashPos = strFileName.ReverseFind('\\');
	CString strFile(strFileName.Mid(nSlashPos+1));

	CString strExt;
	int nDotPos = strFile.ReverseFind('.');
	if(nDotPos!=-1)
		strExt = strFile.Mid(nDotPos);
	
	if(strExt.CompareNoCase(STR_SCL_EXTENSION)==0)
		OpenDocument(m_pCaptureTemplate,strFileName);
	else
		OpenDocument(m_pSubroutineTemplate,strFileName);
}

//--------------------------------------------------------------------------

BOOL CModellerApp::DoesDocumentExist(CString _strFileName)
{
    BOOL bRet = FALSE;

    CStringArray strFileArray ;
    long nCount = 0 ;
    //----
    nCount = m_RepositoryAPI.GetProjectCaptureFiles(strFileArray ) ;

    for( int nIndex = 0 ; nIndex < nCount ; nIndex ++ )
    {
        CString strFileName ;
        strFileName = strFileArray.GetAt( nIndex ) ;

        if ( strFileName.CompareNoCase(_strFileName) == 0 )
            return TRUE;
    }

    return FALSE;
}


//--------------------------------------------------------------------------

void CModellerApp::OnFileNew()
{
    OpenDocument(m_pCaptureTemplate,"");
}

//---------------------------------------------------------------------------

void CModellerApp::OnFileOpen()
{
    CCaptureOpenDlg dlg;
    if(dlg.DoModal() != IDOK)
        return;

    OpenDocument(m_pCaptureTemplate,dlg.m_strCapture);
}

//---------------------------------------------------------------------------

void CModellerApp::OpenDocument(CDocTemplate *pTemplate ,CString _strFileName)
{
    CWaitCursor wc;

    if(!_strFileName.IsEmpty() && IsDocumentOpen(_strFileName, TRUE))
        return;

    CBasedDocument* pDoc = static_cast<CBasedDocument*>(pTemplate->OpenDocumentFile(_strFileName));
    ASSERT_KINDOF(CBasedDocument, pDoc);

	CSCLDocInfo *pDocInfo = pDoc->GetSCLDocInfo();

    //Create a temporary working directory
    CString strTmpPath = pDocInfo->GetTmpProjectPath();
    CString strWorkingFolder;
    GetTempFileName(strTmpPath,"CYR",0,strWorkingFolder.GetBuffer(MAX_PATH));
    strWorkingFolder.ReleaseBuffer();
    DeleteFile(strWorkingFolder);
    CreateDirectory(strWorkingFolder,NULL);

    if(_strFileName.IsEmpty())
        pDocInfo->SetFileInfo(UNTITLED_SCL_DOCUMENT);
    else
    {
		pDocInfo->SetOriginalPath(_strFileName);

		int nSlashPos = _strFileName.ReverseFind('\\');
		if(nSlashPos == -1)
			nSlashPos = 0;

		pDocInfo->SetFileInfo(_strFileName.Mid(nSlashPos+1));
	}

    pDocInfo->SetWorkingFolder(strWorkingFolder + "\\");

    pDoc->RetrieveFromRepository();
    if(!_strFileName.IsEmpty())
    {
        pDoc->SetTitle(_strFileName);
    }
}

//---------------------------------------------------------------------------

CString CModellerApp::GetRegistryPath()
{
    CString strPath ;
    //---
    strPath = _T("Software\\" ) ;
    strPath +=  m_pszRegistryKey ;
    strPath +=  _T("\\" ) ;
    strPath += m_pszAppName ;
    return strPath ;
}

//--------------------------------------------------------------------------

int CModellerApp::ExitInstance()
{
    if(m_bRegisterClass)
        ::UnregisterClass(m_strClassName, AfxGetInstanceHandle());

    SetGlobalOffLine( m_bOffLine ) ;
    m_Serialization.Save() ;

    SavePreferences();

    return CWinApp::ExitInstance();
}

//--------------------------------------------------------------------

void CModellerApp::SetGlobalOffLine( BOOL bOffLine )
{
    INTERNET_CONNECTED_INFO ci;
    memset(&ci, 0, sizeof(ci));
    if(bOffLine)
    {
        ci.dwConnectedState = INTERNET_STATE_DISCONNECTED_BY_USER;
        ci.dwFlags = ISO_FORCE_DISCONNECTED;
    }
    else
        ci.dwConnectedState = INTERNET_STATE_CONNECTED;

    InternetSetOption(NULL,
                      INTERNET_OPTION_CONNECTED_STATE, &ci, sizeof(ci));
}

//--------------------------------------------------------------------

BOOL CModellerApp::IsGlobalOffLine()
{
    DWORD dwState = 0;
    DWORD dwSize = sizeof(DWORD);
    BOOL bRet = FALSE;

    if(InternetQueryOption(NULL,
                           INTERNET_STATE_DISCONNECTED_BY_USER, &dwState, &dwSize))
        bRet = TRUE;


    return bRet;
}

//--------------------------------------------------------------------

// App command to run the dialog
void CModellerApp::OnAppAbout()
{
    CAboutDlg aboutDlg;
    aboutDlg.DoModal();
}
//--------------------------------------------------------------------

BOOL CModellerApp::IsDocumentOpen(CString _strFileName, BOOL bActivate)
{
	CMDIFrameWnd *pParent = (CMDIFrameWnd*)AfxGetMainWnd();
	if(!pParent)
		return false;

	CWnd *pMDIClient = CWnd::FromHandle(pParent->m_hWndMDIClient);
	if(!pMDIClient)
		return false;

	CWnd *pChild = pMDIClient->GetWindow(GW_CHILD);
	while(pChild)
	{
		CBasedDocument* pDoc = (CBasedDocument*)((CMDIChildWnd*)pChild)->GetActiveDocument();
        if(pDoc->GetSCLDocInfo()->GetOriginalPath() == _strFileName)
		{
			if(bActivate)
				((CMDIChildWnd*)pChild)->MDIActivate();
			return true;
		}
		pChild = pChild->GetWindow(GW_HWNDNEXT);
	}
	return false;
}

void CModellerApp::OnOptionsBrowser()
{
    CSelectBrowserDlg dlg;
    dlg.SetCurrentBrowser(m_nBrowserID);
    if(dlg.DoModal() != IDOK)
        return;
    m_nBrowserID = dlg.GetCurrentBrowser();
}

void CModellerApp::ReadPreferences()
{
    m_strVariablePrefix = GetProfileString("Editor","Variable Prefix","");

    m_bConsole = GetProfileInt("Gateway","Console",false);
    m_bTrace = GetProfileInt("Gateway","Trace",false);
	m_bAutoCookieGeneration = GetProfileInt("Gateway","Auto Cookie Gen",true);

    m_GatewayData.m_bLocal = GetProfileInt("Gateway","Local",TRUE);
    m_GatewayData.m_lAdministrationPort  = GetProfileInt("Gateway","Admin Port",3000);
    m_GatewayData.m_lListenPort = GetProfileInt("Gateway","Listen Port",81);

    //Address
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress = GetProfileString("Gateway","Proxy Address"," ");
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lAddressPort = GetProfileInt("Gateway","Proxy Port",0);

    //Secure
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxySecure = GetProfileString("Gateway","Proxy Secure"," ");
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lSecurePort         = GetProfileInt("Gateway","Secure Port",0);
    ;
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal   = GetProfileInt("Gateway","Bypass Proxy",FALSE);

    //No proxy
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy = GetProfileString("Gateway","No Proxy"," ");

    m_nBrowserID = GetProfileInt("Gateway","Browser",ID_BROWSER_IE5);
}

void CModellerApp::SavePreferences()
{
    WriteProfileString("Editor","Variable Prefix",m_strVariablePrefix);

    WriteProfileInt("Gateway","Console",m_bConsole);
    WriteProfileInt("Gateway","Trace",m_bTrace);
    WriteProfileInt("Gateway","Auto Cookie Gen",m_bAutoCookieGeneration);

    WriteProfileInt("Gateway","Local",m_GatewayData.m_bLocal);
    WriteProfileInt("Gateway","Admin Port",m_GatewayData.m_lAdministrationPort);
    WriteProfileInt("Gateway","Listen Port",m_GatewayData.m_lListenPort);

    //Address
    WriteProfileString("Gateway","Proxy Address",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress);
    WriteProfileInt("Gateway","Proxy Port",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lAddressPort);

    //Secure
    WriteProfileString("Gateway","Proxy Secure",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxySecure);
    WriteProfileInt("Gateway","Secure Port",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lSecurePort);
    WriteProfileInt("Gateway","Bypass Proxy",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal);

    //No proxy
    WriteProfileString("Gateway","No Proxy",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy);

    WriteProfileInt("Gateway","Browser",m_nBrowserID);
}

//-----------------------------------------------------------------------

void CModellerApp::OnOptionsGateway()
{
    CBrowserInformation *pBrowserInfo;
    switch(m_nBrowserID)
    {
    case ID_BROWSER_IE4:
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE4Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_IE5 :
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_NS47 :
        pBrowserInfo = static_cast<CBrowserInformation*> (new CNS47Information(AfxGetMainWnd())) ;
        break ;
    default:
        ASSERT(FALSE);
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break;
    }

    BOOL bResult = pBrowserInfo->GetDefaultParameters(&m_GatewayData.m_ProxyInfo.m_LocalProxy);

    delete pBrowserInfo;
    if(!bResult)
        return;

    CGatewayOptionsDlg GatewayOptionsDlg(&m_GatewayData);
    GatewayOptionsDlg.m_bTrace = m_bTrace;
    GatewayOptionsDlg.m_bConsole = m_bConsole;
	GatewayOptionsDlg.m_bAutoCookieCheck = m_bAutoCookieGeneration;
    if(GatewayOptionsDlg.DoModal() != IDOK)
        return;

    m_bTrace = GatewayOptionsDlg.m_bTrace;
    m_bConsole = GatewayOptionsDlg.m_bConsole;
	m_bAutoCookieGeneration = GatewayOptionsDlg.m_bAutoCookieCheck;
}

//-----------------------------------------------------------------------

BOOL CModellerApp::IsCaptureLocal()
{
    return m_GatewayData.m_bLocal;
}

//-----------------------------------------------------------------------

void CModellerApp::OnOptionsVariables()
{
    CVariableOptionDlg dlg;
    dlg.m_strPrefix = m_strVariablePrefix;
    if(dlg.DoModal() == IDOK)
        m_strVariablePrefix = dlg.m_strPrefix;
}

//-----------------------------------------------------------------------

void CModellerApp::OnToolsEdit()
{
    COpenDocumentDlg dlg;
    if(dlg.DoModal() != IDOK)
        return;

    OpenDocument(m_pSubroutineTemplate,dlg.m_strFileName);
}

//-----------------------------------------------------------------------

void CModellerApp::GetGatewayInfomation( _TPreferencesGatewayData *pData )
{
	pData->m_bLocal = m_GatewayData.m_bLocal;
	pData->m_lAdministrationPort = m_GatewayData.m_lAdministrationPort;
	pData->m_lListenPort = m_GatewayData.m_lListenPort;
	pData->m_ProxyInfo.m_LocalProxy.m_bByPassProxyLocal = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_bByPassProxyLocal;
	pData->m_ProxyInfo.m_LocalProxy.m_lAddressPort = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_lAddressPort;
	pData->m_ProxyInfo.m_LocalProxy.m_lSecurePort = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_lSecurePort;
	pData->m_ProxyInfo.m_LocalProxy.m_strAddressesNoProxy = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_strAddressesNoProxy;
	pData->m_ProxyInfo.m_LocalProxy.m_strProxyAddress = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_strProxyAddress;
	pData->m_ProxyInfo.m_LocalProxy.m_strProxySecure = m_GatewayData.m_ProxyInfo.m_LocalProxy.m_strProxySecure;
	pData->m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal = m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal;
	pData->m_ProxyInfo.m_RemoteProxy.m_lAddressPort = m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lAddressPort;
	pData->m_ProxyInfo.m_RemoteProxy.m_lSecurePort = m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lSecurePort;
	pData->m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy = m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy;
	pData->m_ProxyInfo.m_RemoteProxy.m_strProxyAddress = m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress;
	pData->m_ProxyInfo.m_RemoteProxy.m_strProxySecure = 	m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxySecure;
}

//-----------------------------------------------------------------------
void CModellerApp::UpdateWindowText()
{
    CString strTitle("Script Modeler");
    ((CFrameWnd*)m_pMainWnd)->SetTitle(strTitle);
    m_pMainWnd->SetWindowText(strTitle);
}

//-----------------------------------------------------------------------

void CModellerApp::OpenFile(const CString& strPath)
{
	if(strPath.IsEmpty())
		return;
	
	OpenDocument(m_pSubroutineTemplate,strPath);
}
