/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OSGridView.cpp : implementation file
//

#include "stdafx.h"
#include "OSGridView.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//----------------------------------------------------------------------------

IMPLEMENT_DYNCREATE( COSGridView, CView)  //OS

//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COSGridView, CView)  //OS
//{{AFX_MSG_MAP(COSGridView)
ON_WM_SIZE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//----------------------------------------------------------------------------

COSGridView::COSGridView()
{}

//----------------------------------------------------------------------------


COSGridView::~COSGridView()
{}


/////////////////////////////////////////////////////////////////////////////
// COSGridView drawing
/////////////////////////////////////////////////////////////////////////////
// COSGridView diagnostics
//----------------------------------------------------------------------------


void COSGridView::AdjustRowsToHeight()
{
    CRect rect;
    GetClientRect(rect);

    int nRowToDisplay, nRowOfData;
    //---

    nRowToDisplay   = ( rect.Height() / m_wndGrid.GetDefCellHeight() ) - 1;
    nRowOfData  = m_wndGrid.GetLastDataRow() + 1 ;

    if( nRowToDisplay <= nRowOfData )
    {
        m_wndGrid.SetRowCount( nRowOfData );
    }
    else
    {
        m_wndGrid.SetRowCount( nRowToDisplay );
    }
}

//----------------------------------------------------------------------------

void COSGridView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);


    if (!::IsWindow(m_wndGrid.m_hWnd))
        return;

    m_wndGrid.MoveWindow( 0, 0, cx, cy , FALSE );
    m_wndGrid.UpdateWindow();

    if( IsWindow( m_hWnd ) )
        AdjustRowsToHeight() ;

    m_wndGrid.ExpandColumnsToFit();
}

//----------------------------------------------------------------------------

void COSGridView::OnInitialUpdate()
{
    CView::OnInitialUpdate();
    //---

    CRect rect;
    GetClientRect(rect);
    m_wndGrid.Create( rect, this, IDS_GRID_DEFAULT );
    m_wndGrid.SetGridBkColor( m_wndGrid.GetTextBkColor() ) ;
    m_wndGrid.SetRowResize( FALSE );
    m_wndGrid.SetColumnResize();
    m_wndGrid.SetSingleRowSelection(TRUE);
    m_wndGrid.SetListMode();
    m_wndGrid.SetEditable( FALSE );
}

//----------------------------------------------------------------------------

void COSGridView::OnDraw(CDC* pDC)    //OS
{
    if (!pDC->IsPrinting())
    {
        // If you don't want to use clipping, you may
        // use the following code instead
        //
        // Erase the background without clipping
        /*
                COLORREF rgb = RGB(255,255,255);
         
                CRect r = m_pwndGrid->GetGetGridRect();
                CRect rect;
                GetClientRect(rect);
         
                // left margin
                CRect t = rect;
                t.right = r.left;
                GXPatB(pDC, t, rgb);
         
                // right margin
                t = rect;
                t.left = r.right;
                GXPatB(pDC, t, rgb);
         
                // top margin
                t = rect;
                t.bottom = r.top;
                GXPatB(pDC, t, rgb);
         
                // bottom margin
                t = rect;
                t.top = r.bottom;
                GXPatB(pDC, t, rgb);
                */
    }
}

//----------------------------------------------------------------------------

COSGridCtrl & COSGridView::GetGridCtrl()
{
    return m_wndGrid ;
}

//----------------------------------------------------------------------------

BOOL COSGridView::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
    NM_GRIDVIEW * pNMGV ;
    CMenu   menu;
    //  CMenu   *submenu;
    BOOL    bRet;
    CPoint  point;
    //---

    pNMGV = (NM_GRIDVIEW *)lParam;

    if( pNMGV == NULL )
        return CView::OnNotify( wParam, lParam, pResult );

    if( pNMGV->hdr.hwndFrom != m_wndGrid.m_hWnd )
        return CView::OnNotify( wParam, lParam, pResult );

    if( (int)pNMGV->hdr.code != NM_RCLICK )
        return CView::OnNotify( wParam, lParam, pResult );

    bRet    = menu.CreatePopupMenu();
    //submenu = menu.GetSubMenu( 0 );

    //  if ( submenu == NULL )
    //      return CView::OnNotify( wParam, lParam, pResult );

    m_wndGrid.SetSelectedRange( pNMGV->iRow, 0,
                                pNMGV->iRow, m_wndGrid.GetColumnCount() - 1,
                                TRUE );

    DoOwnerContextMenu( &menu );

    GetCursorPos( &point );

    // Convert to screen coordinates.

    // Post the menu.
    menu.TrackPopupMenu( TPM_LEFTALIGN | TPM_RIGHTBUTTON,
                         point.x,    point.y,
                         GetParentFrame(), NULL );

    return CView::OnNotify(wParam, lParam, pResult);
}
