/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarAutoGenDlg.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "VarAutoGenDlg.h"
#include "htmlhelp.h"
#include "Help\VarAutoGenDlg.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarAutoGenDlg, CDialog)
//{{AFX_MSG_MAP(CVarAutoGenDlg)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVarAutoGenDlg dialog
//---------------------------------------------------------------------------

CVarAutoGenDlg::CVarAutoGenDlg( CVarAutoGen * pVarAutoGen, CWnd* pParent /*=NULL*/ )
        : CDialog(CVarAutoGenDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CVarAutoGenDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pVarAutoGen = pVarAutoGen ;
}

/////////////////////////////////////////////////////////////////////////////
// CVarAutoGenDlg message handlers
//---------------------------------------------------------------------------

void CVarAutoGenDlg::OnOK()
{
    long    lLowerLimit ;
    long    lUpperLimit ;
    long    lStepValue ;
    CString strPrefix ;

    CWnd *      pEdit ;
    CString     strValue ;
    //---

    pEdit = static_cast<CWnd *>( GetDlgItem( IDC_VAR_AUTO_FROM ) ) ;
    pEdit->GetWindowText( strValue ) ;

    lLowerLimit = atol( strValue ) ;

    pEdit = static_cast<CWnd *>( GetDlgItem( IDC_VAR_AUTO_TO ) ) ;
    pEdit->GetWindowText( strValue ) ;

    lUpperLimit = atol( strValue ) ;

    pEdit = static_cast<CWnd *>( GetDlgItem( IDC_VAR_AUTO_STEP ) ) ;
    pEdit->GetWindowText( strValue ) ;

    lStepValue  = atol( strValue ) ;

    if ( lUpperLimit < lLowerLimit )
    {
        CString     strMsg ;
        CString     strTitle ;
        //---
        strMsg.LoadString( IDS_BAD_AUTOGEN_PARAM ) ;
        strTitle.LoadString( IDS_ERROR ) ;

        MessageBox( strMsg, strTitle );
        return;
    }

    if ( lStepValue < 1 )
    {
        CString strMsg ;
        CString     strTitle ;
        //---
        strMsg.LoadString( IDS_BAD_AUTOGEN_STEP ) ;
        strTitle.LoadString( IDS_ERROR ) ;
        MessageBox( strMsg, strTitle );
        return;
    }

    pEdit = static_cast<CWnd *>( GetDlgItem( IDC_VAR_AUTO_PREFIX ) ) ;
    pEdit->GetWindowText( strValue ) ;

    m_pVarAutoGen->m_strPrefix      = strValue ;
    m_pVarAutoGen->m_lLowerLimit    = lLowerLimit ;
    m_pVarAutoGen->m_lUpperLimit    = lUpperLimit ;
    m_pVarAutoGen->m_lStepValue     = lStepValue ;

    CDialog::OnOK();
}

//---------------------------------------------------------------------------

BOOL CVarAutoGenDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CSpinButtonCtrl * pctlSpin ;

    pctlSpin = static_cast<CSpinButtonCtrl *>( GetDlgItem( IDC_FROM_SPIN ) ) ;
    pctlSpin->SetRange32(0,0x7fffffff);
    pctlSpin->SetPos( m_pVarAutoGen->m_lLowerLimit ) ;

    pctlSpin = static_cast<CSpinButtonCtrl *>( GetDlgItem( IDC_TO_SPIN ) ) ;
    pctlSpin->SetRange32(0,0x7fffffff);
    pctlSpin->SetPos( m_pVarAutoGen->m_lUpperLimit ) ;

    pctlSpin = static_cast<CSpinButtonCtrl *>( GetDlgItem( IDC_STEP_SPIN ) ) ;
    pctlSpin->SetRange32(0,0x7fffffff);
    pctlSpin->SetPos( m_pVarAutoGen->m_lStepValue ) ;

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


HBRUSH CVarAutoGenDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarAutoGenDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
