/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarDBPPage.cpp : implementation file
//

#include "stdafx.h"
#include "VarDBPPage.h"
#include "htmlhelp.h"
#include "Help\VarDBPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarDBPPage, CPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarDBPPage, CPropertyPage)
//{{AFX_MSG_MAP(CVarDBPPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarDBPPage::CVarDBPPage(CVarDBParam* _pVarDBParam)
        :   CPropertyPage(CVarDBPPage::IDD)
{
    m_pVarDBParam = _pVarDBParam ;
}

//---------------------------------------------------------------------------

CVarDBPPage::~CVarDBPPage()
{}

//---------------------------------------------------------------------------


BOOL CVarDBPPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog();
    UCHAR szDSN[255];
    short   pcbDSN;
    RETCODE errResult;
    HENV    hv;

    VERIFY( m_cbDSN.SubclassDlgItem( ID_DSN_COMBO, this ));

    SQLAllocEnv(&hv);

    errResult = SQLDataSources(hv, SQL_FETCH_FIRST, szDSN, sizeof(szDSN), &pcbDSN, NULL, 0, 0);
    while ( errResult != SQL_NO_DATA_FOUND && errResult != SQL_ERROR && errResult != SQL_INVALID_HANDLE )
    {
        m_cbDSN.AddBitmap(IDB_DSN, (char *)szDSN);
        errResult = SQLDataSources(hv, SQL_FETCH_NEXT, szDSN, sizeof(szDSN), &pcbDSN, NULL, 0, 0);
    }

    SQLFreeEnv(hv);

    Values2Ctrl();

    return FALSE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarDBPPage::Ctrl2Values()
{
    m_cbDSN.GetWindowText(m_pVarDBParam->m_strDSN) ;

    GetDlgItemText(ID_LOGIN_EDIT, m_pVarDBParam->m_strUserLogin) ;

    GetDlgItemText(ID_PWD_EDIT, m_pVarDBParam->m_strUserPwd) ;
}

//---------------------------------------------------------------------------

void CVarDBPPage::Values2Ctrl()
{
    CWnd* pEdit ;

    if(m_cbDSN.SetCurSel(m_cbDSN.FindStringExact(0, m_pVarDBParam->m_strDSN))
            == CB_ERR)
    {
        m_cbDSN.SetCurSel(0) ;
    }

    pEdit = static_cast<CWnd*>(GetDlgItem(ID_LOGIN_EDIT)) ;
    pEdit->SetWindowText( m_pVarDBParam->m_strUserLogin ) ;

    pEdit = static_cast<CWnd*>(GetDlgItem(ID_PWD_EDIT)) ;
    pEdit->SetWindowText( m_pVarDBParam->m_strUserPwd ) ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBPPage::OnWizardBack()
{
    return IDD_VAR_GENERAL_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBPPage::OnWizardNext()
{
    CVarDBSession * pVarDBSession;
    Ctrl2Values();

    pVarDBSession = &(m_pVarDBParam->m_VarDBSession);

    if( !pVarDBSession->Connect(m_pVarDBParam->m_strDSN,m_pVarDBParam->m_strUserLogin,m_pVarDBParam->m_strUserPwd) )
        return -1 ;

    return IDD_VAR_DBREQ_PPAGE ;
}

//---------------------------------------------------------------------------


HBRUSH CVarDBPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarDBPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp+="::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
