/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarGeneralPPage.cpp : implementation file
//

#include "stdafx.h"
#include <ExtendedComboBox.h>
#include "VarGeneralPPage.h"
#include "VarPSheet.h"
#include <ConstantesLib.h>
#include "resource.h"
#include "htmlhelp.h"
#include "Help\VarGeneralPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarGeneralPPage, CPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarGeneralPPage, CPropertyPage)
    //{{AFX_MSG_MAP(CVarGeneralPPage)
    ON_BN_CLICKED(ID_VARORD_SEQ_RADIO, OnVarOrdSeqRadio)
    ON_BN_CLICKED(ID_VARORD_RAN_RADIO, OnVarOrdRanRadio)
    ON_BN_CLICKED(ID_VARTYP_INT_RADIO, OnVarTypIntRadio)
    ON_BN_CLICKED(ID_VARTYP_CHAR_RADIO, OnVarTypCharRadio)
    ON_WM_CTLCOLOR()
    ON_WM_HELPINFO()
	ON_CBN_SELCHANGE(ID_VAR_SOURCE_COMBO, OnSelchangeVarSourceCombo)
	ON_BN_CLICKED(IDC_REP_RANDOM, OnRepRandom)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CVarGeneralPPage::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVarGeneralPPage)
	DDX_Control(pDX, IDC_SEED, m_ctlSeed);
	DDX_Control(pDX, ID_VAR_NAME_EDIT, m_ctlNameEdit);
	DDX_Control(pDX, ID_VAR_SOURCE_COMBO, m_ctlSource);
	DDX_Text(pDX, IDC_SEED, m_lSeed);
	//}}AFX_DATA_MAP
}

//---------------------------------------------------------------------------
// CVarGeneralPPage Dialog

CVarGeneralPPage::CVarGeneralPPage(CVarGeneral * _pVarGeneral, BOOL _bCreate)
:   CPropertyPage(CVarGeneralPPage::IDD)
{
    m_pVarGeneral = _pVarGeneral ;
    m_bCreateMode = _bCreate ;   
	m_lSeed = 0;
}

//---------------------------------------------------------------------------

CVarGeneralPPage::~CVarGeneralPPage()
{
}

//---------------------------------------------------------------------------

BOOL CVarGeneralPPage::OnInitDialog() 
{
    CPropertyPage::OnInitDialog();

    CExtendedComboBox*  pCombo;
    int         iIndex;

	m_ctlNameEdit.SetMaxLength(60);

    pCombo  = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SCOPE_COMBO));

    /*$$$iIndex = pCombo->AddString("Private") ;
    pCombo->SetItemData(iIndex,VAR_SCOPE_PRIVATE);*/
    
    iIndex = pCombo->AddString("Local")   ;
    pCombo->SetItemData(iIndex,VAR_SCOPE_LOCAL);

    iIndex = pCombo->AddString("Script")  ;
    pCombo->SetItemData(iIndex,VAR_SCOPE_SCRIPT);

    iIndex = pCombo->AddString("Thread")  ;
    pCombo->SetItemData(iIndex,VAR_SCOPE_THREAD);

    iIndex = pCombo->AddString("Global")  ;
    pCombo->SetItemData(iIndex,VAR_SCOPE_GLOBAL);

    //
    pCombo = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SOURCE_COMBO));
 
    iIndex = pCombo->AddString("Variable");
    pCombo->SetItemData(iIndex,VAR_SOURCE_VALUE);

	iIndex = pCombo->AddString("Value List");
    pCombo->SetItemData(iIndex,VAR_SOURCE_LIST);

    iIndex = pCombo->AddString("File")      ;
    pCombo->SetItemData(iIndex,VAR_SOURCE_FILE);

    iIndex = pCombo->AddString("Database")  ;
    pCombo->SetItemData(iIndex,VAR_SOURCE_DATABASE);

    pCombo->EnableWindow(m_bCreateMode) ;

    Values2Ctrl();

	return TRUE ;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::Values2Ctrl()
{
   CExtendedComboBox*  pCombo ;

   pCombo = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SCOPE_COMBO)) ;
   pCombo->SetCurSel(pCombo->FindItem(m_pVarGeneral->m_lVarScope)) ;

   //
   pCombo = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SOURCE_COMBO)) ;
   pCombo->SetCurSel(pCombo->FindItem(m_pVarGeneral->m_lSourceInfo)) ;

   //
   CButton*  pBut ;

   if (m_pVarGeneral->m_lVarOrder == VAR_ORDER_SEQUENTIAL ) 
   {
       pBut = static_cast<CButton*>(GetDlgItem(ID_VARORD_SEQ_RADIO)) ;
       pBut->SetCheck(1) ;
   }
   else 
   if (m_pVarGeneral->m_lVarOrder == VAR_ORDER_RANDOM ) 
   {
       pBut = static_cast<CButton*>(GetDlgItem(ID_VARORD_RAN_RADIO)) ;
       pBut->SetCheck(1) ;         
   }
   else
   {
       pBut = static_cast<CButton*>(GetDlgItem(IDC_REP_RANDOM)) ;
       pBut->SetCheck(1) ;         
		m_ctlSeed.EnableWindow();
   }

   if (m_pVarGeneral->m_lVarType == VAR_DATATYPE_CHAR) {
    
       pBut = static_cast<CButton*>(GetDlgItem(ID_VARTYP_CHAR_RADIO)) ;
       pBut->SetCheck(1) ;
   }
   else {
       pBut = static_cast<CButton*>(GetDlgItem(ID_VARTYP_INT_RADIO)) ;
       pBut->SetCheck(1) ;         
   }

   CWnd* pEdit = static_cast<CWnd*>(GetDlgItem(ID_VAR_NAME_EDIT)) ;
   pEdit->SetWindowText( m_pVarGeneral->m_strVarName ) ;

	m_lSeed = m_pVarGeneral->m_lSeed;

	UpdateData(FALSE);
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::Ctrl2Values()
{
	UpdateData();
    CExtendedComboBox*  pCombo ;

    pCombo = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SCOPE_COMBO)) ;
    m_pVarGeneral->m_lVarScope = pCombo->GetItemData(pCombo->GetCurSel()) ;

    //
    pCombo = static_cast<CExtendedComboBox*>(GetDlgItem(ID_VAR_SOURCE_COMBO)) ;
    m_pVarGeneral->m_lSourceInfo = pCombo->GetItemData(pCombo->GetCurSel()) ;

    //
    CButton*  pBut ;

    pBut = static_cast<CButton*>(GetDlgItem(ID_VARORD_SEQ_RADIO)) ;
    if (pBut->GetCheck())
          m_pVarGeneral->m_lVarOrder = VAR_ORDER_SEQUENTIAL ;
 
	pBut = static_cast<CButton*>(GetDlgItem(ID_VARORD_RAN_RADIO)) ;
    if (pBut->GetCheck())
		m_pVarGeneral->m_lVarOrder = VAR_ORDER_RANDOM ;
	
	pBut = static_cast<CButton*>(GetDlgItem(IDC_REP_RANDOM)) ;
    if (pBut->GetCheck())
		m_pVarGeneral->m_lVarOrder = VAR_ORDER_REP_RANDOM ;
    
    pBut = static_cast<CButton*>(GetDlgItem(ID_VARTYP_CHAR_RADIO)) ;
    if (pBut->GetCheck()) {
          
          m_pVarGeneral->m_lVarType = VAR_DATATYPE_CHAR ;
    }
    else {
          m_pVarGeneral->m_lVarType = VAR_DATATYPE_INT ;
    }
    
    GetDlgItemText(ID_VAR_NAME_EDIT, m_pVarGeneral->m_strVarName) ; 

	m_pVarGeneral->m_lSeed = m_lSeed;
}

//---------------------------------------------------------------------------

LRESULT CVarGeneralPPage::OnWizardBack() 
{   
    return CPropertyPage::OnWizardBack() ;
}

//---------------------------------------------------------------------------

LRESULT CVarGeneralPPage::OnWizardNext() 
{   
    if( !Verify() )
        return -1 ; //nope

    Ctrl2Values() ;

    switch( m_pVarGeneral->m_lSourceInfo )
    {  
    case VAR_SOURCE_LIST :
        return 0 ; // => next page in the pages array ;
    case VAR_SOURCE_FILE :
        return IDD_VAR_FILE_PPAGE ;
    case VAR_SOURCE_DATABASE :
        return IDD_VAR_DB_PPAGE ;
    } 

    return -1 ; //nope
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::OnVarOrdSeqRadio() 
{
    CheckRadioButton(ID_VARORD_SEQ_RADIO, ID_VARORD_RAN_RADIO, ID_VARORD_SEQ_RADIO);    
	m_ctlSeed.EnableWindow(FALSE);
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::OnVarOrdRanRadio() 
{
    CheckRadioButton(ID_VARORD_SEQ_RADIO, ID_VARORD_RAN_RADIO, ID_VARORD_RAN_RADIO);    
	m_ctlSeed.EnableWindow(FALSE);
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::OnVarTypIntRadio() 
{
    CheckRadioButton(ID_VARTYP_CHAR_RADIO, ID_VARTYP_INT_RADIO, ID_VARTYP_INT_RADIO);   
}

//---------------------------------------------------------------------------

void CVarGeneralPPage::OnVarTypCharRadio() 
{
    CheckRadioButton(ID_VARTYP_CHAR_RADIO, ID_VARTYP_INT_RADIO, ID_VARTYP_CHAR_RADIO);  
}

//---------------------------------------------------------------------------

BOOL CVarGeneralPPage::Verify() 
{
    CString strVarName ;
    
    GetDlgItemText( ID_VAR_NAME_EDIT, strVarName ) ;

    if ( strVarName.IsEmpty() )
    {
        CString     strMsg ;
        CString     strTitle ;
        //---
        strMsg.LoadString( IDS_VARNAME_EMPTY ) ;
        strTitle.LoadString( IDS_ERROR ) ;

        MessageBox( strMsg, strTitle );
        return FALSE ;
    }

    return TRUE ;
}

//---------------------------------------------------------------------------

HBRUSH CVarGeneralPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);
    
    return hbr;
}

BOOL CVarGeneralPPage::OnHelpInfo(HELPINFO* pHelpInfo) 
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    
    return true;
}

void CVarGeneralPPage::OnSelchangeVarSourceCombo() 
{
	int nSel = m_ctlSource.GetCurSel();
	if(nSel == CB_ERR)
		return;

	CVarPSheet *pSheet = (CVarPSheet*)GetParent();
	switch(m_ctlSource.GetItemData(nSel))
    {  
	case VAR_SOURCE_VALUE:
		pSheet->SetWizardButtons(PSWIZB_FINISH);
	break;
    case VAR_SOURCE_LIST :
    case VAR_SOURCE_FILE :
    case VAR_SOURCE_DATABASE :
		pSheet->SetWizardButtons(PSWIZB_NEXT);
	break;
    } 
}

void CVarGeneralPPage::OnRepRandom() 
{
	m_ctlSeed.EnableWindow();
}

BOOL CVarGeneralPPage::OnWizardFinish() 
{
    Ctrl2Values() ;
	
	return CPropertyPage::OnWizardFinish();
}

BOOL CVarGeneralPPage::OnSetActive() 
{
	OnSelchangeVarSourceCombo();
	return CPropertyPage::OnSetActive();
}
