/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarValListPPage.cpp : implementation file
//
#include "stdafx.h"
#include "VarValListPPage.h"
#include "VarAutoGenDlg.h"
#include "htmlhelp.h"
#include "Help\VarValListPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarValListPPage, CPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarValListPPage, CPropertyPage)
//{{AFX_MSG_MAP(CVarValListPPage)
ON_BN_CLICKED(IDC_AUTOGEN, OnAutoGenerate)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_NOTIFY(LVN_ENDLABELEDIT, IDC_VALLIST_LIST, OnEndlabeleditVallistList)
ON_WM_SHOWWINDOW()
ON_WM_CANCELMODE()
	ON_BN_CLICKED(IDC_VAR_GENFILE, OnVarGenfile)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarValListPPage::CVarValListPPage(CVarInfo * _pVarInfo)
        :   CPropertyPage(CVarValListPPage::IDD)
{
    //{{AFX_DATA_INIT(CVarValListPPage	)
	m_bGenFile = FALSE;
	//}}AFX_DATA_INIT
    m_pVarInfo = _pVarInfo ;
}

//---------------------------------------------------------------------------

CVarValListPPage::~CVarValListPPage()
{}

//---------------------------------------------------------------------------
void CVarValListPPage::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVarValListPPage)
	DDX_Control(pDX, IDC_VALLIST_LIST, m_ListValue);
	DDX_Check(pDX, IDC_VAR_GENFILE, m_bGenFile);
	//}}AFX_DATA_MAP
}


BOOL CVarValListPPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog() ;

    ResetGrid();

	m_ListValue.InsertColumn(0,"",LVCFMT_LEFT,500,0);


	Values2Ctrl();
	UpdateData(FALSE);
	OnVarGenfile();

    return TRUE ;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarValListPPage::Values2Ctrl()
{
    long lNbValues;

    lNbValues = m_pVarInfo->m_VarValueList.m_Values.GetSize() ;

    ResetGrid();

	int nNextItem = m_ListValue.GetItemCount();
    for(int i=0;i<lNbValues;i++)
    {
        m_ListValue.InsertItem(nNextItem++, m_pVarInfo->m_VarValueList.m_Values.GetAt(i) ) ;
    }
	m_bGenFile = m_pVarInfo->m_VarValueList.m_bGenerateFile;
}

//---------------------------------------------------------------------------

void CVarValListPPage::Ctrl2Values()
{
	UpdateData();

    long lNbItems ;

    m_pVarInfo->m_VarValueList.m_Values.RemoveAll() ;
    lNbItems = m_ListValue.GetItemCount() ;

    m_pVarInfo->m_VarValueList.m_Values.SetSize(lNbItems) ;

    for(int i=0;i<lNbItems;i++)
    {

        CString strValue ;

        strValue = m_ListValue.GetItemText(i,0) ;
        m_pVarInfo->m_VarValueList.m_Values.SetAt(i, strValue) ;

    }

    m_pVarInfo->m_VarValueList.m_bGenerateFile = m_bGenFile?true:false;

}

//---------------------------------------------------------------------------

LRESULT CVarValListPPage::OnWizardBack()
{
    return CPropertyPage::OnWizardBack() ;
}

//---------------------------------------------------------------------------

LRESULT CVarValListPPage::OnWizardNext()
{
    Ctrl2Values() ;

    return CPropertyPage::OnWizardNext() ;
}

//---------------------------------------------------------------------------

BOOL CVarValListPPage::OnSetActive()
{
    Values2Ctrl() ;
	UpdateData(FALSE);
	OnVarGenfile();

    return CPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------

void CVarValListPPage::OnAutoGenerate()
{
    CVarAutoGen     VarAutoGen ;
    CVarAutoGenDlg  Dlg( &VarAutoGen ) ;
    CString         strValue ;
    CStringArray    arstrValues ;
    long            lNbValues ;

    if( Dlg.DoModal() == IDCANCEL )
        return ;

    for( long i = VarAutoGen.m_lLowerLimit; i <= VarAutoGen.m_lUpperLimit ; i += VarAutoGen.m_lStepValue )
    {
        strValue.Format( "%s%ld", VarAutoGen.m_strPrefix, i ) ;
        arstrValues.Add( strValue ) ;
    }

    lNbValues   = arstrValues.GetSize() ;

    if( lNbValues > 0 )
    {

        m_ListValue.SetRedraw( FALSE ) ;

		int nNextItem = m_ListValue.GetItemCount();
        for( long i = 0; i < lNbValues; i++ )
        {
            m_ListValue.InsertItem(nNextItem++, arstrValues.GetAt( i ) ) ;
        }

        m_ListValue.SetRedraw() ;
    }
    m_ListValue.Invalidate();
}

//---------------------------------------------------------------------------

HBRUSH CVarValListPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

//---------------------------------------------------------------------------

void CVarValListPPage::ResetGrid()
{
    for( int i = m_ListValue.GetItemCount(); i> 0; i-- )
    {
        m_ListValue.DeleteItem( 0 );
    }
}

BOOL CVarValListPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CVarValListPPage::OnAdd() 
{
	int nNumItems = m_ListValue.GetItemCount();
	int nItem = m_ListValue.InsertItem(nNumItems," ");
	m_ListValue.EnsureVisible(nItem,FALSE);
	m_ListValue.SetFocus();
	m_ListValue.EditLabel(nItem);
}

void CVarValListPPage::OnDelete() 
{
	CArray <int,int> arSelected;

	POSITION pos = m_ListValue.GetFirstSelectedItemPosition();
	while(pos)
		arSelected.Add(m_ListValue.GetNextSelectedItem(pos));

	int nNumDeleted=0;
	for(int n=0;n<arSelected.GetSize();n++)
	{
		m_ListValue.DeleteItem(arSelected[n]-nNumDeleted);
		nNumDeleted++;
	}
}

void CVarValListPPage::OnEndlabeleditVallistList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
	// TODO: Add your control notification handler code here
	
	*pResult = 1;
}

void CVarValListPPage::OnVarGenfile() 
{
	UpdateData();
	if(m_bGenFile)
		((CPropertySheet*)GetParent())->SetWizardButtons(PSWIZB_NEXT|PSWIZB_BACK);
	else
		((CPropertySheet*)GetParent())->SetWizardButtons(PSWIZB_FINISH|PSWIZB_BACK);
}

BOOL CVarValListPPage::OnWizardFinish() 
{
	Ctrl2Values();
	return CPropertyPage::OnWizardFinish();
}
