/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// QueryEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "QueryEditDlg.h"
#include "CECSNMPQuery.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CQueryEditDlg dialog


CQueryEditDlg::CQueryEditDlg(CString strServer, CString strQuery, CWnd* pParent /*=NULL*/)
	: CDialog(CQueryEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CQueryEditDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_szQuery = strQuery;
	m_szServer = strServer;
}


void CQueryEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CQueryEditDlg)
	DDX_Control(pDX, IDC_QUERY_LIST, m_ctlWalkList);
	DDX_Control(pDX, IDC_CATEGORY, m_ctlCategory);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CQueryEditDlg, CDialog)
	//{{AFX_MSG_MAP(CQueryEditDlg)
	ON_CBN_SELCHANGE(IDC_CATEGORY, OnSelchangeCategory)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQueryEditDlg message handlers

BOOL CQueryEditDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_ctlCategory.InsertString(SYSTEM, "system");
	m_ctlCategory.InsertString(INTERFACES, "interfaces");
	m_ctlCategory.InsertString(AT, "at");
	m_ctlCategory.InsertString(IP, "ip");
	m_ctlCategory.InsertString(ICMP, "icmp");
	m_ctlCategory.InsertString(TCP, "tcp");
	m_ctlCategory.InsertString(UDP, "udp");
	m_ctlCategory.InsertString(EGP, "egp");
	m_ctlCategory.InsertString(DOT3, "dot3");
	m_ctlCategory.InsertString(SNMP, "snmp");
	m_ctlCategory.InsertString(MS_IIS, "Microsoft IIS");
	m_ctlCategory.SetCurSel(0);
	m_ctlCategory.EnableWindow();

	CRect r;
	m_ctlWalkList.GetWindowRect(r);
	m_ctlWalkList.InsertColumn(0,"Query", LVCFMT_LEFT, r.Width()/2-2);
	m_ctlWalkList.InsertColumn(1,"Current Value", LVCFMT_LEFT, r.Width()/2-2);
	m_ctlWalkList.EnableWindow();
	m_ctlWalkList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	if (m_szQuery != "")
	{
		CString strQ = m_szQuery;
		int nPos = strQ.Find(" ", 0);
		CString community = strQ.Left(nPos);
		strQ = strQ.Right(strQ.GetLength() - nPos -1);
		strQ.TrimRight();

		m_ctlCategory.EnableWindow();
		m_ctlWalkList.EnableWindow();

		int cat = StringToCat(strQ);
		m_ctlCategory.SetCurSel(cat);
		WalkTable(cat);

		for(int i =0; i<m_ctlWalkList.GetItemCount(); i++)
		{
			if (m_ctlWalkList.GetItemText(i,0) == strQ)
			{
				m_ctlWalkList.SetFocus();
				m_ctlWalkList.SetItemState(i, LVIS_SELECTED|LVIS_DROPHILITED, LVIS_SELECTED);
			}
		}
	}
	else
	{
		WalkTable(0);
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CQueryEditDlg::WalkTable(int nTable)
{
	CWaitCursor wc;
	m_ctlWalkList.DeleteAllItems();

	// Test that an SNMP server is running
	try
	{
		CECSNMPQuery query;
		QUERY_DEF def;
		def.header = m_szServer;
		def.community = "public";
		def.server = m_szServer;
		def.nTimeout = 5000;
		def.query = CatToString(nTable);

		CStringArray results;
		if(query.ExecuteWalk(def, results))
		{
			for (int i =0; i<results.GetSize(); i++)
			{
				CString strRes = results.GetAt(i);
				int nPos = strRes.Find(" =",0);
				int nLen = strRes.GetLength();
				m_ctlWalkList.InsertItem(i,strRes.Left(nPos));
				m_ctlWalkList.SetItemText(i,1,strRes.Right(nLen-nPos-3));
			}
		}
	}
	catch (...)
	{
		m_ctlCategory.EnableWindow(FALSE);
		m_ctlWalkList.EnableWindow(FALSE);
		m_ctlWalkList.DeleteAllItems();
	}
}

CString CQueryEditDlg::CatToString(int nCat)
{
	switch (nCat)
	{
		case SYSTEM:
		{
			return "system";
			break;
		}
		case INTERFACES:
		{
			return "interfaces";
			break;
		}
		case AT:
		{
			return "at";
			break;
		}
		case IP:
		{
			return "ip";
			break;
		}
		case ICMP:
		{
			return "icmp";
			break;
		}
		case TCP:
		{
			return "tcp";
			break;
		}
		case UDP:
		{
			return "udp";
			break;
		}
		case EGP:
		{
			return "egp";
			break;
		}
		case DOT3:
		{
			return "dot3";
			break;
		}
		case SNMP:
		{
			return "snmp";
			break;
		}
		case MS_IIS:
		{
			return "enterprises.microsoft.software.internetServer";
			break;
		}
	}
	return "";
}

int CQueryEditDlg::StringToCat(CString query)
{
	int nPos = query.Find("." , 0);

	if (query.Left(nPos) == "system")
	{
		return SYSTEM;
	}
	else if (query.Left(nPos) == "interfaces")
	{
		return INTERFACES;
	}
	else if (query.Left(nPos) == "at")
	{
		return AT;
	}
	else if (query.Left(nPos) == "ip")
	{
		return IP;
	}
	else if (query.Left(nPos) == "icmp")
	{
		return ICMP;
	}
	else if (query.Left(nPos) == "tcp")
	{
		return TCP;
	}
	else if (query.Left(nPos) == "udp")
	{
		return UDP;
	}
	else if (query.Left(nPos) == "egp")
	{
		return EGP;
	}
	else if (query.Left(nPos) == "dot3")
	{
		return DOT3;
	}
	else if (query.Left(nPos) == "snmp")
	{
		return SNMP;
	}
	else if (query.Left(45) == "enterprises.microsoft.software.internetServer")
	{
		return MS_IIS;
	}
	return -1;
}

void CQueryEditDlg::OnSelchangeCategory() 
{
	int nCat = m_ctlCategory.GetCurSel();
	WalkTable(nCat);	
}

void CQueryEditDlg::OnOK() 
{
	// Get the selected query
	POSITION pos = m_ctlWalkList.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		AfxMessageBox("Nothing Selected!!");
		return;
	}
	CWaitCursor wc;

	int nItem = m_ctlWalkList.GetNextSelectedItem(pos);
	CString strQuery = m_ctlWalkList.GetItemText(nItem,0);

	// Test query to see if it returns an integer value
	try
	{
		CECSNMPQuery query;
		QUERY_DEF def;
		def.header = m_szServer;
		def.community = "public";
		def.server = m_szServer;
		def.nTimeout = 5000;
		def.query = strQuery;


		if (query.ExecuteSimple(def) == -1)
		{
			return;
		}

		m_szQuery = "public "+ strQuery;
	}
	catch (...)
	{
		m_ctlCategory.EnableWindow(FALSE);
		m_ctlWalkList.EnableWindow(FALSE);
		m_ctlWalkList.DeleteAllItems();
	}
	
	CDialog::OnOK();
}

CString CQueryEditDlg::GetQuery() 
{
	return m_szQuery;
}